#include "bhand_old.h"

#include "bhand.h"

// global variables defined in BHand.h as extern
BHand* _pBH = NULL;

// A variable that is defined in bhand.cpp
extern bool _clearFlag;

///////////////////////////////////////////////////////////////////////////////
// Comaptibility commands for vesion 1.0 implementation
///////////////////////////////////////////////////////////////////////////////


// Open specified fingers to their limits
int	Open( const char motor )
{
	char str[2] = { motor, 0 };
	if( _pBH )
		return _pBH->Open(str);
	else
		return BHERR_NOTINITIALIZED;
}


// Close specified fingers to their limits
int	Close( const char motor)
{
	char str[2] = { motor, 0 };
	if( _pBH )
		return _pBH->Close(str);
	else
		return BHERR_NOTINITIALIZED;
}


// Open specified fingers by specified amount
int	StepOpen( const char motor , int value )
{
	char str[2] = { motor, 0 };
	if( _pBH )
		return _pBH->StepOpen(str,value);
	else
		return BHERR_NOTINITIALIZED;
}


// Close specified fingers by specified amount
int	StepClose( const char motor , int value )
{
	char str[2] = { motor, 0 };
	if( _pBH )
		return _pBH->StepClose(str,value);
	else
		return BHERR_NOTINITIALIZED;
}


// Move specified fingers to position
int	GoToPosition( const char motor , int value )
{
	char str[2] = { motor, 0 };
	if( _pBH )
		return _pBH->GoToPosition(str,value);
	else
		return BHERR_NOTINITIALIZED;
}


// Call GoToPosition for each motor
int	GoToDifferentPositions( int value1, int value2, int value3, int value4 )
{
	if( _pBH )
		return _pBH->GoToDifferentPositions(value1,value2,value3,value4);
	else
		return BHERR_NOTINITIALIZED;
}


// Move all fingers to home position
int	GoToHome( void )
{
	if( _pBH )
		return _pBH->GoToHome();
	else
		return BHERR_NOTINITIALIZED;
}


// Stop specified fingers
int	StopMotor( const char motor)
{
	char str[2] = { motor, 0 };
	if( _pBH )
		return _pBH->StopMotor(str);
	else
		return BHERR_NOTINITIALIZED;
}


// Set parameter for specified motors;
int	Set( const char motor, char parameter, int value )
{
	char str[2] = { motor, 0 };
	if( _pBH )
	{
		switch( parameter )
		{
		case 'C':
			return _pBH->Set(str,"MCV",value);
		case 'O':
			return _pBH->Set(str,"MOV",value);
		case 'M':
			return _pBH->Set(str,"HOLD",value);
		case 'S':
			return _pBH->Set(str,"MSG",value);
		default:
			return BHERR_BADPARAMETER;
		}
	}
	else
		return BHERR_NOTINITIALIZED;
}


// Get parameter for specified motors;
int	Get( const char motor, char parameter, int* result)
{
	char str[2] = { motor, 0 };
	if( _pBH )
	{
		switch( parameter )
		{
		case 'C':
			return _pBH->Get(str,"MCV",result);
		case 'O':
			return _pBH->Get(str,"MOV",result);
		case 'P':
			return _pBH->Get(str,"P",result);
		case 'M':
			return _pBH->Get(str,"S",result);
		case 'S':
			return _pBH->Get(str,"SG",result);
		default:
			return BHERR_BADPARAMETER;
		}
	}
	else
		return BHERR_NOTINITIALIZED;
}


// Save specified fingers to permanent storage
int	Save( const char motor)
{
	char str[2] = { motor, 0 };
	if( _pBH )
		return _pBH->Save(str);
	else
		return BHERR_NOTINITIALIZED;
}


// Load specified fingers from permanent storage
int	Load( const char motor)
{
	char str[2] = { motor, 0 };
	if( _pBH )
		return _pBH->Load(str);
	else
		return BHERR_NOTINITIALIZED;
}


// Set specified fingers to defaults
int	Default( const char motor)
{
	char str[2] = { motor, 0 };
	if( _pBH )
		return _pBH->Default(str);
	else
		return BHERR_NOTINITIALIZED;
}


// Return CPU temperature
int	Temperature( int* result)
{
	if( _pBH )
		return _pBH->Temperature(result);
	else
		return BHERR_NOTINITIALIZED;
}


// Initialize com port, send Reset to hand - called once per hand
int	InitSoftware( int comport )
{
	// range checking
	if( comport<1 || comport>BH_MAXPORT )
		return BHERR_PORTOUTOFRANGE;

	if( (!_clearFlag) && _BHandArray[comport-1] )
		return BHERR_BHANDEXISTS;
	else
	{
		_pBH = new BHand;
		return _pBH->InitSoftware(comport);
	}
}


// Initialize hand
int	InitHand( void)
{
	if( _pBH )
		return _pBH->InitHand( "GS" );
	else
		return BHERR_NOTINITIALIZED;
}


// Select default hand
int	SelectHand( int comport )
{
	// range checking
	if( comport<1 || comport>BH_MAXPORT )
		return BHERR_PORTOUTOFRANGE;

	// set default hand if present
	if( _BHandArray[comport-1] )
	{
		_pBH = _BHandArray[comport-1];
		return 0;
	}
	else
		return BHERR_NOTINITIALIZED;
}
