// Created By Will Potter
// Copyright Barrett Technology
// August 2009
// Gives the Barrett Hand AcceleGlove Control

#include "GloveLinux.h"
#include "BHand.h"
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdint.h>
#include <sys/time.h>
#include <math.h>
#include <sys/poll.h>
#include <termios.h>
#include <sys/ioctl.h>



Glove::Glove(BHand* bh, Visualizer* opengl) {
    m_hand = bh;
    m_port = -1;

    m_opengl = opengl;

    for(int i = 0; i < 18; i++) {
        data[i] = 0;
        olddata[i] = 0;
        datadif[i] = 0;
    }
}

Glove::~Glove() {

}

void Glove::PortSelect( int port) {
    m_port = port - 1;
}

bool Glove::Initialize() {
    glove_fd = OpenGloveSerial(m_port);

    if(glove_fd == -1)
	return false;
    else
	return true;
}

void Glove::RunOnce() {

    serial_port_data_to_array(glove_fd, data);
    //usleep(50000);
    //calibvalues(data);

    motor1 = motor1position(data, motor1, datadif);
    motor2 = motor2position(data, motor2, datadif);
    motor3 = motor3position(data, motor3, datadif);
	spread = spreadposition(data, spread, datadif);



	printf("Motor Positions: ");
    printf("Motor 1: %3d Motor 2: %3d Motor 3: %3d Motor 4: %3d \n", motor1, motor2, motor3, 0);

    for(int i = 0; i < 18; i++) {
        olddata[i] = data[i];
    }
}

bool Glove::PrepareRT() {

    int result = 0;
    //RTSetFlags( const char* motor, bool LCV, int LCVC, bool LCPG,
    //					   bool LFV, int LFVC, bool LFS, bool LFAP, bool LFDP, int LFDPC )
    //

    //! set flags for Real-Time according to parameters in the tab
    if( (result = m_hand->RTSetFlags( "123S", 1, 1, 1, 0, 0, 1, 0, 1, 1, 1,0,0,0,0 )) )
        return false;

  //  wxPuts(wxT("Set Flags"));

    //! start realtime mode with the selected motors
	if( (result = m_hand->RTStart( "123S" )) )
        return false;

   // wxPuts(wxT("RTStarted"));

    //! update real-time mode once
    m_hand->RTUpdate();

  //  wxPuts(wxT("Starting RT Mode with one RT update"));
 //   previous2[0] = previous2[1] = previous2[2] = previous2[3] = 0;
   // accum_error2[0] = accum_error2[1] = accum_error2[2] = accum_error2[3] = 0;
  //  t_real3.Start();

    return true;
}

bool Glove::RTPositionUpdate() {

    float dp[4];
    float ap[4];

    for(unsigned int i = 0; i < 4; i++) {
        dp[i] = m_hand->RTGetDeltaPos('1' + i);
        ap[i] = m_hand->RTGetPosition('1' + i);

        m_hand->RTSetGain('1' + i, 40);
    }

    int motors[4];

    motors[0] = motor1;
    motors[1] = motor2;
    motors[2] = motor3;
	motors[3] = spread;

    m_hand->RTSetGain('4', 40);

    int velocity;

    double prop_gain = .05;

    for(unsigned int i = 0; i < 3; i++) {

        velocity = (int)((motors[i] - ap[i]) * prop_gain);

        if(velocity > 254) {
            velocity = 254;
        } else if (velocity < -254) {
            velocity = -254;
        }

        m_hand->RTSetVelocity('1' + i, velocity);
    }

    m_hand->RTUpdate(1, 1);

    return true;
}

void Glove::AbortRun() {
    tcflush(glove_fd, TCIFLUSH);
    close(glove_fd);
	m_hand->RTAbort();
    m_hand->Command("T");
}

int Glove::OpenGloveSerial(int port) {
        char portName[] = "/dev/ttyUSB0";
		int fd = -1;

		portName[11] = '0' + port;


        struct termios tc_config;

        // Open Serial Port.
        fd = open( portName, O_RDWR|O_NOCTTY|O_NONBLOCK );
        if( fd < 0 ) {
                return -1;;
        }

        // Zero configuration.
        bzero( &tc_config, sizeof(tc_config));

        // Get existing configuration, and then modify.
        tcgetattr( fd, &tc_config );

        // 38400 baud.
        cfsetispeed( &tc_config, B38400);
        cfsetospeed( &tc_config, B38400);
        // 8 bits, 1 stop bit, no flow control
        tc_config.c_cflag &= ~(CSIZE);
        tc_config.c_cflag |= CSIZE & CS8;
        tc_config.c_cflag |= CREAD|CLOCAL;
        tc_config.c_cflag &= ~CSTOPB;

        // No Parity.
        tc_config.c_iflag = IGNPAR|IGNBRK;
        tc_config.c_oflag = 0;
        tc_config.c_lflag = 0;

        // Wait for Up to 5 tenths of a second for more data.
        tc_config.c_cc[VTIME] = 5;
        // Require a minimum of 1 byte for read to return.
        tc_config.c_cc[VMIN] = 1;

        // Flush any data on the line.
        tcflush( fd,  TCIOFLUSH );
        // Change the configuration.
        if( tcsetattr( fd, TCSANOW, &tc_config ) )
        {
          close(fd);
          return -1;
        }

        // Turn blocking back on.
        fcntl( fd, F_SETFL, 0 );
        return fd;

}

unsigned int* Glove::serial_port_data_to_array(int glove_fd, unsigned int * data) {

        ssize_t result;
        //typedef unsigned char is uint8_t
        uint8_t dataBuffer[44];
        uint8_t x;
        uint8_t offset;
        // Connect to the glove.

                // Do raw data request.
                write( glove_fd, "r", 1 );

                // Read 44 bytes in response.
                offset = 0;
                do {
                        result = read( glove_fd, dataBuffer+offset, 44-offset );
                        if( (int)-1 == result )
                        {
                                perror("Read returned with error: ");
                        }
                        else
                        {
                                if( 0 == result )
                                {
                                        printf("Glove connection lost.\n");
                                }
                                else
                                {
                                        offset += result;
                                }
                        }
                }
                while( result > 0 && offset < 44 );

                if( offset == 44 )
                {
                        for( x = 0; x < 18; x++ )
                        {

                                data[x] = (unsigned int)(((uint16_t)dataBuffer[(2*x)] << 8) | dataBuffer[(2*x)+1]);
                        }
                }

        return data;

}


int Glove::motor1position(unsigned int data[18],int motor,int datadif[18]) {
    if(datadif[5] > 10 || datadif[5] < -10) {
    if( (270 < data[15] && data[15] < 350) && (299 < data[16] && data[16] < 364) && (205 < data[17] && data [17] < 225) ) { //When palm is held relatively flat, palm down.
        if( (320 <= data[5] && data[5] <= 440) ) {
			motor = (int)(( ( (-141.54167) * (data[5]) ) + (62288.33333) ));
        } if( (data[5] < 320) ) {
			motor = 16995;
		} if( (data[5] > 440) ) {
			motor = 10;
		}
    }
    if( (290 < data[15] && data[15] < 355) && (299 < data[16] && data[16] < 359) && (data[17] > 415) ) { //When palm is held relatively flat, palm up
        if( (254 > data[5]) ) { //When Index Finger is open, or flat
			motor = 10;
        } if( (434 < data[5]) ) { //When Index Finger is closed
			motor = 17000;
        } if( (434 >= data[5] && 254 <= data[5]) ) {
			motor = ( ( (1699/18) * (data[5]) ) - (215683/9 ) );
		}
    }
    }
    return motor;
}

int Glove::motor3position(unsigned int data[18], int motor,int datadif[18]) {
    if(datadif[8] > 10 || datadif[8] < -10) {
    if( (270 < data[15] && data[15] < 350) && (299 < data[16] && data[16] < 364) && (205 < data[17] && data [17] < 225) ) {//When palm is held relatively flat, palm down.
		if( (360 <= data[8] && data[8] <= 460) ) {//When Index Finger is held "relatively flat"
			motor = (int)((((-169.85) * (data[8])) + (78141) ));
		} if( (360 > data[8]) ) {
			motor = 17000;
		} if( (460 < data[8]) ) {
			motor = 10;
		}
    }
    if( (290 < data[15] && data[15] < 355) && (299 < data[16] && data[16] < 359) && (data[17] > 415) ) { //When palm is held relatively flat, palm up
        if( ( 230 > data[8]) ) { //When Middle Finger is open, or flat
			motor = 10;
        } if( ( 230 <= data[8] && 425 >= data[8] ) ) {
			motor = ( ( (16990/185) * (data[5]) ) - (781170/37) );
		} if( ( 425 < data[8]) ) { //When Middle Finger is closed
			motor = 17000;
        }
    }
    }
    return motor;
}

int Glove::motor2position(unsigned int data[18], int motor, int datadif[18]) {
    if(datadif[11] > 10 || datadif[11] < -10) {
    if( (270 < data[15] && data[15] < 350) && (299 < data[16] && data[16] < 364) && (205 < data[17] && data [17] < 225) ) {
        if( ( 450 < data[11] ) ) {
            motor = 10;
        } if( ( 310 > data[11] ) ) {
            motor = 17000;
        } if( ( 450 >= data[11] && data[11] >= 310 ) ) {
            motor = (int)(( ( (-121.32143) * (data[11]) ) + (54604.64286) ));
        }
    }
    if( (290 < data[15] && data[15] < 355) && (299 < data[16] && data[16] < 359) && (data[17] > 415) ) {
        if( ( 430 < data[11] ) ) {
            motor = 17000;
        } if( ( 230 > data[11] ) ) {
            motor = 10;
        } if( ( 430 >= data[11] && data[11] >= 230) ) {
            motor = ( ( (1699/20) * (data[2]) ) + (39057/2) );
        }
    }
    }
    return motor;
}

int Glove::spreadposition(unsigned int data[18],int motor,int datadif[18]) {
    if(datadif[2] > 10 || datadif[2] < -10) {
    if( (270 < data[15] && data[15] < 350) && (299 < data[16] && data[16] < 364) && (205 < data[17] && data [17] < 225) ) {//When palm is held relatively flat, palm down.
        if( ( 410 < data[2] ) ) {//When Thumb is held "relatively flat"
			motor = 10;
        } if( ( 410 >= data[2] && 350 <= data[2]) ) {
			motor = ( ( (-312/10) * (data[2]) ) + 13094 );
        } if( ( data[2] < 350 ) ) {//For when the Thumb is closed
			motor = 3145;
        }
    }
    if( (290 < data[15] && data[15] < 355) && (299 < data[16] && data[16] < 359) && (data[17] > 415) ) { //When palm is held relatively flat, palm up
        if( ( data[2] < 255 ) ) { //When Thumb is open, or flat
			motor = 10;
        } if( ( 365 < data[2]  ) ) { //When Thumb is closed
			motor = 3130;
        } if( ( 365 >= data[2] && 255 <= data[2] ) ) {
			motor = ( ( (312/11) * (data[2]) ) + (79450/11) );
		}
    }

    if(motor<=100)
    {motor=10;}
    if(motor>=3000)
    {motor=3145;}
    }
    return motor;
}
