// Created By Will Potter
// Copyright Barrett Technology
// August 2009
// Gives the Barrett Hand AcceleGlove Control

#include "GloveWindows.h"
#include "BHand.h"
#include "Visual_Tab/Visualizer.h"
#include <stdio.h>
#include <string.h>
//#include <unistd.h> // TODO: find why this was used before
#include <errno.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
//#include <stdint.h> // TODO: find why this was used before
//#include <sys/time.h>
#include <math.h>
//#include <windows.h>
#include <Windows.h>
#include <WinBase.h>

#define READ_TIMEOUT      500

#include "Common/tmr.h"

HiResTimer t_real3;

int previous2[4];
float accum_error2[4];

Glove::Glove(BHand* bh, Visualizer* opengl) {
    m_hand = bh;
    m_opengl = opengl;
    m_port = -1;
    motor1 = 0;
    motor2 = 0;
    motor3 = 0;
    spread = 0;
    for(unsigned int i = 0; i < 18; i++) {
        data[i] = 0;
        olddata[i] = 0;
        datadif[i] = 0;
    }
}

Glove::~Glove() {

}

void Glove::PortSelect( int port) {
    m_port = port;
}

bool Glove::PrepareRT() {

    int result = 0;
    //RTSetFlags( const char* motor, bool LCV, int LCVC, bool LCPG,
    //					   bool LFV, int LFVC, bool LFS, bool LFAP, bool LFDP, int LFDPC )
    //

    //! set flags for Real-Time according to parameters in the tab
    if ((result = m_hand->RTSetFlags( "123S", 1, 1, 1, 0, 0, 1, 0, 1, 1, 1,0,0,0,0 )))
        return false;

    wxPuts(wxT("Set Flags"));

    //! start realtime mode with the selected motors
	if ((result = m_hand->RTStart( "123S" )))
        return false;

    wxPuts(wxT("RTStarted"));

    //! update real-time mode once
    m_hand->RTUpdate();

    wxPuts(wxT("Starting RT Mode with one RT update"));
    previous2[0] = previous2[1] = previous2[2] = previous2[3] = 0;
    accum_error2[0] = accum_error2[1] = accum_error2[2] = accum_error2[3] = 0;
    t_real3.Start();

    return true;
}

bool Glove::Initialize() {
    hSerial = OpenGloveSerial(m_port);

    if( hSerial == INVALID_HANDLE_VALUE ) {
        return false;
    } else {
        return true;
    }
}

void Glove::RunOnce() {

    serial_port_data_to_array(hSerial, data);
    //usleep(50000);
    for(int m=0;m<18;m++) {
        datadif[m] = data[m] - olddata[m];
    }
    motor1 = motor1position(data, motor1, datadif);
    motor2 = motor2position(data, motor2, datadif);
    motor3 = motor3position(data, motor3, datadif);
    spread = spreadposition(data, spread, datadif);
    RTPositionUpdate();
    for(int z=0;z<18;z++) {
        olddata[z] = data[z];
    }
}

bool Glove::RTPositionUpdate() {

    float dp[4];
    float ap[4];

    for(unsigned int i = 0; i < 4; i++) {
        dp[i] = m_hand->RTGetDeltaPos('1' + i);
        ap[i] = m_hand->RTGetPosition('1' + i);

        m_hand->RTSetGain('1' + i, 40);
    }

    int motors[4];

    motors[0] = motor1;
    motors[1] = motor2;
    motors[2] = motor3;
	motors[3] = spread;

    m_hand->RTSetGain('4', 40);

    int velocity;

    float prop_gain, int_gain, deriv_gain = 0;

    prop_gain = 0.05f;
    int_gain = 0.00005f;
    deriv_gain = 0.01f;

    t_real3.Stop();
    float time_diff = t_real3.GetDurationInSecs() * 1000.0;

    for(unsigned int i = 0; i < 4; i++) {

        velocity = (int)((motors[i] - ap[i]) * prop_gain);

        velocity = (int)(velocity + accum_error2[i] * int_gain);

        velocity = (int)(velocity + ( (float)((motors[i] - previous2[i]) - dp[i]) / time_diff) * deriv_gain);

        if(velocity > 254) {
            velocity = 254;
        } else if (velocity < -254) {
            velocity = -254;
        }

        if(i == 3) {
            velocity /= 2;
        }

        m_hand->RTSetVelocity('1' + i, velocity);

        accum_error2[i] += (float)(motors[i] - ap[i]) * time_diff;
    }

    m_hand->RTUpdate(1, 1);

    float f1 = ((ap[0]-10.0) /16985.0) * 140;
    float f2 = ((ap[1]-10.0) /16985.0) * 140;
    float f3 = ((ap[2]-10.0) /16985.0) * 140;
    float spread = ((ap[3] - 20) / 3140) * 180;

    m_opengl->RefreshOpenGL(f1,f2,f3,spread);

    t_real3.Start();

    for(unsigned int i = 0; i < 4; i++)
        previous2[i] = motors[i];

    return true;

}

void Glove::AbortRun() {
    CloseHandle(hSerial);
    m_hand->RTAbort();
    m_hand->Command("T");
}

HANDLE Glove::OpenGloveSerial(int port) {

    char portName[] = "COM0";

    if (port >= 1 && port <= 6)
        portName[3] += port;
	else if (port == -1)
        return INVALID_HANDLE_VALUE;

    HANDLE hSerial;
    hSerial = CreateFile(portName, GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
    if(hSerial==INVALID_HANDLE_VALUE){
        if(GetLastError()==ERROR_FILE_NOT_FOUND){
        printf("Serial Port Does Not Exist.");
        }
        printf("Other Error");
    }
    DCB dcbSerialParams = {0};
    dcbSerialParams.DCBlength=sizeof(dcbSerialParams);
    if (!GetCommState(hSerial, &dcbSerialParams)) {
        printf("Failed to get Comm State");
    }
    dcbSerialParams.BaudRate=CBR_38400;
    dcbSerialParams.ByteSize=8;
    dcbSerialParams.StopBits=ONESTOPBIT;
    dcbSerialParams.Parity=NOPARITY;
    dcbSerialParams.fOutxCtsFlow =0;
    if(!SetCommState(hSerial, &dcbSerialParams)){
        printf("Failed to Set Serial Port");
    }
    COMMTIMEOUTS timeouts={0};
    timeouts.ReadIntervalTimeout=50;
    timeouts.ReadTotalTimeoutConstant=50;
    timeouts.ReadTotalTimeoutMultiplier=10;
    timeouts.WriteTotalTimeoutConstant=50;
    timeouts.WriteTotalTimeoutMultiplier=10;
    if(!SetCommTimeouts(hSerial, &timeouts)){
        printf("Timeouts could not be set.");
	}

	return hSerial;
}

unsigned int* Glove::serial_port_data_to_array(HANDLE hSerial, unsigned int * data) {
	// TODO: this will need to be put back in to work
/*	char* query = "r";
	BOOL result;
	int x;
	uint8_t offset;
	uint8_t dataBuffer[44];
	DWORD Written;
	DWORD Read;


	// Connect to the glove.
	// Do raw data request.
	WriteFile( hSerial, query, 1 , &Written, NULL);
	offset = 0;
	// Read 44 bytes in response.
	do {
		result = ReadFile(hSerial, dataBuffer, 44, &Read, NULL );

		if( 0 == result ){
			printf("Glove connection lost.\n");
			break;
		} else {
			offset += Read;
		}
	}
	while( Read > 0 && offset < 44 );

	if( offset == 44 ) {
		for( x = 0; x < 18; x++ ) {
			//data[x] = (unsigned int)(((int)dataBuffer[(2*x)] << 8) | dataBuffer[(2*x)+1]);
			data[x] = (unsigned int)(((uint16_t)dataBuffer[(2*x)] << 8) | dataBuffer[(2*x)+1]);
		}
	}
*/
	return data;

}

int Glove::motor1position(unsigned int data[18],int motor,int datadif[18]) {
    if(datadif[5] > 10 || datadif[5] < -10) {
    if( (270 < data[15] && data[15] < 350) && (299 < data[16] && data[16] < 364) && (205 < data[17] && data [17] < 225) ) { //When palm is held relatively flat, palm down.
        if( (320 <= data[5] && data[5] <= 440) ) {
			motor = (int)( ( (-141.54167) * (data[5]) ) + (62288.33333) );
        } if( (data[5] < 320) ) {
			motor = 16995;
		} if( (data[5] > 440) ) {
			motor = 10;
		}
    }
    if( (290 < data[15] && data[15] < 355) && (299 < data[16] && data[16] < 359) && (data[17] > 415) ) { //When palm is held relatively flat, palm up
        if( (254 > data[5]) ) { //When Index Finger is open, or flat
			motor = 10;
        } if( (434 < data[5]) ) { //When Index Finger is closed
			motor = 17000;
        } if( (434 >= data[5] && 254 <= data[5]) ) {
			motor = ( ( (1699/18) * (data[5]) ) - (215683/9 ) );
		}
    }
    }
    return motor;
}

int Glove::motor3position(unsigned int data[18], int motor,int datadif[18]) {
    if(datadif[8] > 10 || datadif[8] < -10) {
    if( (270 < data[15] && data[15] < 350) && (299 < data[16] && data[16] < 364) && (205 < data[17] && data [17] < 225) ) {//When palm is held relatively flat, palm down.
		if( (360 <= data[8] && data[8] <= 460) ) {//When Index Finger is held "relatively flat"
			motor = (int)(((-169.85) * (data[8])) + (78141) );
		} if( (360 > data[8]) ) {
			motor = 17000;
		} if( (460 < data[8]) ) {
			motor = 10;
		}
    }
    if( (290 < data[15] && data[15] < 355) && (299 < data[16] && data[16] < 359) && (data[17] > 415) ) { //When palm is held relatively flat, palm up
        if( ( 230 > data[8]) ) { //When Middle Finger is open, or flat
			motor = 10;
        } if( ( 230 <= data[8] && 425 >= data[8] ) ) {
			motor = ( ( (16990/185) * (data[5]) ) - (781170/37) );
		} if( ( 425 < data[8]) ) { //When Middle Finger is closed
			motor = 17000;
        }
    }
    }
    return motor;
}

int Glove::motor2position(unsigned int data[18], int motor, int datadif[18]) {
    if(datadif[11] > 10 || datadif[11] < -10) {
    if( (270 < data[15] && data[15] < 350) && (299 < data[16] && data[16] < 364) && (205 < data[17] && data [17] < 225) ) {
        if( ( 450 < data[11] ) ) {
            motor = 10;
        } if( ( 310 > data[11] ) ) {
            motor = 17000;
        } if( ( 450 >= data[11] && data[11] >= 310 ) ) {
            motor = (int)( ( (-121.32143) * (data[11]) ) + (54604.64286) );
        }
    }
    if( (290 < data[15] && data[15] < 355) && (299 < data[16] && data[16] < 359) && (data[17] > 415) ) {
        if( ( 430 < data[11] ) ) {
            motor = 17000;
        } if( ( 230 > data[11] ) ) {
            motor = 10;
        } if( ( 430 >= data[11] && data[11] >= 230) ) {
            motor = ( ( (1699/20) * (data[2]) ) + (39057/2) );
        }
    }
    }
    return motor;
}

int Glove::spreadposition(unsigned int data[18],int motor,int datadif[18]) {
    if(datadif[2] > 10 || datadif[2] < -10) {
    if( (270 < data[15] && data[15] < 350) && (299 < data[16] && data[16] < 364) && (205 < data[17] && data [17] < 225) ) {//When palm is held relatively flat, palm down.
        if( ( 410 < data[2] ) ) {//When Thumb is held "relatively flat"
			motor = 10;
        } if( ( 410 >= data[2] && 350 <= data[2]) ) {
			motor = ( ( (-312/10) * (data[2]) ) + 13094 );
        } if( ( data[2] < 350 ) ) {//For when the Thumb is closed
			motor = 3145;
        }
    }
    if( (290 < data[15] && data[15] < 355) && (299 < data[16] && data[16] < 359) && (data[17] > 415) ) { //When palm is held relatively flat, palm up
        if( ( data[2] < 255 ) ) { //When Thumb is open, or flat
			motor = 10;
        } if( ( 365 < data[2]  ) ) { //When Thumb is closed
			motor = 3130;
        } if( ( 365 >= data[2] && 255 <= data[2] ) ) {
			motor = ( ( (312/11) * (data[2]) ) + (79450/11) );
		}
    }

    if(motor<=100)
    {motor=10;}
    if(motor>=3000)
    {motor=3145;}
    }
    return motor;
}
