/******************************************************************************
 * Name:      BHControlMain.cpp
 * Purpose:   Contains high-level code for creating tabs in the GUI
 * Authors:   Kevin Doo and Edward Hannigan
 * Created:   2009-05-26
 * Copyright: Barrett Technology 2010
 *****************************************************************************/

#include "BHControlMain.h"


//(*InternalHeaders(BHControlFrame)
#include <wx/artprov.h>
#include <wx/bitmap.h>
#include <wx/intl.h>
#include <wx/image.h>
#include <wx/string.h>
//*)

#include <wx/filedlg.h>
#include <wx/msgdlg.h>


#include "BHand.h"

#include "Acceleglove_Tab/Glove.h"
#include "Common/mathplot.h"
#include "Common/tmr.h"
#include "Config_Tab/Config.h"
#include "RealTime_Tab/RealPanel.h"
#include "RealTime_Tab/RealPlotPanel.h"
#include "Super_Tab/SuperPanel.h"
#include "Visual_Tab/Visualizer.h"
#include "Visual_Tab/VisualPanel.h"

#ifndef LINUX
//#include <crtdbg.h>
#endif

//#define CREATE_CONSOLE

#define HAND_ATTACHED true
//#define HAND_ATTACHED false

wxString bhversion(wxT("Version 4.4.3  Build Date: Dec 30, 2010"));


// Allocated space for the pointer to the BHControl frame
BHControlFrame *BHControlFrame::bhcontrol;

bool use280Config = true;

const char fileSettingsName[] = "bhsettings.ini";

//(*IdInit(BHControlFrame)
const long BHControlFrame::ID_CONFIG_Upload_StaticBox = wxNewId();
const long BHControlFrame::ID_CONFIG_CurrentPath_StaticBox = wxNewId();
const long BHControlFrame::ID_CONFIG_Configuration_StaticBox = wxNewId();
const long BHControlFrame::ID_CONFIG_AutoInit_CheckBox = wxNewId();
const long BHControlFrame::ID_CONFIG_ComPort_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_BaudRate_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_Com_Choice = wxNewId();
const long BHControlFrame::ID_CONFIG_BaudRate_Choice = wxNewId();
const long BHControlFrame::ID_CONFIG_TestDelays_StaticBox = wxNewId();
const long BHControlFrame::ID_CONFIG_Motors_StaticBox = wxNewId();
const long BHControlFrame::ID_CONFIG_Feedback_StaticBox = wxNewId();
const long BHControlFrame::ID_CONFIG_Control_StaticBox = wxNewId();
const long BHControlFrame::ID_CONFIG_Results_StaticBoxID_ = wxNewId();
const long BHControlFrame::ID_CONFIG_DelayHistogram_StaticBox = wxNewId();
const long BHControlFrame::ID_CONFIG_Motor1_CheckBox = wxNewId();
const long BHControlFrame::ID_CONFIG_Motor3_CheckBox = wxNewId();
const long BHControlFrame::ID_CONFIG_Motor2_CheckBox = wxNewId();
const long BHControlFrame::ID_CONFIG_Motor4_CheckBox = wxNewId();
const long BHControlFrame::ID_CONFIG_VelocityFeedback_CheckBox = wxNewId();
const long BHControlFrame::ID_CONFIG_StrainFeedback_CheckBox = wxNewId();
const long BHControlFrame::ID_CONFIG_PosFeedback_CheckBox = wxNewId();
const long BHControlFrame::ID_CONFIG_DpFeedback_CheckBox = wxNewId();
const long BHControlFrame::ID_CONFIG_VelocityControl_CheckBox = wxNewId();
const long BHControlFrame::ID_CONFIG_Velocity_TextCtrl = wxNewId();
const long BHControlFrame::ID_CONFIG_Max_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_Min_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_Avg_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_OutIn_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_AvgValue_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_OutInValue_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_MaxValue_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_MinValue_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_CurrentPath_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_FilePath_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_Browse_Button = wxNewId();
const long BHControlFrame::ID_CONFIG_CurrentFile_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_CurrentFile_StaticBox = wxNewId();
const long BHControlFrame::ID_CONFIG_FileName_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_Initialize_Button = wxNewId();
const long BHControlFrame::ID_CONFIG_Reset_Button = wxNewId();
const long BHControlFrame::ID_CONFIG_Upload_Button = wxNewId();
const long BHControlFrame::ID_CONFIG_TestDelays_Button = wxNewId();
const long BHControlFrame::ID_CONFIG_FirmwareOnHand_StaticBox = wxNewId();
const long BHControlFrame::ID_CONFIG_FirmwareVersion_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_FirmwareOnHand_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_Upload_Gauge = wxNewId();
const long BHControlFrame::ID_CONFIG_UploadMessage_StaticText = wxNewId();
const long BHControlFrame::ID_CONFIG_PropGainControl_CheckBox = wxNewId();
const long BHControlFrame::ID_CONFIG_Serial_RadioButton = wxNewId();
const long BHControlFrame::ID_CONFIG_CAN_RadioButton = wxNewId();
const long BHControlFrame::ID_BOOTLOADER_CONFIGURATION_STATICTEXT = wxNewId();
const long BHControlFrame::ID_CONFIG_Hands_Choice = wxNewId();
const long BHControlFrame::ID_MODEL_NUM_STATICTEXT = wxNewId();
const long BHControlFrame::ID_Configure_Tab = wxNewId();
const long BHControlFrame::ID_BHControl_NoteBook = wxNewId();
const long BHControlFrame::ID_BHCONTROL_VERSION_STATICTEXT = wxNewId();
const long BHControlFrame::ID_MainPanel = wxNewId();
const long BHControlFrame::ID_TIMER1 = wxNewId();
//*)

BEGIN_EVENT_TABLE(BHControlFrame, wxFrame)
	EVT_IDLE(BHControlFrame::OnIdle)
	EVT_ERASE_BACKGROUND(BHControlFrame::OnErase)
	EVT_CLOSE(BHControlFrame::OnClose)
END_EVENT_TABLE()


void BHControlFrame::OnErase(wxEraseEvent& event)
{
	BHControlVersionStaticText->Refresh();
}

#ifndef LINUX
#ifdef CREATE_CONSOLE
void createConsole()
{
	int hConHandle;
	long lStdHandle;


	FILE *fp;

	// allocate a console for this app
	AllocConsole();

	// set the screen buffer to be big enough to let us scroll text
	CONSOLE_SCREEN_BUFFER_INFO coninfo;
	GetConsoleScreenBufferInfo(GetStdHandle(STD_OUTPUT_HANDLE), &coninfo);
	coninfo.dwSize.Y = 100; //MAX_CONSOLE_LINES;
	SetConsoleScreenBufferSize(GetStdHandle(STD_OUTPUT_HANDLE), coninfo.dwSize);

	// redirect unbuffered STDOUT to the console
	lStdHandle = (long)GetStdHandle(STD_OUTPUT_HANDLE);
	hConHandle = _open_osfhandle(lStdHandle, _O_TEXT);

	fp = _fdopen( hConHandle, "w" );

	*stdout = *fp;

	setvbuf( stdout, NULL, _IONBF, 0 );

	// make cout, wcout, cin, wcin, wcerr, cerr, wclog and clog
	// point to console as well
	//ios::sync_with_stdio();
}
#endif
#endif

BHControlFrame::BHControlFrame(wxWindow* parent)
{
	//(*Initialize(BHControlFrame)
	Create(parent, wxID_ANY, _("BHControl"), wxDefaultPosition, wxDefaultSize, wxCAPTION|wxSYSTEM_MENU|wxCLOSE_BOX|wxMINIMIZE_BOX|wxCLIP_CHILDREN, _T("wxID_ANY"));
	SetClientSize(wxSize(794,700));
	{
	wxIcon FrameIcon;
	FrameIcon.CopyFromBitmap(wxArtProvider::GetBitmap(wxART_MAKE_ART_ID_FROM_STR(_T("aaaa")),wxART_FRAME_ICON));
	SetIcon(FrameIcon);
	}
	MainPanel = new wxPanel(this, ID_MainPanel, wxPoint(300,-10), wxDefaultSize, wxTAB_TRAVERSAL, _T("ID_MainPanel"));
	BHControl_NoteBook = new wxNotebook(MainPanel, ID_BHControl_NoteBook, wxPoint(3,4), wxSize(788,695), 0, _T("ID_BHControl_NoteBook"));
	Configure_Tab = new wxPanel(BHControl_NoteBook, ID_Configure_Tab, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL, _T("ID_Configure_Tab"));
	CONFIG_Upload_StaticBox = new wxStaticBox(Configure_Tab, ID_CONFIG_Upload_StaticBox, _("Upload Firmware to Hand"), wxPoint(261,4), wxSize(510,252), 0, _T("ID_CONFIG_Upload_StaticBox"));
	CONFIG_CurrentPath_StaticBox = new wxStaticBox(Configure_Tab, ID_CONFIG_CurrentPath_StaticBox, wxEmptyString, wxPoint(388,46), wxSize(368,38), 0, _T("ID_CONFIG_CurrentPath_StaticBox"));
	CONFIG_Configuration_StaticBox = new wxStaticBox(Configure_Tab, ID_CONFIG_Configuration_StaticBox, _("Configuration"), wxPoint(17,4), wxSize(230,252), 0, _T("ID_CONFIG_Configuration_StaticBox"));
	CONFIG_AutoInit_CheckBox = new wxCheckBox(Configure_Tab, ID_CONFIG_AutoInit_CheckBox, _("Auto-Init Hand"), wxPoint(72,88), wxDefaultSize, 0, wxDefaultValidator, _T("ID_CONFIG_AutoInit_CheckBox"));
	CONFIG_AutoInit_CheckBox->SetValue(false);
	CONFIG_ComPort_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_ComPort_StaticText, _("Com Port:"), wxPoint(60,120), wxDefaultSize, 0, _T("ID_CONFIG_ComPort_StaticText"));
	CONFIG_ComPort_StaticText->Hide();
	CONFIG_BaudRate_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_BaudRate_StaticText, _("Baud Rate:"), wxPoint(60,148), wxDefaultSize, 0, _T("ID_CONFIG_BaudRate_StaticText"));
	CONFIG_Com_Choice = new wxChoice(Configure_Tab, ID_CONFIG_Com_Choice, wxPoint(140,116), wxSize(80,30), 0, 0, 0, wxDefaultValidator, _T("ID_CONFIG_Com_Choice"));
	CONFIG_Com_Choice->SetSelection( CONFIG_Com_Choice->Append(_("Com 1")) );
	CONFIG_Com_Choice->Append(_("Com 2"));
	CONFIG_Com_Choice->Append(_("Com 3"));
	CONFIG_Com_Choice->Append(_("Com 4"));
	CONFIG_Com_Choice->Append(_("Com 5"));
	CONFIG_Com_Choice->Append(_("Com 6"));
	CONFIG_Com_Choice->Append(_("Com 7"));
	CONFIG_Com_Choice->Append(_("Com 8"));
	CONFIG_Com_Choice->Append(_("Com 9"));
	CONFIG_Com_Choice->Hide();
	CONFIG_BaudRate_Choice = new wxChoice(Configure_Tab, ID_CONFIG_BaudRate_Choice, wxPoint(140,144), wxSize(80,30), 0, 0, 0, wxDefaultValidator, _T("ID_CONFIG_BaudRate_Choice"));
	CONFIG_BaudRate_Choice->SetSelection( CONFIG_BaudRate_Choice->Append(_("9600")) );
	CONFIG_BaudRate_Choice->Append(_("19200"));
	CONFIG_BaudRate_Choice->Append(_("38400"));
	CONFIG_TestDelays_StaticBox = new wxStaticBox(Configure_Tab, ID_CONFIG_TestDelays_StaticBox, _("Test Delays"), wxPoint(20,268), wxSize(751,376), 0, _T("ID_CONFIG_TestDelays_StaticBox"));
	CONFIG_Motors_StaticBox = new wxStaticBox(Configure_Tab, ID_CONFIG_Motors_StaticBox, _("Motors"), wxPoint(36,292), wxSize(99,140), 0, _T("ID_CONFIG_Motors_StaticBox"));
	CONFIG_Feedback_StaticBox = new wxStaticBox(Configure_Tab, ID_CONFIG_Feedback_StaticBox, _("Feedback"), wxPoint(36,436), wxSize(227,64), 0, _T("ID_CONFIG_Feedback_StaticBox"));
	CONFIG_Control_StaticBox = new wxStaticBox(Configure_Tab, ID_CONFIG_Control_StaticBox, _("Control"), wxPoint(145,292), wxSize(118,140), 0, _T("ID_CONFIG_Control_StaticBox"));
	CONFIG_Results_StaticBox = new wxStaticBox(Configure_Tab, ID_CONFIG_Results_StaticBoxID_, _("Results (ms)"), wxPoint(36,552), wxSize(228,80), 0, _T("ID_CONFIG_Results_StaticBoxID_"));
	CONFIG_DelayHistogram_StaticBox = new wxStaticBox(Configure_Tab, ID_CONFIG_DelayHistogram_StaticBox, _("Delay Histogram"), wxPoint(280,292), wxSize(474,340), 0, _T("ID_CONFIG_DelayHistogram_StaticBox"));
	CONFIG_Motor1_CheckBox = new wxCheckBox(Configure_Tab, ID_CONFIG_Motor1_CheckBox, _("One"), wxPoint(48,320), wxDefaultSize, 0, wxDefaultValidator, _T("ID_CONFIG_Motor1_CheckBox"));
	CONFIG_Motor1_CheckBox->SetValue(false);
	CONFIG_Motor3_CheckBox = new wxCheckBox(Configure_Tab, ID_CONFIG_Motor3_CheckBox, _("Three"), wxPoint(48,376), wxDefaultSize, 0, wxDefaultValidator, _T("ID_CONFIG_Motor3_CheckBox"));
	CONFIG_Motor3_CheckBox->SetValue(false);
	CONFIG_Motor2_CheckBox = new wxCheckBox(Configure_Tab, ID_CONFIG_Motor2_CheckBox, _("Two"), wxPoint(48,348), wxDefaultSize, 0, wxDefaultValidator, _T("ID_CONFIG_Motor2_CheckBox"));
	CONFIG_Motor2_CheckBox->SetValue(false);
	CONFIG_Motor4_CheckBox = new wxCheckBox(Configure_Tab, ID_CONFIG_Motor4_CheckBox, _("Spread"), wxPoint(48,404), wxDefaultSize, 0, wxDefaultValidator, _T("ID_CONFIG_Motor4_CheckBox"));
	CONFIG_Motor4_CheckBox->SetValue(false);
	CONFIG_VelocityFeedback_CheckBox = new wxCheckBox(Configure_Tab, ID_CONFIG_VelocityFeedback_CheckBox, _("Velocity"), wxPoint(48,456), wxDefaultSize, 0, wxDefaultValidator, _T("ID_CONFIG_VelocityFeedback_CheckBox"));
	CONFIG_VelocityFeedback_CheckBox->SetValue(false);
	CONFIG_StrainFeedback_CheckBox = new wxCheckBox(Configure_Tab, ID_CONFIG_StrainFeedback_CheckBox, _("Strain"), wxPoint(152,456), wxSize(-1,-1), 0, wxDefaultValidator, _T("ID_CONFIG_StrainFeedback_CheckBox"));
	CONFIG_StrainFeedback_CheckBox->SetValue(false);
	CONFIG_PosFeedback_CheckBox = new wxCheckBox(Configure_Tab, ID_CONFIG_PosFeedback_CheckBox, _("Position"), wxPoint(48,476), wxDefaultSize, 0, wxDefaultValidator, _T("ID_CONFIG_PosFeedback_CheckBox"));
	CONFIG_PosFeedback_CheckBox->SetValue(false);
	CONFIG_DpFeedback_CheckBox = new wxCheckBox(Configure_Tab, ID_CONFIG_DpFeedback_CheckBox, _("Delta Pos"), wxPoint(152,476), wxDefaultSize, 0, wxDefaultValidator, _T("ID_CONFIG_DpFeedback_CheckBox"));
	CONFIG_DpFeedback_CheckBox->SetValue(false);
	CONFIG_VelocityControl_CheckBox = new wxCheckBox(Configure_Tab, ID_CONFIG_VelocityControl_CheckBox, _("Velocity"), wxPoint(156,316), wxDefaultSize, 0, wxDefaultValidator, _T("ID_CONFIG_VelocityControl_CheckBox"));
	CONFIG_VelocityControl_CheckBox->SetValue(false);
	CONFIG_Velocity_TextCtrl = new wxTextCtrl(Configure_Tab, ID_CONFIG_Velocity_TextCtrl, _("0"), wxPoint(168,344), wxSize(59,21), 0, wxDefaultValidator, _T("ID_CONFIG_Velocity_TextCtrl"));
	CONFIG_Velocity_TextCtrl->Disable();
	CONFIG_Max_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_Max_StaticText, _("Max:"), wxPoint(48,600), wxDefaultSize, 0, _T("ID_CONFIG_Max_StaticText"));
	CONFIG_Min_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_Min_StaticText, _("Min:"), wxPoint(48,576), wxDefaultSize, 0, _T("ID_CONFIG_Min_StaticText"));
	CONFIG_Avg_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_Avg_StaticText, _("Avg.:"), wxPoint(152,576), wxDefaultSize, 0, _T("ID_CONFIG_Avg_StaticText"));
	CONFIG_OutIn_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_OutIn_StaticText, _("Out/In:"), wxPoint(152,600), wxDefaultSize, 0, _T("ID_CONFIG_OutIn_StaticText"));
	CONFIG_OutIn_StaticText->Hide();
	CONFIG_AvgValue_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_AvgValue_StaticText, _("n/a"), wxPoint(208,576), wxDefaultSize, 0, _T("ID_CONFIG_AvgValue_StaticText"));
	CONFIG_OutInValue_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_OutInValue_StaticText, _("n/a"), wxPoint(208,600), wxDefaultSize, 0, _T("ID_CONFIG_OutInValue_StaticText"));
	CONFIG_OutInValue_StaticText->Hide();
	CONFIG_MaxValue_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_MaxValue_StaticText, _("n/a"), wxPoint(96,600), wxDefaultSize, 0, _T("ID_CONFIG_MaxValue_StaticText"));
	CONFIG_MinValue_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_MinValue_StaticText, _("n/a"), wxPoint(96,576), wxDefaultSize, 0, _T("ID_CONFIG_MinValue_StaticText"));
	CONFIG_CurrentPath_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_CurrentPath_StaticText, _("Current Path:"), wxPoint(280,60), wxDefaultSize, 0, _T("ID_CONFIG_CurrentPath_StaticText"));
	CONFIG_FilePath_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_FilePath_StaticText, _("Browse for firmware file..."), wxPoint(396,60), wxSize(351,20), 0, _T("ID_CONFIG_FilePath_StaticText"));
	CONFIG_FilePath_StaticText->SetMaxSize(wxSize(396,13));
	CONFIG_Browse_Button = new wxButton(Configure_Tab, ID_CONFIG_Browse_Button, _("Browse"), wxPoint(680,96), wxSize(80,28), 0, wxDefaultValidator, _T("ID_CONFIG_Browse_Button"));
	CONFIG_CurrentFile_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_CurrentFile_StaticText, _("Current File:"), wxPoint(280,100), wxDefaultSize, 0, _T("ID_CONFIG_CurrentFile_StaticText"));
	CONFIG_CurrentFile_StaticBox = new wxStaticBox(Configure_Tab, ID_CONFIG_CurrentFile_StaticBox, wxEmptyString, wxPoint(388,86), wxSize(279,38), 0, _T("ID_CONFIG_CurrentFile_StaticBox"));
	CONFIG_FileName_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_FileName_StaticText, _("n/a"), wxPoint(396,100), wxSize(259,20), 0, _T("ID_CONFIG_FileName_StaticText"));
	CONFIG_FileName_StaticText->SetMinSize(wxSize(259,20));
	CONFIG_FileName_StaticText->SetMaxSize(wxSize(259,13));
	CONFIG_Initialize_Button = new wxButton(Configure_Tab, ID_CONFIG_Initialize_Button, _("Initialize Library"), wxPoint(80,176), wxSize(120,32), 0, wxDefaultValidator, _T("ID_CONFIG_Initialize_Button"));
	CONFIG_Reset_Button = new wxButton(Configure_Tab, ID_CONFIG_Reset_Button, _("Reset Defaults"), wxPoint(80,216), wxSize(120,32), 0, wxDefaultValidator, _T("ID_CONFIG_Reset_Button"));
	CONFIG_Reset_Button->Disable();
	CONFIG_Upload_Button = new wxButton(Configure_Tab, ID_CONFIG_Upload_Button, _("Upload"), wxPoint(680,140), wxSize(80,28), 0, wxDefaultValidator, _T("ID_CONFIG_Upload_Button"));
	CONFIG_TestDelays_Button = new wxButton(Configure_Tab, ID_CONFIG_TestDelays_Button, _("Test Delays"), wxPoint(80,512), wxSize(112,32), 0, wxDefaultValidator, _T("ID_CONFIG_TestDelays_Button"));
	CONFIG_TestDelays_Button->Disable();
	CONFIG_FirmwareOnHand_StaticBox = new wxStaticBox(Configure_Tab, ID_CONFIG_FirmwareOnHand_StaticBox, wxEmptyString, wxPoint(432,130), wxSize(234,38), 0, _T("ID_CONFIG_FirmwareOnHand_StaticBox"));
	CONFIG_FirmwareVersion_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_FirmwareVersion_StaticText, _("n/a"), wxPoint(444,144), wxSize(220,20), 0, _T("ID_CONFIG_FirmwareVersion_StaticText"));
	CONFIG_FirmwareOnHand_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_FirmwareOnHand_StaticText, _("Firmware On Hand:"), wxPoint(280,140), wxDefaultSize, 0, _T("ID_CONFIG_FirmwareOnHand_StaticText"));
	CONFIG_Upload_Gauge = new wxGauge(Configure_Tab, ID_CONFIG_Upload_Gauge, 100, wxPoint(272,188), wxSize(487,25), 0, wxDefaultValidator, _T("ID_CONFIG_Upload_Gauge"));
	CONFIG_Upload_Gauge->SetShadowWidth(50);
	CONFIG_Upload_Gauge->SetBezelFace(50);
	CONFIG_UploadMessage_StaticText = new wxStaticText(Configure_Tab, ID_CONFIG_UploadMessage_StaticText, _("Please power on the Barrett Hand now"), wxPoint(280,224), wxSize(475,17), wxALIGN_CENTRE, _T("ID_CONFIG_UploadMessage_StaticText"));
	CONFIG_UploadMessage_StaticText->Hide();
	CONFIG_PropGainControl_CheckBox = new wxCheckBox(Configure_Tab, ID_CONFIG_PropGainControl_CheckBox, _("Prop. Gain"), wxPoint(156,380), wxDefaultSize, 0, wxDefaultValidator, _T("ID_CONFIG_PropGainControl_CheckBox"));
	CONFIG_PropGainControl_CheckBox->SetValue(false);
	CONFIG_Serial_RadioButton = new wxRadioButton(Configure_Tab, ID_CONFIG_Serial_RadioButton, _("Serial"), wxPoint(140,60), wxSize(80,17), 0, wxDefaultValidator, _T("ID_CONFIG_Serial_RadioButton"));
	CONFIG_CAN_RadioButton = new wxRadioButton(Configure_Tab, ID_CONFIG_CAN_RadioButton, _("CAN"), wxPoint(64,60), wxSize(76,17), 0, wxDefaultValidator, _T("ID_CONFIG_CAN_RadioButton"));
	CONFIG_CAN_RadioButton->SetValue(true);
	BootloaderConfigurationStaticText = new wxStaticText(Configure_Tab, ID_BOOTLOADER_CONFIGURATION_STATICTEXT, _("Configure by selecting serial or CAN"), wxPoint(396,32), wxSize(353,17), 0, _T("ID_BOOTLOADER_CONFIGURATION_STATICTEXT"));
	handsChoice = new wxChoice(Configure_Tab, ID_CONFIG_Hands_Choice, wxPoint(104,20), wxSize(127,28), 0, 0, 0, wxDefaultValidator, _T("ID_CONFIG_Hands_Choice"));
	ModelNumStaticText = new wxStaticText(Configure_Tab, ID_MODEL_NUM_STATICTEXT, _("Model Num"), wxPoint(24,28), wxDefaultSize, 0, _T("ID_MODEL_NUM_STATICTEXT"));
	BHControl_NoteBook->AddPage(Configure_Tab, _("Configure"), false);
	BHControlVersionStaticText = new wxStaticText(MainPanel, ID_BHCONTROL_VERSION_STATICTEXT, _("BHControl GUI Version"), wxPoint(382,4), wxDefaultSize, wxALIGN_LEFT, _T("ID_BHCONTROL_VERSION_STATICTEXT"));
	Timer1.SetOwner(this, ID_TIMER1);
	Timer1.Start(250, false);
	Center();

	Connect(ID_CONFIG_Serial_RadioButton,wxEVT_COMMAND_RADIOBUTTON_SELECTED,(wxObjectEventFunction)&BHControlFrame::OnCONFIG_Serial_RadioButtonSelect);
	Connect(ID_CONFIG_CAN_RadioButton,wxEVT_COMMAND_RADIOBUTTON_SELECTED,(wxObjectEventFunction)&BHControlFrame::OnCONFIG_CAN_RadioButtonSelect);
	Connect(ID_BHControl_NoteBook,wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGED,(wxObjectEventFunction)&BHControlFrame::OnBHControl_NoteBookPageChanged);
	Connect(ID_BHControl_NoteBook,wxEVT_COMMAND_NOTEBOOK_PAGE_CHANGING,(wxObjectEventFunction)&BHControlFrame::OnBHControl_NoteBookPageChanging);
	//*)

#ifndef LINUX
	wxIcon FrameIcon(wxT("aaaa"));
	SetIcon(FrameIcon);

#ifdef CREATE_CONSOLE
	createConsole();
#endif
#endif

	init();

	///////////////////////////////////////////////////
	// Manually connected Event Handlers for the GUI //
	///////////////////////////////////////////////////

	Connect(ID_TIMER1, wxEVT_TIMER, (wxObjectEventFunction)&BHControlFrame::OnTimer1Trigger);

	// Configure Panel
	Connect(ID_CONFIG_Hands_Choice, wxEVT_COMMAND_CHOICE_SELECTED, (wxObjectEventFunction)&BHControlFrame::OnSelectHand);
	Connect(ID_CONFIG_BaudRate_Choice, wxEVT_COMMAND_CHOICE_SELECTED, (wxObjectEventFunction)&BHControlFrame::OnBaudRate);
	Connect(ID_CONFIG_Initialize_Button, wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction)&BHControlFrame::OnInitialize);
	Connect(ID_CONFIG_TestDelays_Button, wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction)&BHControlFrame::OnTestDelays);
	Connect(ID_CONFIG_Browse_Button, wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction)&BHControlFrame::OnBrowse);
	Connect(ID_CONFIG_Upload_Button, wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction)&BHControlFrame::OnUpload);
	Connect(ID_CONFIG_Reset_Button, wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction)&BHControlFrame::OnSetConfigDefaults);

	Connect(ID_CONFIG_VelocityControl_CheckBox, wxEVT_COMMAND_CHECKBOX_CLICKED, (wxObjectEventFunction)&BHControlFrame::OnConfigControlVelocity);
}

BHControlFrame::~BHControlFrame()
{
	delete m_glove;

#ifdef LINUX
	BHControl_NoteBook->DeleteAllPages(); // Documentation says panels should be deleted but that isn't the case under Windows
#else
	// These calls seem to be needed to free memory in Windows, looks like it may be a wxWidgets issue
	delete m_visual_panel;
	delete m_real_plot_panel;
	delete m_real_panel;
	delete m_super_panel;
	delete m_config;
#endif

	delete bh;

	//(*Destroy(BHControlFrame)
	//*)
}

void BHControlFrame::loadSettings()
{
	FILE *pFile;

	pFile = fopen(fileSettingsName, "r");
	if (pFile != NULL)
	{
		char buf[256];
		char *arg;
		while (fgets(buf, 256, pFile) != NULL)
		{
			if ((arg = strchr(buf, '\n')) != NULL)
				*arg = 0;
			if ((arg = strchr(buf, '\r')) != NULL)
				*arg = 0;

			// Get pointer to character after equal sign
			arg = strstr(buf, " = ");
			if (arg != NULL)
				arg += 3;
			while (*arg == ' ') arg++;

			if (strstr(buf, "CONFIG_HAND_MODEL_NUM") != NULL && arg != NULL)
			{
				char *modelNum = arg;
				selectHand(modelNum);
			}
			else if (strstr(buf, "CONFIG_HAND_COM_METHOD") != NULL && arg != NULL)
			{
				int comMethod = atoi(arg);
				selectCAN(comMethod == 1);
			}
			else if (strstr(buf, "CONFIG_HAND_AUTO_INIT") != NULL && arg != NULL)
			{
				bool autoIndex = atoi(arg) ? 1 : 0;
				CONFIG_AutoInit_CheckBox->SetValue(autoIndex);
			}
			else if (strstr(buf, "CONFIG_COM_PORT") != NULL && arg != NULL)
			{
#ifdef LINUX
				int comport = atoi(arg); if (comport < -9) comport = 1; else if (comport > 9) comport = 9 + 10;
				CONFIG_Com_Choice->SetSelection(comport + 9);
#else
				int comport = atoi(arg); if (comport < 1) comport = 1; else if (comport > 9) comport = 9;
				CONFIG_Com_Choice->SetSelection(comport - 1);
#endif
			}
			else if (strstr(buf, "CONFIG_BAUD_RATE") != NULL && arg != NULL)
			{
				char *baud = arg;
				CONFIG_BaudRate_Choice->SetStringSelection(wxString(baud, wxConvUTF8));
			}
		}
		fclose (pFile);
	}
}

void BHControlFrame::saveSettings()
{
	FILE *pFile;

	pFile = fopen(fileSettingsName, "w");
	if (pFile != NULL)
	{
		// Get Settings from GUI
		char modelNum[100]; strcpy(modelNum, handsChoice->GetStringSelection().mb_str());
		int comMethod = CONFIG_Serial_RadioButton->GetValue() ? 0 : 1;
		bool autoInitHand = CONFIG_AutoInit_CheckBox->GetValue();
#ifdef LINUX
		int comport = CONFIG_Com_Choice->GetSelection() - 9;
#else
		int comport = CONFIG_Com_Choice->GetSelection() + 1;
#endif
		long baudrate; CONFIG_BaudRate_Choice->GetStringSelection().ToLong(&baudrate);

		// Save Settings from GUI
		fprintf(pFile, "CONFIG_HAND_MODEL_NUM = %s\n", modelNum);
		fprintf(pFile, "CONFIG_HAND_COM_METHOD = %d\n", comMethod);
		fprintf(pFile, "CONFIG_HAND_AUTO_INIT = %d\n", autoInitHand);
		fprintf(pFile, "CONFIG_COM_PORT = %d\n", comport);
		fprintf(pFile, "CONFIG_BAUD_RATE = %d\n", (int)baudrate);

		fclose(pFile);
	}
}


void BHControlFrame::init()
{
#ifdef LINUX
	CONFIG_Com_Choice->Clear();
	CONFIG_Com_Choice->Append(_("USB9"));
	CONFIG_Com_Choice->Append(_("USB8"));
	CONFIG_Com_Choice->Append(_("USB7"));
	CONFIG_Com_Choice->Append(_("USB6"));
	CONFIG_Com_Choice->Append(_("USB5"));
	CONFIG_Com_Choice->Append(_("USB4"));
	CONFIG_Com_Choice->Append(_("USB3"));
	CONFIG_Com_Choice->Append(_("USB2"));
	CONFIG_Com_Choice->Append(_("USB1"));
	CONFIG_Com_Choice->Append(_("USB0"));
	CONFIG_Com_Choice->Append(_("S0"));
	CONFIG_Com_Choice->Append(_("S1"));
	CONFIG_Com_Choice->Append(_("S2"));
	CONFIG_Com_Choice->Append(_("S3"));
	CONFIG_Com_Choice->Append(_("S4"));
	CONFIG_Com_Choice->Append(_("S5"));
	CONFIG_Com_Choice->Append(_("S6"));
	CONFIG_Com_Choice->Append(_("S7"));
	CONFIG_Com_Choice->Append(_("S8"));

	CONFIG_Com_Choice->SetSelection(9);
#endif



	// Set the static pointer for the BHControlFrame instance
	bhcontrol = this;

	// Create main object for the hand
	bh = new BHand();

	// Initialize tabs
	m_config = new Config(BHControl_NoteBook, bh, CONFIG_Com_Choice, CONFIG_Upload_Gauge, CONFIG_MaxValue_StaticText, CONFIG_MinValue_StaticText, CONFIG_AvgValue_StaticText, CONFIG_OutInValue_StaticText, CONFIG_FirmwareVersion_StaticText, CONFIG_UploadMessage_StaticText);
	m_config->setBHControlFrame(this);

	// Append BarrettHand model numbers to listbox
	unsigned int numHands = bhandGetHardwareNumModels();
	for (unsigned int i = 0; i < numHands; i++)
	{
		const char *handModelNum = bhandGetHardwareDesc(i)->getModelNumber();
		handsChoice->Append(wxString(handModelNum, wxConvUTF8));
	}
	handsChoice->Select(0);

	// Create supervisor panel
	m_super_panel = new SuperPanel(BHControl_NoteBook, bh);
	BHControl_NoteBook->AddPage(m_super_panel, _("Supervise"), false);

	// Some ideas for keeping configuration options in sync
	// Set rest of configuration options based on "selected" model number
	char modelNum[100];
	strcpy(modelNum, handsChoice->GetStringSelection().mb_str());
	selectHand(modelNum);

	// Create real time panel
	m_real_panel = new RealPanel(BHControl_NoteBook, bh, m_super_panel);
	BHControl_NoteBook->AddPage(m_real_panel, _("Real Time"), false);

	// Create real time plots panel
	m_real_plot_panel = new RealPlotPanel(BHControl_NoteBook, bh, m_real_panel, m_super_panel);
	m_real_panel->SetRealPlotPanel(m_real_plot_panel);
	BHControl_NoteBook->AddPage(m_real_plot_panel, _("Real Time Plots"), false);

	// Create visual panel
	m_visual_panel = new VisualPanel(BHControl_NoteBook, bh, m_super_panel);
	BHControl_NoteBook->AddPage(m_visual_panel, _("Visual"), false);

	// Create object for glove hardware
	m_glove = new Glove(bh, m_visual_panel->getOpenGLVisual());

	// Disable all tabs if hand is attached (hand must be initiallized first)
	if (HAND_ATTACHED)
		SetDefaults(false);


	// Set some default values in config tab
	CONFIG_AutoInit_CheckBox->SetValue(true);

	// Right-align static text version label by hand (Ubuntu cannot right align static text label)
	int w, h, x, y;
	BHControlVersionStaticText->SetLabel(bhversion);
	BHControlVersionStaticText->GetClientSize(&w, &h);
	BHControlVersionStaticText->GetPosition(&x, &y);
	BHControlVersionStaticText->Move(x + 402 - w, y);


#ifndef LINUX
//	_CrtSetBreakAlloc(25627);
#endif

	loadSettings();
}


void BHControlFrame::selectHand(const char *modelNum)
{
	if (strcmp(modelNum, "BH8-280") == 0)
	{
		// 280 hand
		selectCAN(true); // select CAN

		// Enable at least the CAN RadioButton
		CONFIG_CAN_RadioButton->Enable(1);
#ifdef BH8_262_HARDWARE
		CONFIG_Serial_RadioButton->Enable(1);
#else
		CONFIG_Serial_RadioButton->Enable(0);
#endif
		CONFIG_DpFeedback_CheckBox->Show(false);
		CONFIG_PropGainControl_CheckBox->Show(false);
	}
	else
	{
		// 262 hand

		// Enable Serial, disable CAN
		selectCAN(false); // select Serial

		// Enable Serial RadioButton
		CONFIG_CAN_RadioButton->Enable(0);
		CONFIG_Serial_RadioButton->Enable(1);

		CONFIG_DpFeedback_CheckBox->Show(true);
		CONFIG_PropGainControl_CheckBox->Show(true);
	}

	unsigned int numHands = bhandGetHardwareNumModels();
	for (unsigned int i = 0; i < numHands; i++)
	{
		const char *handModelNum = bhandGetHardwareDesc(i)->getModelNumber();
		if (strcmp(modelNum, handModelNum) == 0)
			handsChoice->Select(i);
	}

	// Set hardware description (list with available properties)
	bh->setHardwareDesc(handsChoice->GetSelection());
	use280Config = (strcmp(bh->getHardwareDesc()->getModelNumber(), "BH8-280") == 0);

	// Fill properties in Supervisory panel
	m_super_panel->InitLibrary();
}


void BHControlFrame::selectCAN(bool useCAN)
{
	if (useCAN)
	{
		CONFIG_CAN_RadioButton->SetValue(1);
		CONFIG_Serial_RadioButton->SetValue(0);

		CONFIG_ComPort_StaticText->Show(false);
		CONFIG_Com_Choice->Show(false);
		CONFIG_BaudRate_StaticText->Show(false);
		CONFIG_BaudRate_Choice->Show(false);

		m_config->setUseCAN(true);
		BootloaderConfigurationStaticText->SetLabel(wxT("Bootloader configured for Pucks In Hand"));
	}
	else
	{
		CONFIG_CAN_RadioButton->SetValue(0);
		CONFIG_Serial_RadioButton->SetValue(1);

		CONFIG_ComPort_StaticText->Show(true);
		CONFIG_Com_Choice->Show(true);
		CONFIG_BaudRate_StaticText->Show(true);
		CONFIG_BaudRate_Choice->Show(true);

		m_config->setUseCAN(false);
		BootloaderConfigurationStaticText->SetLabel(wxT("Bootloader configured for BH8-262 Hand"));
	}
}


void BHControlFrame::SetDefaults(bool flag)
{
	// default settings/initial accessibility for the GUI
	CONFIG_Initialize_Button->Enable(!flag);
	CONFIG_Reset_Button->Enable(flag);
	CONFIG_TestDelays_Button->Enable(flag);
	//m_super_panel->Enable(flag);
	m_super_panel->EnablePanel(flag);
	m_real_panel->Enable(flag);
	m_real_plot_panel->Enable(flag);
	//m_visual_panel->Enable(flag);
	m_visual_panel->EnablePanel(flag);
	CONFIG_Com_Choice->Enable(!flag);
}


///////////////////////////////////////////////////////////////////////////////
// Beginning of Events for main GUI events
///////////////////////////////////////////////////////////////////////////////

void BHControlFrame::OnTimer1Trigger(wxTimerEvent& event)
{
	Timer1.Stop();

	static bool initBefore = false;
	if (m_config->IsInitialized() && !initBefore)
	{
		// Call necessary code to "enter tabs" (is it necessary to postpone "SetDefaults"?)
		if (BHControl_NoteBook->GetSelection() == 2)
			m_real_panel->EnterTab();
		if (BHControl_NoteBook->GetSelection() == 4)
			m_visual_panel->EnterTab();
		initBefore = true;
	}

	m_config->UploadService();
	if (m_config->IsInitialized())
	{
		m_super_panel->Service();
		m_real_panel->Service();
		m_visual_panel->Service();
	}

	// Ensure interval is set to desired value
	if (m_super_panel->IsRunning())
		Timer1.Start(50);
	else if (m_visual_panel->IsRunning())
		Timer1.Start(50);
	else if (m_real_panel->RTIsRunning())
		Timer1.Start(25);
	else if (m_config->UploadGetState())
		Timer1.Start(100);
	else
		Timer1.Start(100);
}

void BHControlFrame::OnClose(wxCloseEvent &event)
{
	saveSettings();
	event.Skip();
}

void BHControlFrame::OnQuit(wxCommandEvent& event)
{
	Close();
}

///////////////////////////////////////////////////////////////////////////////
// Beginning of Events for Configuration of the BarrettHand
///////////////////////////////////////////////////////////////////////////////

void BHControlFrame::OnInitialize(wxCommandEvent& event)
{
	// initialize the hand software (hand motors also if auto-run is checked)
	m_config->initialize(handsChoice->GetSelection());
}

void BHControlFrame::OnCONFIG_CAN_RadioButtonSelect(wxCommandEvent& event)
{
	selectCAN(true);
}

void BHControlFrame::OnCONFIG_Serial_RadioButtonSelect(wxCommandEvent& event)
{
	selectCAN(false);
}

void BHControlFrame::OnSelectHand(wxCommandEvent& event)
{
	char modelNum[100];
	strcpy(modelNum, handsChoice->GetStringSelection().mb_str());
	selectHand(modelNum);
}

void BHControlFrame::OnBaudRate(wxCommandEvent& event)          { m_config->OnBaudRate(event);          }
void BHControlFrame::OnSetConfigDefaults(wxCommandEvent& event) { m_config->OnSetConfigDefaults(event); }




///////////////////////////////////////////////////////////////////////////////
// Beginning of Events for Test Delays
///////////////////////////////////////////////////////////////////////////////

void BHControlFrame::OnConfigControlVelocity(wxCommandEvent& event)
{
	// enable/disable corresponding textctrl of Velocity Control Checkbox
	//CONFIG_Velocity_TextCtrl->Enable(CONFIG_VelocityControl_CheckBox->GetValue());
}

// Event for Test Delays button press
void BHControlFrame::OnTestDelays(wxCommandEvent& event) { m_config->OnTestDelays(event); }




///////////////////////////////////////////////////////////////////////////////
// Beginning of Events for Uploading new firmware
///////////////////////////////////////////////////////////////////////////////

void BHControlFrame::OnUpload(wxCommandEvent& event)
{
	// trigger an active state for uploading firmware in the idle handler
	m_config->UploadStart();
}

void BHControlFrame::OnBrowse(wxCommandEvent& event)
{
	wxString filePath;
	wxString fileName;


	// File Dialog for selecting new firmware to upload
	wxFileDialog *FileDialog_S19 = new wxFileDialog(this, _("Select file"), wxEmptyString, wxEmptyString, wxFileSelectorDefaultWildcardStr, wxFD_DEFAULT_STYLE, wxDefaultPosition, wxDefaultSize, _T("wxFileDialog"));
	if (m_config->use280Bootloader())
		FileDialog_S19->SetWildcard(wxT("TEK Firmware files (*.tek)|*.tek|All files (*.*)|*.*"));
	else
		FileDialog_S19->SetWildcard(wxT("S19 Firmware files (*.s19)|*.s19|All files (*.*)|*.*"));


	// display a file dialog to load a firmware file
	// get a valid file and then update the GUI and Config class
	if (FileDialog_S19->ShowModal() == wxID_OK)
	{
		filePath = FileDialog_S19->GetPath();
		fileName = FileDialog_S19->GetFilename();

		wxString tempFileName = fileName.Upper();

		if (m_config->use280Bootloader() ? tempFileName.Contains(wxT("S19")) : tempFileName.Contains(wxT("TEK")))
		{
			wxPuts(wxT("Invalid firmware extension"));

			wxMessageDialog response(NULL, wxT("Firmware extension cannot contain the wrong extension"), wxT("Invalid firmware extension"), wxOK | wxICON_ERROR);
			response.ShowModal();

			delete FileDialog_S19;
			return;
		}

		CONFIG_FilePath_StaticText->SetLabel(filePath);
		CONFIG_FileName_StaticText->SetLabel(fileName);
	}

	m_config->setFilePath(filePath);

	delete FileDialog_S19;
}


///////////////////////////////////////////////////////////////////////////////
// Beginning of Other GUI Events
///////////////////////////////////////////////////////////////////////////////

void BHControlFrame::OnBHControl_NoteBookPageChanged(wxNotebookEvent& event)
{
	if (m_config->IsInitialized())
	{
		if (event.GetSelection() == 2)
			m_real_panel->EnterTab();
		if (event.GetSelection() == 4)
			m_visual_panel->EnterTab();
	}
}

void BHControlFrame::OnBHControl_NoteBookPageChanging(wxNotebookEvent& event)
{
	if (m_config->IsInitialized())
	{
		if (event.GetOldSelection() == 1)
			m_super_panel->LeaveTab(); // Super tab was selected
		else if (event.GetOldSelection() == 2)
			m_real_panel->LeaveTab(); // RealTime tab was selected
		else if (event.GetOldSelection() == 3)
			m_real_plot_panel->LeaveTab(); // RealTime plot tab was selected
		else if (event.GetOldSelection() == 4)
			m_visual_panel->LeaveTab(); // Visual tab was selected
	}
}

void BHControlFrame::OnIdle(wxIdleEvent& event)
{
	// Nothing needs to be done
}
