/******************************************************************************
 * Name:      BHControlMain.h
 * Purpose:   Contains high-level code for creating tabs in the GUI
 * Authors:   Kevin Doo and Edward Hannigan
 * Created:   2009-05-26
 * Copyright: Barrett Technology 2010
 *****************************************************************************/

#ifndef BHCONTROLMAIN_H
#define BHCONTROLMAIN_H

//(*Headers(BHControlFrame)
#include <wx/notebook.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/checkbox.h>
#include <wx/radiobut.h>
#include <wx/statbox.h>
#include <wx/panel.h>
#include <wx/choice.h>
#include <wx/button.h>
#include <wx/frame.h>
#include <wx/timer.h>
#include <wx/gauge.h>
//*)

class Visualizer;
class Config;
class Real;
class Glove;
class BHand;

class SuperPanel;
class RealPanel;
class RealPlotPanel;
class VisualPanel;

class BHControlFrame;

class BHControlFrame: public wxFrame
{
public:

	BHControlFrame(wxWindow * parent);
	virtual ~BHControlFrame();


	// Useful for getting a pointer to the BHControl GUI
	static BHControlFrame *getBHControlFrame() { return bhcontrol; }

	void SetDefaults(bool flag);

	// Declare all these member variables for the Control GUI to be public
	// so that they will be accessible from config.cpp

	//(*Declarations(BHControlFrame)
	wxStaticText* CONFIG_MaxValue_StaticText;
	wxTextCtrl* CONFIG_Velocity_TextCtrl;
	wxCheckBox* CONFIG_DpFeedback_CheckBox;
	wxStaticBox* CONFIG_TestDelays_StaticBox;
	wxStaticBox* CONFIG_Upload_StaticBox;
	wxChoice* handsChoice;
	wxStaticBox* CONFIG_Results_StaticBox;
	wxChoice* CONFIG_BaudRate_Choice;
	wxStaticText* CONFIG_Max_StaticText;
	wxStaticText* CONFIG_FilePath_StaticText;
	wxStaticText* CONFIG_Min_StaticText;
	wxCheckBox* CONFIG_Motor2_CheckBox;
	wxStaticText* CONFIG_FirmwareOnHand_StaticText;
	wxStaticBox* CONFIG_Feedback_StaticBox;
	wxStaticText* BootloaderConfigurationStaticText;
	wxChoice* CONFIG_Com_Choice;
	wxStaticText* BHControlVersionStaticText;
	wxCheckBox* CONFIG_PropGainControl_CheckBox;
	wxStaticBox* CONFIG_Configuration_StaticBox;
	wxCheckBox* CONFIG_Motor3_CheckBox;
	wxStaticBox* CONFIG_CurrentPath_StaticBox;
	wxCheckBox* CONFIG_PosFeedback_CheckBox;
	wxStaticText* CONFIG_OutInValue_StaticText;
	wxCheckBox* CONFIG_AutoInit_CheckBox;
	wxNotebook* BHControl_NoteBook;
	wxStaticText* CONFIG_Avg_StaticText;
	wxStaticText* CONFIG_FirmwareVersion_StaticText;
	wxButton* CONFIG_TestDelays_Button;
	wxStaticBox* CONFIG_CurrentFile_StaticBox;
	wxRadioButton* CONFIG_CAN_RadioButton;
	wxStaticBox* CONFIG_Control_StaticBox;
	wxStaticBox* CONFIG_FirmwareOnHand_StaticBox;
	wxRadioButton* CONFIG_Serial_RadioButton;
	wxButton* CONFIG_Browse_Button;
	wxPanel* MainPanel;
	wxStaticText* CONFIG_ComPort_StaticText;
	wxCheckBox* CONFIG_VelocityControl_CheckBox;
	wxStaticBox* CONFIG_Motors_StaticBox;
	wxButton* CONFIG_Upload_Button;
	wxStaticText* CONFIG_CurrentPath_StaticText;
	wxStaticText* ModelNumStaticText;
	wxCheckBox* CONFIG_Motor4_CheckBox;
	wxCheckBox* CONFIG_VelocityFeedback_CheckBox;
	wxCheckBox* CONFIG_Motor1_CheckBox;
	wxStaticText* CONFIG_AvgValue_StaticText;
	wxPanel* Configure_Tab;
	wxButton* CONFIG_Initialize_Button;
	wxCheckBox* CONFIG_StrainFeedback_CheckBox;
	wxStaticBox* CONFIG_DelayHistogram_StaticBox;
	wxStaticText* CONFIG_FileName_StaticText;
	wxStaticText* CONFIG_UploadMessage_StaticText;
	wxStaticText* CONFIG_BaudRate_StaticText;
	wxStaticText* CONFIG_MinValue_StaticText;
	wxButton* CONFIG_Reset_Button;
	wxTimer Timer1;
	wxGauge* CONFIG_Upload_Gauge;
	wxStaticText* CONFIG_CurrentFile_StaticText;
	wxStaticText* CONFIG_OutIn_StaticText;
	//*)


private:

	// Pointer to the BHControl frame
	static BHControlFrame *bhcontrol;

	// Pointer to object for BarrettHand hardware
	BHand *bh;

	// Pointers to main panels of GUI
	SuperPanel *m_super_panel;
	RealPanel *m_real_panel;
	RealPlotPanel *m_real_plot_panel;
	VisualPanel *m_visual_panel;

	Config *m_config;

	Glove *m_glove;

	// Helper methods for the GUI
	void loadSettings();
	void saveSettings();
	void init();

	void selectHand(const char *modelNum);
	void selectCAN(bool useCAN);


	// BHControl - Events
	void OnClose(wxCloseEvent &event);
	void OnIdle(wxIdleEvent& event);
	void OnErase(wxEraseEvent& event);
	void OnEraseNotebook(wxEraseEvent &event);


	// Configure - Configuration
	void OnSelectHand(wxCommandEvent& event);
	void OnInitialize(wxCommandEvent& event);
	void OnBaudRate(wxCommandEvent& event);
	void OnBrowse(wxCommandEvent& event);
	void OnUpload(wxCommandEvent& event);
	void OnSetConfigDefaults(wxCommandEvent& event);
	void OnTestDelays(wxCommandEvent& event);

	// Configure - Test Delays
	void OnConfigControlVelocity(wxCommandEvent& event);

	//(*Handlers(BHControlFrame)
	void OnQuit(wxCommandEvent& event);
	void OnTimer1Trigger(wxTimerEvent& event);
	void OnBHControl_NoteBookPageChanged(wxNotebookEvent& event);
	void OnBHControl_NoteBookPageChanging(wxNotebookEvent& event);
	void OnCONFIG_CAN_RadioButtonSelect(wxCommandEvent& event);
	void OnCONFIG_Serial_RadioButtonSelect(wxCommandEvent& event);
	//*)

	//(*Identifiers(BHControlFrame)
	static const long ID_CONFIG_Upload_StaticBox;
	static const long ID_CONFIG_CurrentPath_StaticBox;
	static const long ID_CONFIG_Configuration_StaticBox;
	static const long ID_CONFIG_AutoInit_CheckBox;
	static const long ID_CONFIG_ComPort_StaticText;
	static const long ID_CONFIG_BaudRate_StaticText;
	static const long ID_CONFIG_Com_Choice;
	static const long ID_CONFIG_BaudRate_Choice;
	static const long ID_CONFIG_TestDelays_StaticBox;
	static const long ID_CONFIG_Motors_StaticBox;
	static const long ID_CONFIG_Feedback_StaticBox;
	static const long ID_CONFIG_Control_StaticBox;
	static const long ID_CONFIG_Results_StaticBoxID_;
	static const long ID_CONFIG_DelayHistogram_StaticBox;
	static const long ID_CONFIG_Motor1_CheckBox;
	static const long ID_CONFIG_Motor3_CheckBox;
	static const long ID_CONFIG_Motor2_CheckBox;
	static const long ID_CONFIG_Motor4_CheckBox;
	static const long ID_CONFIG_VelocityFeedback_CheckBox;
	static const long ID_CONFIG_StrainFeedback_CheckBox;
	static const long ID_CONFIG_PosFeedback_CheckBox;
	static const long ID_CONFIG_DpFeedback_CheckBox;
	static const long ID_CONFIG_VelocityControl_CheckBox;
	static const long ID_CONFIG_Velocity_TextCtrl;
	static const long ID_CONFIG_Max_StaticText;
	static const long ID_CONFIG_Min_StaticText;
	static const long ID_CONFIG_Avg_StaticText;
	static const long ID_CONFIG_OutIn_StaticText;
	static const long ID_CONFIG_AvgValue_StaticText;
	static const long ID_CONFIG_OutInValue_StaticText;
	static const long ID_CONFIG_MaxValue_StaticText;
	static const long ID_CONFIG_MinValue_StaticText;
	static const long ID_CONFIG_CurrentPath_StaticText;
	static const long ID_CONFIG_FilePath_StaticText;
	static const long ID_CONFIG_Browse_Button;
	static const long ID_CONFIG_CurrentFile_StaticText;
	static const long ID_CONFIG_CurrentFile_StaticBox;
	static const long ID_CONFIG_FileName_StaticText;
	static const long ID_CONFIG_Initialize_Button;
	static const long ID_CONFIG_Reset_Button;
	static const long ID_CONFIG_Upload_Button;
	static const long ID_CONFIG_TestDelays_Button;
	static const long ID_CONFIG_FirmwareOnHand_StaticBox;
	static const long ID_CONFIG_FirmwareVersion_StaticText;
	static const long ID_CONFIG_FirmwareOnHand_StaticText;
	static const long ID_CONFIG_Upload_Gauge;
	static const long ID_CONFIG_UploadMessage_StaticText;
	static const long ID_CONFIG_PropGainControl_CheckBox;
	static const long ID_CONFIG_Serial_RadioButton;
	static const long ID_CONFIG_CAN_RadioButton;
	static const long ID_BOOTLOADER_CONFIGURATION_STATICTEXT;
	static const long ID_CONFIG_Hands_Choice;
	static const long ID_MODEL_NUM_STATICTEXT;
	static const long ID_Configure_Tab;
	static const long ID_BHControl_NoteBook;
	static const long ID_BHCONTROL_VERSION_STATICTEXT;
	static const long ID_MainPanel;
	static const long ID_TIMER1;
	//*)
	//static const long ID_TIMER;

	DECLARE_EVENT_TABLE()
};

#endif // BHCONTROLMAIN_H
