#include "BootloaderPuck2.h"

#ifdef BH8_280_HARDWARE

#include "puck2.h"

extern int pucksInHandPropertyNumDefault();
extern int pucksInHandPropertyNumSpreadDefault();

extern void pucksInHandPropertyDefault(int index, int *property, int *value);
extern void pucksInHandPropertySpreadDefault(int index, int *property, int *value);


BootloaderPuck2::BootloaderPuck2(BHand *bhand, wxNotebook *bhNotebook, wxGauge* gauge, wxStaticText* firmOH, wxStaticText* helpMSG) :
	BGUIBootloader(bhand, bhNotebook, gauge, firmOH, helpMSG)
{
}


BootloaderPuck2::~BootloaderPuck2()
{
}


///////////////////////////////////////////////////////////////////////////////
// Method for Puck 2 property set
///////////////////////////////////////////////////////////////////////////////

int BootloaderPuck2::setPropertySlow(int bus, unsigned int puckID, unsigned int puckProperty, bool verify, unsigned int value)
{
	DELAY(10);
	int r = setPuckPropertyUInt32(puckID, puckProperty, value, verify);
	DELAY(10);
	return r;
}


///////////////////////////////////////////////////////////////////////////////
// Methods for Sending and Receiving CAN messages
///////////////////////////////////////////////////////////////////////////////

int BootloaderPuck2::canSendMsg(int bus, int id, char len, unsigned char *data, int blocking)
{
	return sendCANMsg(false, PUCK2_ID_PC, id, len, data);
}

int BootloaderPuck2::canReadMsg(int bus, int *id, int *len, unsigned char *data, int blocking)
{
	return readCANMsg(id, len, data);
}


///////////////////////////////////////////////////////////////////////////////
// Method for Puck 2 firmware download
///////////////////////////////////////////////////////////////////////////////

//int BootloaderPuck2::firmwareDL(int id, char *fn)
//{
	//FILE *fp;
	//int in_id;
	//int in_len;
	//unsigned char in_data[8];
	//unsigned char out_Data[8];
	//int i;
	//int cnt = 1;
	//char line[100];
	//int rcpt;
	//int lineTotal, lineCt;

/*	cnt = 1;

	// Open the file
	if ((fp=fopen(fn, "r"))==NULL)
		return(1);

	// Count the lines
	lineTotal = 0;
	while (fgets(line, 99, fp) != NULL)
		++lineTotal;

	// Reset the file pointer
	fclose(fp);
	if ((fp=fopen(fn, "r"))==NULL)
		return(1);*/

	// Reset the puck (SET STAT 0)
	// setPropertySlow(bus, id, property, verify, value);
	//setPropertySlow(0, id, 5, FALSE, 0L); // Reset
	//usleep(1000000); // Wait a sec for the Reset to complete

	// Send the key byte (0x00AA) to indicate firmware download
/*	setPropertySlow(0, id, 0, FALSE, 0x000000AA);
	// For each line in the file
	//sendData[0] = 0x80 | VERS;
	//sendData[1] = 0x00;
	lineCt = 0;

	printf("\n\n");*/

/*	while (fgets(line, 99, fp) != NULL)
	{
		i = 1;
		++lineCt;
		//printf("%s", line);
		printf("\rDownload progress: %d%%   ", (int)(100.0 * lineCt / lineTotal));
		fflush(stdout);
		while (line[i] >= '0')
		{
			// Wait for n "Get VERS"
			for (rcpt = 0; rcpt < cnt; rcpt++)
			{
				while (canReadMsg(0, &in_id, &in_len, in_data, 1))
					usleep(100);
			}
			// Send the byte
			out_Data[0] = line[i];
			canSendMsg(0, id, 1, out_Data, 1);
			++i;
		}
	}*/
	//fclose(fp);

	//printf("\n\n   Download complete!\n\n");

//	return 0;
//}


bool BootloaderPuck2::UploadPrepareFirmware(unsigned int comport)
{
	// prepare firmware uploading to the hand
	if (puck2CANInit())
	{
		wxString message = wxT("Unable to initialize Pucks.  Check connections.");
		wxMessageDialog notice(NULL, message, wxT("Info"), wxOK);
		notice.ShowModal();

		return false;
	}

	// Set firmware download state
	puckDownloadIndex = 0;
	puckDownloadIDS[0] = 11; // finger 1 - changed for testing
	puckDownloadIDS[1] = 12; // finger 2
	puckDownloadIDS[2] = 13; // finger 3
	puckDownloadIDS[3] = 14; // spread
	puckDownloadNum = 4;     // Upload firmware to all 4 pucks in the hand

	// Message to user
	wxString message = wxT("Please power on the hand, wait a few seconds, and then click the \"OK\" Button.");
	wxMessageDialog notice(NULL, message, wxT("Info"), wxOK);
	notice.ShowModal();

	// wait for the hand to be turned on
	m_helpMSG->SetLabel(wxT("Preparing to upload firmware"));
	m_helpMSG->Show(true);

	return true;
}


void BootloaderPuck2::unlockPuck(unsigned int puckID)
{
	setPuckPropertyUInt32(puckID, PROP_LOCK, 0); // puck expects lock to be 0 and then

	// SET LOCK 18384, SET LOCK 23, SET LOCK 3145, SET LOCK 1024, SET LOCK 1
	setPuckPropertyUInt32(puckID, PROP_LOCK, 18384);
	setPuckPropertyUInt32(puckID, PROP_LOCK, 23);
	setPuckPropertyUInt32(puckID, PROP_LOCK, 3145);
	setPuckPropertyUInt32(puckID, PROP_LOCK, 1024);
	setPuckPropertyUInt32(puckID, PROP_LOCK, 1);
}

int BootloaderPuck2::setRole(unsigned int puckID, unsigned int lower7bits)
{
	unsigned int puckRole;
	int r = getPuckPropertyUInt32(puckID, PROP_ROLE, &puckRole);
	if (r)
	{
		printf("Problem reading ROLE property\n");
		return r; // There is a problem reading Puck property
	}

	// Set the lower 7 bits of the Puck's role to zero
	puckRole &= 0xffffff80;
	puckRole |= lower7bits;

	r = setPuckPropertyUInt32(puckID, PROP_ROLE, puckRole);
	if (r)
	{
		printf("Problem setting ROLE property\n");
		return r; // There is a problem setting Puck property
	}

	unsigned int newPuckRole;
	r = getPuckPropertyUInt32(puckID, PROP_ROLE, &newPuckRole);
	if (r || (newPuckRole & (0xffffff80 | lower7bits)) != puckRole)
	{
		printf("Problem verifying ROLE property\n");
		return false; // There is a problem reading Puck property
	}

	r = setPuckPropertyUInt32(puckID, PROP_SAVE, PROP_ROLE);
	if (r)
	{
		printf("Problem saving ROLE property\n");
		return r; // There is a problem setting Puck property
	}
	DELAY(100); // wait to be sure ROLE has been saved

	//if (puckRole & 0x80)
	//	printf("  Puck %d has 20 MHz crystal\n", puckID); // 5765 & 0xffffff80
	//else
	//	printf("  Puck %d has 32 MHz crystal\n", puckID); // 5637 & 0xffffff80

	printf("  Puck %d role set to %d\n", puckID, puckRole);

	return 0;
}


bool BootloaderPuck2::UploadWaitReset()
{
	char line[100];
	int err;

	///////////////////////////////////////////////////////////////////////////
	// Beginning of firmwareDL
	///////////////////////////////////////////////////////////////////////////

	cnt = 1;

	// Open the file
	if ((fp=fopen(m_file_path.mb_str(), "r"))==NULL)
		return(1);

	// Count the lines
	lineTotal = 0;
	while (fgets(line, 99, fp) != NULL)
		++lineTotal;

	// Reset the file pointer
	fclose(fp);
	if ((fp=fopen(m_file_path.mb_str(), "r"))==NULL)
		return(1);

	unsigned int id = puckDownloadIDS[puckDownloadIndex];

	// Unlock the Puck
	unlockPuck(id);

	if ((err = setRole(id, 0)))
	{
		m_helpMSG->SetLabel(wxT("Uploading firmware unsuccessful"));
		return false;
	}


	///////////////////////////////////////////////////////////////////////////
	// Middle of firmwareDL
	///////////////////////////////////////////////////////////////////////////

	// Reset the puck (SET STAT 0)
	// setPropertySlow(bus, id, property, verify, value);
	err = setPropertySlow(0, id, 5, FALSE, 0L); // Reset
	DELAY(1000); // Wait a sec for the Reset to complete

	// Check for error in setting puck property
	if (err)
	{
		m_helpMSG->SetLabel(wxT("Uploading firmware - reset unsuccessful"));
		return false;
	}

	///////////////////////////////////////////////////////////////////////////
	// Before switching to uploading stage
	///////////////////////////////////////////////////////////////////////////


	// Send the key byte (0x00AA) to indicate firmware download
	err = setPropertySlow(0, id, 0, FALSE, 0x000000AA);

	// Check for error in setting puck property
	if (err)
	{
		m_helpMSG->SetLabel(wxT("Uploading firmware - sending key byte unsuccessful"));
		return false;
	}

	// For each line in the file
	//sendData[0] = 0x80 | VERS;
	//sendData[1] = 0x00;
	lineCt = 0;

	m_gaugeValue = 0;

	m_helpMSG->SetLabel(wxString::Format(wxT("Uploading firmware to puck %d of %d"), puckDownloadIndex + 1, puckDownloadNum));

	// Create thread for uploading firmware
	m_thread.setName("Upload firmware thread");
	m_thread.setPriority(Poco::Thread::PRIO_HIGHEST);
	m_thread.start(*this);

	return true;
}


bool BootloaderPuck2::UploadFirmwareLine()
{
	char line[100];
	unsigned int id = puckDownloadIDS[puckDownloadIndex];
	if (fgets(line, 99, fp) != NULL)
	{
		i = 1;
		++lineCt;

		float per = (float)lineCt / (float)lineTotal;
		//printf("\rDownload progress: %1.0f%%   ", 100.0f * per); fflush(stdout);
		m_gaugeValue = (int)(per * 1000.0f);

		while (line[i] >= '0')
		{
			// Wait for n "Get VERS"
			for (rcpt = 0; rcpt < cnt; rcpt++)
			{
				while (canReadMsg(0, &in_id, &in_len, in_data, 1))
				{
					// DELAY(1);
				}
			}
			// Send the byte
			out_Data[0] = line[i];
			canSendMsg(0, id, 1, out_Data, 1);
			++i;
		}
	}
	else
	{
		printf("\n\n   Download to puck %d complete!\n\n", id);
        return true;
	}

	return false;
}

#define PUCK2_BOOTLOADER_AWAKEN_TRIES 6

bool BootloaderPuck2::awakePuck(int puckID)
{
	int r;
	unsigned int stat;

	// Discard any received messages that are queued up
	while (canReadMsg(0, &in_id, &in_len, in_data, 1) == 0);
	//{
	//	printf("disgard message id=%d, in_len=%d\n, %d %d %d %d %d %d %d %d", in_id, in_len,
	//		in_data[0], in_data[1], in_data[2], in_data[3], in_data[4], in_data[5], in_data[6], in_data[7]
	//		);
	//}

	// Check if puck is already awake
	for (int n = 1; n <= PUCK2_BOOTLOADER_AWAKEN_TRIES; n++)
	{
		r = getPuckPropertyUInt32(puckID, PROP_STAT, &stat);

		printf("   n = %d, id = %d, STAT = %d, r = %d\n", n, puckID, stat, r);

		if (!r && stat == 2)
			return true;

		if (n < PUCK2_BOOTLOADER_AWAKEN_TRIES)
		{
			// SET STAT 2
			DELAY(10); // Wait a little extra to help ensure next message arrives successfully
			setPuckPropertyUInt32(puckID, PROP_STAT, 2); // wake puck up
			DELAY(1500); // 500 milliseconds isn't long enough, maybe 1000 is not long enough
		}
	}
	return false;
}

bool BootloaderPuck2::finishPuck(int puckID)
{
	unsigned int puckVersion;
	int r = getPuckPropertyUInt32(puckID, PROP_VERS, &puckVersion);
	if (r)
	{
		printf("Firmware Version on Puck %d not detected\n", puckID);
		return false;
	}
	else
	{
		printf("Firmware Version on Puck %d = %d\n", puckID, puckVersion);

		printf("Ready to finalize hand puck %d by setting ROLE\n", puckID);
		// Unlock the Puck
		unlockPuck(puckID);
		printf("Unlocked puck\n");
		int err;
		if ((err = setRole(puckID, 5))) // try to set lower 7 bits to 5 (to be a hand puck)
		{
			printf("Problem setting ROLE to hand Puck %d\n", err);
			return false;
		}
		else
			printf("Firmware Version on Puck %d = %d\n", puckID, puckVersion);
		return true;
	}
}

bool BootloaderPuck2::finishUpload()
{
	// Ensure that pucks are awake
	for (unsigned int i = 0; i < puckDownloadNum; i++)
	{
		unsigned int id = puckDownloadIDS[i];
		if (!awakePuck(id))
		{
			printf("At least one of pucks did not wake up.\n");
			return false; // There is a problem setting Puck property
		}
	}

	// Set defaults
	int p, v;

	int jidx[] = { 8, 9, 10, 11 };
	int pidx[] = { 3, 4, 1, 2 };

	for (unsigned int i = 0; i < puckDownloadNum; i++)
	{
		unsigned int id = puckDownloadIDS[i];

		// Write default properties
		for (int n = 0; n < pucksInHandPropertyNumDefault(); n++)
		{
			pucksInHandPropertyDefault(n, &p, &v);
			setPuckPropertyUInt32(id, p, v); DELAY(1);
		}
		if (id == 14)
		{
			// Spread on BH8-280
			for (int n = 0; n < pucksInHandPropertyNumSpreadDefault(); n++)
			{
				pucksInHandPropertySpreadDefault(n, &p, &v);
				setPuckPropertyUInt32(id, p, v); DELAY(1);
			}
		}

		setPuckPropertyUInt32(id, PROP_JIDX, jidx[id - 11]); DELAY(1); // hardcoded id offset
		setPuckPropertyUInt32(id, PROP_PIDX, pidx[id - 11]); DELAY(1);

		// SAVE -1 (saves all properties)
		setPuckPropertyUInt32(id, PROP_SAVE, (unsigned int)-1);

		printf("Puck %d saved default parameters\n", id);
	}

	DELAY(100); // Wait a while for the propeties to be set

	// Recommend reboot
	//wxString message = wxT("Reboot is required.  Please power cycle to complete the firmware upload.");
	//wxMessageDialog notice(NULL, message, wxT("Info"), wxOK);
	//notice.ShowModal();

	m_helpMSG->SetLabel(wxT("Reboot is required.  Please power cycle to complete the firmware upload."));

	return true;
}

void BootloaderPuck2::UploadCleanup(bool errorFlag)
{
	// Close the file stream and return
	fclose(fp);

	// Clear progress bar
	m_gauge->SetValue(0);

	// Clear firmware label
	m_firmwareOH->SetLabel(wxT(""));

	if (!errorFlag)
	{
		// Wake up Puck and write ROLE
		unsigned int id = puckDownloadIDS[puckDownloadIndex];

		//DELAY(1000); // Wait a sec for the Auto-Reset to complete
		//setPropertySlow(0, id, 5, FALSE, 0L); // Then reset manually anyways (this wasn't helping - still would not be able to upload version 148)

		DELAY(1500); // Wait a sec for the Reset to complete

		if (!awakePuck(id))
		{
			m_helpMSG->SetLabel(wxT("Unable to wake up one of the Pucks."));
			errorFlag = true;
		}
		else if (!finishPuck(id))
		{
			m_helpMSG->SetLabel(wxT("Unable to get firmware VERS or set lower bits of ROLE."));
			errorFlag = true;
		}

	}

	if (errorFlag)
	{
		// Close CAN communication
		puck2CANClose();

		uploadState = Upload_Idle;
		m_bhNotebook->Enable();
		return;
	}

	// Return upload state to the idle state if finished uploading firmware to each Puck
	if (++puckDownloadIndex == puckDownloadNum)
	{
		m_helpMSG->SetLabel(wxT("Finalizing firmware upload. Please wait."));
		if (finishUpload())
			m_helpMSG->SetLabel(wxT("Uploading firmware to pucks successful. Please power cycle hand."));
		else
			m_helpMSG->SetLabel(wxT("Uploading firmware to pucks unsuccessful"));

		// Close CAN communication
		puck2CANClose();

		uploadState = Upload_Idle;
		m_bhNotebook->Enable();
	}
	else
	{
		// Start uploading firmware again
		uploadState = Upload_Wait;
	}
}

#endif // BH8_280_HARDWARE
