#include "CppGenerator.h"

#include <wx/file.h>
#include <wx/wx.h>
#include <wx/textctrl.h>
//#include <unistd.h>

#ifndef LINUX
#include <Windows.h>
//#include <AtlBase.h>
#endif

extern bool use280Config;

//! generate a supervisory C++ file
wxString CppGenerator::generateSupervisoryFile(wxString commands)
{
	wxString toReturn;

	toReturn.Append(generateHeaderAndInit());
	toReturn.Append(generateSupervisoryFunction(commands, wxT("Execute")));
	toReturn.Append(generateMainSupervisory());
	return toReturn;
}

//! generate a realtime C++ file
wxString CppGenerator::generateRealTimeFile(
	wxString before_commands, wxString after_commands, wxString rt_prepare, wxString rt_start, std::vector<wxTextCtrl *>* rt_cv,
	std::vector<wxTextCtrl *>* rt_cpg, wxTextCtrl* rt_t_condition, wxString file_path, bool* flags)
{
	wxString toReturn;

	toReturn.Append(generateHeaderAndInit());
	toReturn.Append(generateSupervisoryFunction(before_commands, wxT("Before")));
	toReturn.Append(generatePrepareRT(rt_prepare, file_path));
	toReturn.Append(generateRunRT(rt_start, rt_cv, rt_cpg, rt_t_condition, flags));
	toReturn.Append(generateSupervisoryFunction(after_commands, wxT("After")));
	toReturn.Append(generateMainRealTime());

	return toReturn;
}

#ifndef LINUX

// Special functions required in Windows to eliminate spaces from paths

wxString getShortPathName(wxString path)
{
	char shortPath[512];
	GetShortPathName((LPCTSTR)path.mb_str(), shortPath, 512);

	wxString short_path(shortPath, wxConvUTF8);

	return short_path;
}

wxString getShortPathFileName(wxString name)
{
	wxString name_only = name.AfterLast('\\');
	//wxString path_only = name.BeforeLast('\\');

	//return getShortPathName(path_only) + wxT('\\') + name_only;
	return name_only;
}

#endif

//! generate a makefile
wxString CppGenerator::generateMakefile(wxString name)
{
	char buf[1024];

	//! gets the current directory (from where the program was run)
#ifndef LINUX
	_getcwd(buf, 1024);
#else
	getcwd(buf, 1024);
#endif

	wxString library_path(buf, wxConvUTF8);

	// Assume user is running from bhcontrol, bhcontrol/bin, bhcontrol/bin/debug, or bhcontrol/bin/release directory and strip it off

#ifdef LINUX
	if (library_path.EndsWith(wxT("/Debug")) || library_path.EndsWith(wxT("/Release")))
		library_path = library_path.BeforeLast('/');
	if (library_path.EndsWith(wxT("/bin")))
		library_path = library_path.BeforeLast('/');

	library_path = library_path.BeforeLast('/'); // Strip "bhcontrol"
#else
	if (library_path.EndsWith(wxT("\\Debug")) || library_path.EndsWith(wxT("\\Release")))
		library_path = library_path.BeforeLast('\\');
	if (library_path.EndsWith(wxT("\\bin")))
		library_path = library_path.BeforeLast('\\');

	library_path = library_path.BeforeLast('\\'); // Strip "bhcontrol"

	library_path = getShortPathName(library_path);
	name = getShortPathFileName(name);

#endif

	wxPuts(library_path);

	wxString header_path = wxT("BHand.h");

	wxString toReturn;

	//! compile the generated .cpp file with the bhand library and link
	toReturn.Append(wxT("#######################################################################\n"));
	toReturn.Append(wxT("# Instuctions on including driver dependencies\n"));
	toReturn.Append(wxT("#    Serial may be left out (the lines below may be commented out)\n"));
	toReturn.Append(wxT("#    if the API was compiled without support for serial\n"));
	toReturn.Append(wxT("#######################################################################\n"));
	toReturn.Append(wxT("PROGRAM = ") + name + wxT("\n"));
	toReturn.Append(wxT("ifeq (${ARCH},windows)\n"));
	toReturn.Append(wxT("\t#######################################################################\n"));
	toReturn.Append(wxT("\t# Windows\n"));

	toReturn.Append(wxT("\tINCLUDES = -I\"$(POCO_BASE)/Foundation/include\" -I\"../../API/\"\n"));
	toReturn.Append(wxT("\tLIBDIR += -L\"$(POCO_BASE)/lib/MinGW/ia32\" -L\"../../API/lib\"\n"));
	toReturn.Append(wxT("\tCFLAGS = -mthreads -O4\n"));
	toReturn.Append(wxT("\tCFLAGS -= -g\n"));
	toReturn.Append(wxT("\tLDLIBS = -lBHand -lwinmm\n"));
	toReturn.Append(wxT("\tLDFLAGS = -s -Wl,--allow-multiple-definition\n"));
	toReturn.Append(wxT("\tCLEANCMD = del ${PROGRAM}.exe *.o\n"));
	toReturn.Append(wxT("\tDEFS =\n"));

	toReturn.Append(wxT("\t#######################################################################\n"));
	toReturn.Append(wxT("\t# Windows CAN Driver Dependency (must be included)\n"));
	toReturn.Append(wxT("\tPCAN_DIR = ../../ThirdParty/PCAN\n"));
	toReturn.Append(wxT("\tINCLUDES += -I\"$(PCAN_DIR)/include\"\n"));
	toReturn.Append(wxT("\tLDLIBS += -lPcan_usb\n"));

	//toReturn.Append(wxT("\t#######################################################################\n"));
	//toReturn.Append(wxT("\t# Windows Serial Driver Dependency (optional)\n"));
	//toReturn.Append(wxT("\tCTB_DIR = ../../ThirdParty/ctb-0.14\n"));
	//toReturn.Append(wxT("\tINCLUDES += -I\"$(CTB_DIR)/include\"\n"));
	//toReturn.Append(wxT("\tLIBDIR += -L\"$(CTB_DIR)/lib\"\n"));
	//toReturn.Append(wxT("\tLDLIBS += -lctb-0.14\n"));

	toReturn.Append(wxT("\t#######################################################################\n"));
	toReturn.Append(wxT("\t# Last\n"));
	toReturn.Append(wxT("\tLDLIBS += -lPocoFoundation\n"));

/*
#ifndef LINUX
	toReturn.Append(wxT("\tBHAND_DIR = ") + library_path + wxT("\n"));
#else
	// Use a hardcoded path where the software is expected to be installed
	toReturn.Append(wxT("\tBHAND_DIR = C:\\PROGRA~1\\BARRET~1\\BHand\n"));
#endif

	toReturn.Append(wxT("\tCTB_DIR = ${BHAND_DIR}\\ThirdParty\\ctb-0.14\n"));
	toReturn.Append(wxT("\tINCLUDES = -I${CTB_DIR}\\include -I\"${POCO_BASE}\\Foundation\\include\" -I${BHAND_DIR}\\API\n"));
	toReturn.Append(wxT("\tLIBDIR = -L${CTB_DIR}\\lib -L\"${POCO_BASE}\\lib\\MinGW\\ia32\"\n"));
	toReturn.Append(wxT("\tCFLAGS = -mthreads -O4\n"));
	toReturn.Append(wxT("\tCFLAGS -= -g\n"));
	toReturn.Append(wxT("\tLDLIBS = -lctb-0.14 -lPocoFoundation\n"));
	toReturn.Append(wxT("\tLDFLAGS = -s -Wl,--allow-multiple-definition\n"));
	toReturn.Append(wxT("\tCLEANCMD = del ${PROGRAM}.exe *.o\n"));
	toReturn.Append(wxT("\tDEFS = \n"));*/




	toReturn.Append(wxT("else \n"));

	toReturn.Append(wxT("\t#######################################################################\n"));
	toReturn.Append(wxT("\t# Leave these commented to use installed headers and BHand library\n"));
	toReturn.Append(wxT("\t#INCLUDES += -I../../API\n"));
	toReturn.Append(wxT("\t#LIBDIR += -L../../API/lib\n"));

	toReturn.Append(wxT("\t#######################################################################\n"));
	toReturn.Append(wxT("\t# Linux (Required)\n"));
	toReturn.Append(wxT("\tBHAND_INSTALL_DIR = /usr/local/Barrett/BHand\n"));
#ifdef LINUX
	toReturn.Append(wxT("\t#BHAND_DIR = ") + library_path + wxT("\n"));
#else
	// Use a hardcoded path where the software is expected to be installed
	toReturn.Append(wxT("\t#BHAND_DIR = ${BHAND_INSTALL_DIR}\n"));
#endif

	toReturn.Append(wxT("\tINCLUDES += -I${BHAND_INSTALL_DIR}/thirdparty/include -I${BHAND_INSTALL_DIR}/API\n"));
	toReturn.Append(wxT("\tLIBDIR += -L${BHAND_INSTALL_DIR}/API/lib\n"));
	toReturn.Append(wxT("\tLDLIBS = -lBHand\n"));
	//toReturn.Append(wxT("\tLDLIBS = -lctb-barrett-0.14 -lPocoFoundationBarrett\n"));
	toReturn.Append(wxT("\tCLEANCMD = rm -f ${PROGRAM} *.o\n"));
	toReturn.Append(wxT("\tDEFS = -DLINUX \n"));

	toReturn.Append(wxT("\t#######################################################################\n"));
	toReturn.Append(wxT("\t# Linux CAN Driver Dependency (must be included)\n"));
	toReturn.Append(wxT("\tINCLUDES += -I../../thirdparty/peak-linux-driver-6.20/driver\n"));
	toReturn.Append(wxT("\tINCLUDES += -I../../thirdparty/peak-linux-driver-6.20/lib\n"));
	toReturn.Append(wxT("\tLDLIBS += -lpcan\n"));

	//toReturn.Append(wxT("\t#######################################################################\n"));
	//toReturn.Append(wxT("\t# Linux Serial Driver Dependency (optional)\n"));
	//toReturn.Append(wxT("\tLDLIBS += -lctb-barrett-0.14\n"));

	toReturn.Append(wxT("\t#######################################################################\n"));
	toReturn.Append(wxT("\t# Last\n"));
	toReturn.Append(wxT("\tLDLIBS += -lPocoFoundationBarrett\n"));

	toReturn.Append(wxT("endif \n"));
	toReturn.Append(wxT("\n"));

	toReturn.Append(wxT("default: ${PROGRAM}.o\n"));
	toReturn.Append(wxT("\tg++ ${CFLAGS} ${PROGRAM}.o ${INCLUDES} ${LDFLAGS} ${LIBDIR} ${LDLIBS} -o ${PROGRAM}\n"));
	toReturn.Append(wxT("\n\n"));

	toReturn.Append(wxT("${PROGRAM}.o: ${PROGRAM}.cpp\n"));
	//toReturn.Append(wxT("\tg++ ${CFLAGS} ${DEFS} -I${BHAND_DIR}/API ${INCLUDES} -c ${PROGRAM}.cpp\n"));
	toReturn.Append(wxT("\tg++ ${CFLAGS} ${DEFS} ${INCLUDES} -c ${PROGRAM}.cpp\n"));
	toReturn.Append(wxT("\n"));
	//toReturn.Append(wxT("bhand.o: ${BHAND_DIR}/API/bhand.cpp\n"));
	//toReturn.Append(wxT("\tg++ ${CFLAGS} ${DEFS} ${INCLUDES} -c ${BHAND_DIR}/API/bhand.cpp\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT(".PHONY: clean \n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("clean: \n"));
	toReturn.Append(wxT("\t${CLEANCMD} \n"));
	toReturn.Append(wxT("\n"));

	return toReturn;
}

//! makes the "header" of the generated file
wxString CppGenerator::generateHeaderAndInit()
{
	char buf[1024];

#ifndef LINUX
	_getcwd(buf, 1024);
#else
	getcwd(buf, 1024);
#endif

	wxString library_path(buf, wxConvUTF8);

	if (library_path.EndsWith(wxT("/Debug")) || library_path.EndsWith(wxT("/Release")))
		library_path = library_path.BeforeLast('/');
	if (library_path.EndsWith(wxT("/bin")))
		library_path = library_path.BeforeLast('/');

	wxString toReturn;
	toReturn.Append(wxT("///////////////////////////////////////////////////////////\n"));
	toReturn.Append(wxT("//                                                       //\n"));
	toReturn.Append(wxT("//           Automatically Generated C++ Code            //\n"));
	toReturn.Append(wxT("//           BHand Control Center Version 1.0            //\n"));
	toReturn.Append(wxT("//                                                       //\n"));
	toReturn.Append(wxT("//                                                       //\n"));
	toReturn.Append(wxT("//                                                       //\n"));
	toReturn.Append(wxT("///////////////////////////////////////////////////////////\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("#include <stdio.h>\n"));
	toReturn.Append(wxT("#include <stdlib.h>\n"));
	toReturn.Append(wxT("#include <math.h>\n"));
	toReturn.Append(wxT("#include <string.h>\n"));
	toReturn.Append(wxT("#include <deque>\n"));
	toReturn.Append(wxT("#include \"BHand.h\"\n"));
	toReturn.Append(wxT("#include \"BHandAppHelper.h\" // for connection menu\n"));
	toReturn.Append(wxT("#include <sys/time.h>// Needed for ms time.\n"));
	toReturn.Append(wxT("#include <time.h>// Needed for ms time.\n"));
	toReturn.Append(wxT("using namespace std;\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("BHand  bh;           // Handles all hand communication\n\n"));
	//toReturn.Append(wxT("int com_port;\n\n"));
	toReturn.Append(wxT("char   buf[100];     // Buffer for reading back hand parameters\n"));
	toReturn.Append(wxT("int    value;        // Some commands use an int* instead of \n"));
	toReturn.Append(wxT("                     // a buffer to relay information\n\n"));
	toReturn.Append(wxT("int    result;       // Return value (error) of all BHand calls\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("///////////////////////////////////////////////////////////\n"));
	toReturn.Append(wxT("// Error Handler - called whenever result != 0           //\n"));
	toReturn.Append(wxT("// Feel free to customize this function                  //\n"));
	toReturn.Append(wxT("///////////////////////////////////////////////////////////\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("void Error()\n"));
	toReturn.Append(wxT("{\n"));
	toReturn.Append(wxT("\tprintf(\"ERROR: %d\\n%s\\n\", result, bh.ErrorMessage(result));\n"));
	toReturn.Append(wxT("\texit(0);\n"));
	toReturn.Append(wxT("}\n\n"));

	// probably not ok
	toReturn.Append(wxT("\n// get the difference in time between two timevals (difference in ms)\n"));
	toReturn.Append(wxT("double diffclock(timeval* currentTime, timeval* startTime)\n"));
	toReturn.Append(wxT("{\n"));
	toReturn.Append(wxT("\tif (((currentTime->tv_usec)/1000. - (startTime->tv_usec)/1000.) < 0.0) {\n") );
	toReturn.Append(wxT("\t\treturn ((1000.0 + (currentTime->tv_usec)/1000.) - (startTime->tv_usec)/1000.);\n") );
	toReturn.Append(wxT("\t} else {\n") );
	toReturn.Append(wxT("\t\treturn ((currentTime->tv_usec)/1000. - (startTime->tv_usec)/1000.);\n") );
	toReturn.Append(wxT("\t}\n") );
	toReturn.Append(wxT("}\n\n\n") );

	toReturn.Append(wxT("///////////////////////////////////////////////////////////\n"));
	toReturn.Append(wxT("//  Initialize hand, set timeouts and baud rate          //\n"));
	toReturn.Append(wxT("///////////////////////////////////////////////////////////\n"));
	toReturn.Append(wxT("void Initialize()\n"));
	toReturn.Append(wxT("{\n"));


	toReturn.Append(wxT("\t// Set hardware description before initialization\n"));
	if (use280Config)
	{
		toReturn.Append(wxT("\t//int hwIndex = BHandHardware::getBHandHardwareIndex(\"BH8-262\");\n"));
		toReturn.Append(wxT("\tint hwIndex = BHandHardware::getBHandHardwareIndex(\"BH8-280\");\n"));
	}
	else
	{
		toReturn.Append(wxT("\tint hwIndex = BHandHardware::getBHandHardwareIndex(\"BH8-262\");\n"));
		toReturn.Append(wxT("\t//int hwIndex = BHandHardware::getBHandHardwareIndex(\"BH8-280\");\n"));
	}
	toReturn.Append(wxT("\tif (hwIndex < 0)\n"));
	toReturn.Append(wxT("\t{\n"));
	toReturn.Append(wxT("\t\tprintf(\"\\n\\nThe API has not been compiled to include target hand.\\n\");\n"));
	toReturn.Append(wxT("\t\t	Error();\n"));
	toReturn.Append(wxT("\t}\n"));

	toReturn.Append(wxT("\tbh.setHardwareDesc(hwIndex);\n"));

	toReturn.Append(wxT("\tbool use280Config = (strcmp(bh.getHardwareDesc()->getModelNumber(), \"BH8-280\") == 0);\n"));
	toReturn.Append(wxT("\t//printf(\"\\nuse280Config = %d\\n\", use280Config);\n"));


	toReturn.Append(wxT("\tif (result = handInitWithMenu(&bh))\n"));
	toReturn.Append(wxT("\t\tError();\n"));

	toReturn.Append(wxString::Format(wxT("\t//if (result = bh.InitSoftware(com_port, THREAD_PRIORITY_TIME_CRITICAL))\n")));
	toReturn.Append(wxT("\t//\tError();\n"));
	toReturn.Append(wxT("\n"));
	//toReturn.Append(wxT("\tif (result = bh.ComSetTimeouts(100, 15000, 100, 5000))\n"));
	//toReturn.Append(wxT("\t\tError();\n"));
	//toReturn.Append(wxT("\n"));
	//toReturn.Append(wxT("\tif (result = bh.Baud(9600))\n"));
	//toReturn.Append(wxT("\t\tError();\n"));
	//toReturn.Append(wxT("\n"));

	toReturn.Append(wxT("\tprintf(\"Initialization...\"); \n"));
	toReturn.Append(wxT("\tif (result = bh.InitHand(\"\"))\n"));
	toReturn.Append(wxT("\t\tError();\n"));
	toReturn.Append(wxT("\telse\n"));
	toReturn.Append(wxT("\t\tprintf(\" Done\\n\");\n"));

	toReturn.Append(wxT("}\n\n\n"));

	toReturn.Append(wxT("///////////////////////////////////////////////////////////\n"));
	toReturn.Append(wxT("//  Loads a file to be run in realtime mode              //\n"));
	toReturn.Append(wxT("///////////////////////////////////////////////////////////\n"));
	toReturn.Append(wxT("void LoadFile(char *toLoad, deque<float*> *file_data)\n"));
	toReturn.Append(wxT("{\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("\tFILE* fp = fopen(toLoad, \"rt\");\n"));
	toReturn.Append(wxT("\tif (!fp)\n"));
	toReturn.Append(wxT("\t\treturn;\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("\tint v1,v2,v3,v4;\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("\twhile (fscanf(fp, \"%d %d %d %d\", &v1, &v2, &v3, &v4) != EOF)\n"));
	toReturn.Append(wxT("{\n"));
	toReturn.Append(wxT("\t\tfloat* values = new float[4];\n"));
	toReturn.Append(wxT("\t\tvalues[0] = v1;\n"));
	toReturn.Append(wxT("\t\tvalues[1] = v2;\n"));
	toReturn.Append(wxT("\t\tvalues[2] = v3;\n"));
	toReturn.Append(wxT("\t\tvalues[3] = v4;\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("\t\tfile_data->push_back(values);\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("\t}\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("\tfclose(fp);\n"));
	toReturn.Append(wxT("}\n\n\n"));

	return toReturn;
}

//! makes a block of supervisory code
wxString CppGenerator::generateSupervisoryFunction(wxString commands, wxString function_name)
{
	wxString toReturn;


	toReturn.Append(wxT("///////////////////////////////////////////////////////////\n"));
	toReturn.Append(wxT("//  Execute commands, return 1 if interrupted with a key //\n"));
	toReturn.Append(wxT("///////////////////////////////////////////////////////////\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("int ") + function_name + wxT("()\n"));
	toReturn.Append(wxT("{\n"));
	toReturn.Append(wxT("\tprintf(\"") + function_name + wxT(": Press Any Key to Abort...\");\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("\n"));

	//! loop through commands and add them to the string
	while (!commands.IsEmpty())
	{
		wxString currentLine = commands.BeforeFirst('\n');
		currentLine.Trim(); // remove carriage return or any other white space at the end of the current line
		//if (currentLine.Contains(wxT("\r")))
		//	currentLine = currentLine.BeforeFirst('\r');

		if (commands.Contains(wxT("(")) && commands.Contains(wxT(")")))
		{
			toReturn.Append(wxT("\tif (result = bh.") + currentLine + wxT(")\n"));
			toReturn.Append(wxT("\t\tError();\n"));
		}
		else
		{
			currentLine = wxT("Command(\"") + currentLine + wxT("\")");
			toReturn.Append(wxT("\tif (result = bh.") + currentLine + wxT(")\n"));
			toReturn.Append(wxT("\t\tError();\n"));
		}

		toReturn.Append(wxT("\tif (UnbufferedGetChar() != EOF)\n"));
		toReturn.Append(wxT("\t\treturn 1;\n\n"));

		commands = commands.AfterFirst('\n');
	}

	toReturn.Append(wxT("\treturn 0;\n}\n\n"));

	return toReturn;
}

wxString CppGenerator::generatePrepareRT(wxString function, wxString file_path)
{
	// prepare RT mode and set the data buffer to contain the file to be loaded
	wxString toReturn;

	toReturn.Append(wxT("///////////////////////////////////////////////////////////\n"));
	toReturn.Append(wxT("//  Set parameters, prepare data buffers                 //\n"));
	toReturn.Append(wxT("///////////////////////////////////////////////////////////\n"));

	toReturn.Append(wxT("void PrepareRealTime()\n"));
	toReturn.Append(wxT("{\n"));
	toReturn.Append(wxT("\tif (result = bh.") + function + wxT(")\n"));
	toReturn.Append(wxT("\t\tError();\n"));

	//file_path.BeforeFirst('\n');
	//file_path.Trim();

	toReturn.Append(wxT("\tsprintf(buf, \"%s\", \"") + file_path + wxT("\");\n"));
	toReturn.Append(wxT("}\n\n"));

	return toReturn;
}

wxString CppGenerator::generateRunRT(wxString function, std::vector<wxTextCtrl *>* rt_cv, std::vector<wxTextCtrl *>* rt_cpg, wxTextCtrl* rt_t_condition, bool* flags)
{
	// run realtime mode
	// substitute values of expressions with corresponding values from var matrix
	wxString toReturn;

	toReturn.Append(wxT("///////////////////////////////////////////////////////////\n") );
	toReturn.Append(wxT("// Run RealRime loop, return 1 if interrupted with a key //\n") );
	toReturn.Append(wxT("///////////////////////////////////////////////////////////\n") );

	toReturn.Append(wxT("int RunRealTime()\n"));
	toReturn.Append(wxT("{\n"));
	toReturn.Append(wxT("\tdouble var[4][7];\n"));
	toReturn.Append(wxT("\tint N=0;\n"));
	toReturn.Append(wxT("\tunsigned int time = 0, tmstart;\n"));
	toReturn.Append(wxT("\tbool terminate = false;\n"));
	toReturn.Append(wxT("\tint pos, m;\n\n"));

	toReturn.Append(wxT("\tstruct timezone m_tz;\n"));
	toReturn.Append(wxT("\tstruct timeval startTime;\n"));
	toReturn.Append(wxT("\tstruct timeval currentTime;\n\n"));

	toReturn.Append(wxT("\tgettimeofday(&startTime, &m_tz);\n"));

	toReturn.Append(wxT("\tdeque<float*> file_data;\n"));
	toReturn.Append(wxT("\tLoadFile(buf, &file_data);\n"));
	toReturn.Append(wxT("\tbh.") + function + wxT(";\n"));
	toReturn.Append(wxT("\tbh.RTUpdate();\n"));

	toReturn.Append(wxT("\tprintf(\"Press Any Key to Abort...\");\n"));
	toReturn.Append(wxT("\twhile(!terminate && UnbufferedGetChar() == EOF)\n"));
	toReturn.Append(wxT("\t{\n\n"));

	toReturn.Append(wxT("\t\tgettimeofday(&currentTime, &m_tz);\n"));
	toReturn.Append(wxT("\t\ttime += (unsigned int)diffclock(&currentTime,&startTime);\n"));
	toReturn.Append(wxT("\t\tgettimeofday(&startTime, &m_tz);\n\n"));

	toReturn.Append(wxT("\t\tfor (int m = 0; m < 4; m++)\n"));
	toReturn.Append(wxT("\t\t{\n"));
	toReturn.Append(wxT("\t\t\tvar[m][0] = bh.RTGetDeltaPos(m + '1');\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("\t\t\tif (!file_data.empty())\n"));
	toReturn.Append(wxT("\t\t\t{\n"));
	toReturn.Append(wxT("\t\t\tvar[m][1] = ((file_data[N % file_data.size()])[m]);   /* file data */\n"));
	toReturn.Append(wxT("\t\t}\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("\t\tvar[m][2] = (double)N;\n"));
	toReturn.Append(wxT("\t\tvar[m][3] = bh.RTGetPosition(m + '1');\n"));
	toReturn.Append(wxT("\t\tvar[m][4] = bh.RTGetStrain(m + '1');\n"));
	toReturn.Append(wxT("\t\tvar[m][5] = (double)time;\n"));
	toReturn.Append(wxT("\t\tvar[m][6] = bh.RTGetVelocity(m + '1');\n"));
	toReturn.Append(wxT("\t}\n\n"));

	// substitute variables out
	for (unsigned int i = 0; i < 4; i++)
	{
		if(flags[i])
		{
			wxString temp = (rt_cv->at(i))->GetValue();
			temp.Replace(wxT("\r"), wxT(""));
			temp.Replace(wxT("\n"), wxT(""));

			for (unsigned int m = 0; m < 4; m++)
			{
				temp.Replace(wxString::Format(wxT("D%i"), (m+1)), wxString::Format(wxT(" var[%i][0]"), m));
				temp.Replace(wxString::Format(wxT("d%i"), (m+1)), wxString::Format(wxT(" var[%i][0]"), m));
				temp.Replace(wxString::Format(wxT("F%i"), (m+1)), wxString::Format(wxT(" var[%i][1]"), m));
				temp.Replace(wxString::Format(wxT("f%i"), (m+1)), wxString::Format(wxT(" var[%i][1]"), m));
				temp.Replace(wxString::Format(wxT("N%i"), (m+1)), wxString::Format(wxT(" var[%i][2]"), m));
				temp.Replace(wxString::Format(wxT("n%i"), (m+1)), wxString::Format(wxT(" var[%i][2]"), m));
				temp.Replace(wxString::Format(wxT("P%i"), (m+1)), wxString::Format(wxT(" var[%i][3]"), m));
				temp.Replace(wxString::Format(wxT("p%i"), (m+1)), wxString::Format(wxT(" var[%i][3]"), m));
				temp.Replace(wxString::Format(wxT("S%i"), (m+1)), wxString::Format(wxT(" var[%i][4]"), m));
				temp.Replace(wxString::Format(wxT("s%i"), (m+1)), wxString::Format(wxT(" var[%i][4]"), m));
				temp.Replace(wxString::Format(wxT("T%i"), (m+1)), wxString::Format(wxT(" var[%i][5]"), m));
				temp.Replace(wxString::Format(wxT("t%i"), (m+1)), wxString::Format(wxT(" var[%i][5]"), m));
				temp.Replace(wxString::Format(wxT("V%i"), (m+1)), wxString::Format(wxT(" var[%i][6]"), m));
				temp.Replace(wxString::Format(wxT("v%i"), (m+1)), wxString::Format(wxT(" var[%i][6]"), m));
			}

			temp.Replace(wxT("D"), wxT(" var[0][0]"));
	//            temp.Replace(wxT("d"), wxT("var[0][0]"));
			temp.Replace(wxT("F"), wxT(" var[0][1]"));
	//            temp.Replace(wxT("f"), wxT("var[0][1]"));
			temp.Replace(wxT("N"), wxT(" var[0][2]"));
	//            temp.Replace(wxT("n"), wxT("var[0][2]"));
			temp.Replace(wxT("P"), wxT(" var[0][3]"));
	//            temp.Replace(wxT("p"), wxT("var[0][3]"));
			temp.Replace(wxT("S"), wxT(" var[0][4]"));
	//            temp.Replace(wxT("s"), wxT("var[0][4]"));
			temp.Replace(wxT("T"), wxT(" var[0][5]"));
	//            temp.Replace(wxT("t"), wxT("var[0][5]"));
			temp.Replace(wxT("V"), wxT(" var[0][6]"));
	//            temp.Replace(wxT("v"), wxT("var[0][6]"));

			toReturn.Append(wxT("\t\tvalue = (int)(") + temp + wxT(");\n"));
			toReturn.Append(wxString::Format(wxT("\t\tbh.RTSetVelocity('%i', value);\n"), (i+1)));

		}
	}

	//! substitute variables out
	for (unsigned int i = 0; i < 4; i++)
	{
		if (flags[i+4])
		{
			wxString temp = (rt_cpg->at(i))->GetValue();

			temp.Replace(wxT("\r"), wxT(""));
			temp.Replace(wxT("\n"), wxT(""));

			for(unsigned int m = 0; m < 4; m++) {
				temp.Replace(wxString::Format(wxT("D%i"), (m+1)), wxString::Format(wxT(" var[%i][0]"), m));
				temp.Replace(wxString::Format(wxT("d%i"), (m+1)), wxString::Format(wxT(" var[%i][0]"), m));
				temp.Replace(wxString::Format(wxT("F%i"), (m+1)), wxString::Format(wxT(" var[%i][1]"), m));
				temp.Replace(wxString::Format(wxT("f%i"), (m+1)), wxString::Format(wxT(" var[%i][1]"), m));
				temp.Replace(wxString::Format(wxT("N%i"), (m+1)), wxString::Format(wxT(" var[%i][2]"), m));
				temp.Replace(wxString::Format(wxT("n%i"), (m+1)), wxString::Format(wxT(" var[%i][2]"), m));
				temp.Replace(wxString::Format(wxT("P%i"), (m+1)), wxString::Format(wxT(" var[%i][3]"), m));
				temp.Replace(wxString::Format(wxT("p%i"), (m+1)), wxString::Format(wxT(" var[%i][3]"), m));
				temp.Replace(wxString::Format(wxT("S%i"), (m+1)), wxString::Format(wxT(" var[%i][4]"), m));
				temp.Replace(wxString::Format(wxT("s%i"), (m+1)), wxString::Format(wxT(" var[%i][4]"), m));
				temp.Replace(wxString::Format(wxT("T%i"), (m+1)), wxString::Format(wxT(" var[%i][5]"), m));
				temp.Replace(wxString::Format(wxT("t%i"), (m+1)), wxString::Format(wxT(" var[%i][5]"), m));
				temp.Replace(wxString::Format(wxT("V%i"), (m+1)), wxString::Format(wxT(" var[%i][6]"), m));
				temp.Replace(wxString::Format(wxT("v%i"), (m+1)), wxString::Format(wxT(" var[%i][6]"), m));
			}

			temp.Replace(wxT("D"), wxT(" var[0][0]"));
	//            temp.Replace(wxT("d"), wxT("var[0][0]"));
			temp.Replace(wxT("F"), wxT(" var[0][1]"));
	//            temp.Replace(wxT("f"), wxT("var[0][1]"));
			temp.Replace(wxT("N"), wxT(" var[0][2]"));
	//            temp.Replace(wxT("n"), wxT("var[0][2]"));
			temp.Replace(wxT("P"), wxT(" var[0][3]"));
	//            temp.Replace(wxT("p"), wxT("var[0][3]"));
			temp.Replace(wxT("S"), wxT(" var[0][4]"));
	//            temp.Replace(wxT("s"), wxT("var[0][4]"));
			temp.Replace(wxT("T"), wxT(" var[0][5]"));
	//            temp.Replace(wxT("t"), wxT("var[0][5]"));
			temp.Replace(wxT("V"), wxT(" var[0][6]"));
	//            temp.Replace(wxT("v"), wxT("var[0][6]"));

			toReturn.Append( wxT("\t\tvalue = ") + temp + wxT(";\n"));
			toReturn.Append( wxString::Format(wxT("\t\tbh.RTSetGain('%i', value);\n"), (i+1)) );
		}
	}

	//! substitute variables out
	if (rt_t_condition != NULL)
	{
			wxString temp = rt_t_condition->GetValue();

			temp.Replace(wxT("\r"), wxT(""));
			temp.Replace(wxT("\n"), wxT(""));

			for (unsigned int m = 0; m < 4; m++) {
				temp.Replace(wxString::Format(wxT("D%i"), (m+1)), wxString::Format(wxT(" var[%i][0]"), m));
				temp.Replace(wxString::Format(wxT("d%i"), (m+1)), wxString::Format(wxT(" var[%i][0]"), m));
				temp.Replace(wxString::Format(wxT("F%i"), (m+1)), wxString::Format(wxT(" var[%i][1]"), m));
				temp.Replace(wxString::Format(wxT("f%i"), (m+1)), wxString::Format(wxT(" var[%i][1]"), m));
				temp.Replace(wxString::Format(wxT("N%i"), (m+1)), wxString::Format(wxT(" var[%i][2]"), m));
				temp.Replace(wxString::Format(wxT("n%i"), (m+1)), wxString::Format(wxT(" var[%i][2]"), m));
				temp.Replace(wxString::Format(wxT("P%i"), (m+1)), wxString::Format(wxT(" var[%i][3]"), m));
				temp.Replace(wxString::Format(wxT("p%i"), (m+1)), wxString::Format(wxT(" var[%i][3]"), m));
				temp.Replace(wxString::Format(wxT("S%i"), (m+1)), wxString::Format(wxT(" var[%i][4]"), m));
				temp.Replace(wxString::Format(wxT("s%i"), (m+1)), wxString::Format(wxT(" var[%i][4]"), m));
				temp.Replace(wxString::Format(wxT("T%i"), (m+1)), wxString::Format(wxT(" var[%i][5]"), m));
				temp.Replace(wxString::Format(wxT("t%i"), (m+1)), wxString::Format(wxT(" var[%i][5]"), m));
				temp.Replace(wxString::Format(wxT("V%i"), (m+1)), wxString::Format(wxT(" var[%i][6]"), m));
				temp.Replace(wxString::Format(wxT("v%i"), (m+1)), wxString::Format(wxT(" var[%i][6]"), m));
			}

			temp.Replace(wxT("D"), wxT(" var[0][0]"));
	//            temp.Replace(wxT("d"), wxT("var[0][0]"));
			temp.Replace(wxT("F"), wxT(" var[0][1]"));
	//            temp.Replace(wxT("f"), wxT("var[0][1]"));
			temp.Replace(wxT("N"), wxT(" var[0][2]"));
	//            temp.Replace(wxT("n"), wxT("var[0][2]"));
			temp.Replace(wxT("P"), wxT(" var[0][3]"));
	//            temp.Replace(wxT("p"), wxT("var[0][3]"));
			temp.Replace(wxT("S"), wxT(" var[0][4]"));
	//            temp.Replace(wxT("s"), wxT("var[0][4]"));
			temp.Replace(wxT("T"), wxT(" var[0][5]"));
	//            temp.Replace(wxT("t"), wxT("var[0][5]"));
			temp.Replace(wxT("V"), wxT(" var[0][6]"));
	//            temp.Replace(wxT("v"), wxT("var[0][6]"));

			toReturn.Append(wxT("\t\tterminate = ") + temp + wxT(";\n"));

	} else {
		toReturn.Append(wxT("\t\tterminate = 0;\n"));
	}

	toReturn.Append(wxT("\t\tN++;\n"));
	toReturn.Append(wxT("\t\tbh.RTUpdate();\n"));
	toReturn.Append(wxT("\t}\n"));

	toReturn.Append(wxT("\tfile_data.clear();\n"));
	toReturn.Append(wxT("\tbh.RTAbort();\n\n"));

	toReturn.Append(wxT("\t\tif (UnbufferedGetChar() != EOF)\n"));
	toReturn.Append(wxT("\t\t\treturn 1;\n"));
	toReturn.Append(wxT("\t\telse\n"));
	toReturn.Append(wxT("\t\t\treturn 0;\n"));
	toReturn.Append(wxT("}\n"));

	return toReturn;
}


wxString CppGenerator::generateMainSupervisory()
{
	// makes the main function
	wxString toReturn;

	toReturn.Append(wxT("int main()\n"));
	toReturn.Append(wxT("{\n") );

	toReturn.Append(wxT("\t/* Turn off output buffering for printf() */\n"));
	toReturn.Append(wxT("\tsetvbuf(stdout, 0, _IONBF, 0);\n\n"));
	toReturn.Append(wxT("\t/* Turn on unbuffered input */\n"));
	toReturn.Append(wxT("\tUnbufferedInputStart();\n\n"));

	toReturn.Append(wxT("\tprintf(\"\\n\\n\\r\\t\\tInitializing Software...\");\n"));
	//toReturn.Append(wxT("\tprintf(\"\\n\\n\\r\\t\\tPlease enter desired serial port> \");\n"));
	//toReturn.Append(wxT("\tscanf(\"%d\", &com_port);\n"));;
	//toReturn.Append(wxT("\tprintf(\"\\n\");\n\n"));

	toReturn.Append(wxT("\tInitialize();\n"));
	toReturn.Append(wxT("\tprintf(\"Executing - \");\n"));
	toReturn.Append(wxT("\tExecute();\n"));
	toReturn.Append(wxT("\tprintf(\" Done\\n\");\n\n"));

	toReturn.Append(wxT("\t/* Turn off unbuffered input */\n"));
	toReturn.Append(wxT("\tUnbufferedInputEnd();\n"));

	toReturn.Append(wxT("\treturn 0;\n"));
	toReturn.Append(wxT("}\n"));

	return toReturn;
}


wxString CppGenerator::generateMainRealTime()
{
	// makes the main function
	wxString toReturn;

	toReturn.Append(wxT("int main()\n"));
	toReturn.Append(wxT("{\n"));

	toReturn.Append(wxT("\t// Turn off output buffering for printf()\n"));
	toReturn.Append(wxT("\tsetvbuf(stdout,0,_IONBF,0);\n\n"));
	toReturn.Append(wxT("\t// Turn on unbuffered input\n"));
	toReturn.Append(wxT("\tUnbufferedInputStart();\n\n"));

	toReturn.Append(wxT("\tprintf(\"\\n\\n\\r\\t\\tInitializing Software...\");\n"));
	//toReturn.Append(wxT("\tprintf(\"\\n\\n\\r\\t\\tPlease enter desired serial port> \");"));
	//toReturn.Append(wxT("\tscanf(\"%d\", &com_port);"));;
	//toReturn.Append(wxT("\tprintf(\"\\n\");\n\n"));

	//toReturn.Append(wxT("\tprintf(\"Initialization...\");\n"));
	toReturn.Append(wxT("\tInitialize();\n"));
	//toReturn.Append(wxT("\tprintf(\" Done\\n\");\n"));
	toReturn.Append(wxT("\t\n"));
	toReturn.Append(wxT("\tprintf(\"Before - \");\n"));
	toReturn.Append(wxT("\tif (Before())\n"));
	toReturn.Append(wxT("\t\treturn 1;\n"));
	toReturn.Append(wxT("\tprintf(\" Done\\n\");\n"));
	toReturn.Append(wxT("\t\n"));
	toReturn.Append(wxT("\tPrepareRealTime();\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("\tprintf(\"RealTime Loop - \");\n"));
	toReturn.Append(wxT("\tif (RunRealTime())\n"));
	toReturn.Append(wxT("\t\treturn 1;\n"));
	toReturn.Append(wxT("\tprintf(\" Done\\n\");\n"));
	toReturn.Append(wxT("\n"));
	toReturn.Append(wxT("\tprintf(\"After - \");\n"));
	toReturn.Append(wxT("\tif (After())\n"));
	toReturn.Append(wxT("\t\treturn 1;\n"));
	toReturn.Append(wxT("\tprintf(\" Done\\n\");\n"));


	toReturn.Append(wxT("\t// Turn off unbuffered input\n"));
	toReturn.Append(wxT("\tUnbufferedInputEnd();\n"));

	toReturn.Append(wxT("\treturn 0;\n"));
	toReturn.Append(wxT("}\n"));

	return toReturn;
}
