#ifndef TMR_H
#define TMR_H

#include "stdio.h"

#ifdef LINUX
	#include <sys/time.h>

	typedef struct HiResTimerStruct
	{
		void Start()
		{
			gettimeofday(&mTimeStart, NULL);
		};
		void Stop()
		{
			gettimeofday(&mTimeStop, NULL);
		};
		double GetDurationInSecs()
		{
			double duration = (double)(mTimeStop.tv_sec - mTimeStart.tv_sec) + (double)(mTimeStop.tv_usec - mTimeStart.tv_usec) / 1000000;
			return duration;
		}
		double GetDurationInMilliSecs()
		{
			double duration = (double)(mTimeStop.tv_sec - mTimeStart.tv_sec) * 1000 + (double)(mTimeStop.tv_usec - mTimeStart.tv_usec) / 1000;
			return duration;
		}
		double GetElapsedInMilliSecs()
		{
			struct timeval mTimeNow;
			gettimeofday(&mTimeNow, NULL);
			return (double)(mTimeNow.tv_sec - mTimeStart.tv_sec) * 1000 + (double)(mTimeNow.tv_usec - mTimeStart.tv_usec) / 1000;
		}
		double GetElapsedInSecs()
		{
			struct timeval mTimeNow;
			gettimeofday(&mTimeNow, NULL);
			return (double)(mTimeNow.tv_sec - mTimeStart.tv_sec) + (double)(mTimeNow.tv_usec - mTimeStart.tv_usec) / 1000000;
		}

		struct timeval mTimeStart;
		struct timeval mTimeStop;
	} HiResTimer;


#else
	#include <Windows.h>

	typedef struct HiResTimerStruct
	{
		void Start()
		{
			QueryPerformanceCounter(&mTimeStart);
		};
		void Stop()
		{
			QueryPerformanceCounter(&mTimeStop);
		};
		double GetDurationInSecs()
		{
			LARGE_INTEGER freq;
			QueryPerformanceFrequency(&freq);
			double duration = (double)(mTimeStop.QuadPart - mTimeStart.QuadPart) / (double)freq.QuadPart;
			return duration;
		}
		double GetDurationInMilliSecs()
		{
			LARGE_INTEGER freq;
			QueryPerformanceFrequency(&freq);
			double duration = 1000 * (double)(mTimeStop.QuadPart - mTimeStart.QuadPart) / (double)freq.QuadPart;
			return duration;
		}
		double GetElapsedInMilliSecs()
		{
			LARGE_INTEGER mTimeNow;
			QueryPerformanceCounter(&mTimeNow);
			LARGE_INTEGER freq;
			QueryPerformanceFrequency(&freq);
			return 1000 * (double)(mTimeNow.QuadPart - mTimeStart.QuadPart) / (double)freq.QuadPart;
		}
		double GetElapsedInSecs()
		{
			LARGE_INTEGER mTimeNow;
			QueryPerformanceCounter(&mTimeNow);
			LARGE_INTEGER freq;
			QueryPerformanceFrequency(&freq);
			return (double)(mTimeNow.QuadPart - mTimeStart.QuadPart) / (double)freq.QuadPart;
		}

		LARGE_INTEGER mTimeStart;
		LARGE_INTEGER mTimeStop;
	} HiResTimer;

#endif

#endif
