#ifndef CONFIGURE_H_INCLUDED
#define CONFIGURE_H_INCLUDED


#include "BHand.h"

#include "BHControlMain.h" // needed for access to BHControl GUI control variables

#include <wx/notebook.h>
#include <wx/image.h>

#include <vector>

#include "Bootloader/Bootloader.h"


class wxPanel;
class mpWindow;
class wxStaticText;
class wxChoice;
class mpBitmapLayer;
class wxString;
class wxMessageDialog;

class BHControlFrame;
class BGUIBootloader;
class BootloaderPuck2;
#ifdef BH8_262_HARDWARE
class Bootloader262;
#endif

enum BHInitLibraryState {
	BHInitLibraryIdle = 0,
	BHInitLibraryStart,
	BHInitLibraryWaitForReset,
	BHInitLibraryWaitForVersResult, // CAN only
	BHInitLibraryWaitForPolesResult, // CAN only
	BHInitLibraryWaitForInitHand,
	};

class Config
{
public:
	Config(wxNotebook *controlNotebook, BHand *bh, wxChoice *comportChoice, wxGauge* gauge, wxStaticText* max, wxStaticText* min, wxStaticText* avg, wxStaticText* outin, wxStaticText* firmOH, wxStaticText* helpMSG);
	~Config();

	void setBHControlFrame(BHControlFrame *bhcontrol);

	bool use280Bootloader() { return m_bootloader == (BGUIBootloader*)m_bootloaderPuck2; }

	// Configuration
	bool initialize(int hwIndex);

	void setUseCAN(bool value);
	void setFilePath(wxString path);
	void ResetDefaults();


	// Upload Firmware
	void UploadStart();
	void UploadService();

	UploadState UploadGetState() { return (m_bootloader != NULL) ? m_bootloader->getUploadState() : Upload_Idle; }

	bool IsInitialized() { return m_initialized; }

	// called in BHControlMain.cpp
	void OnBaudRate(wxCommandEvent& event);
	void OnSetConfigDefaults(wxCommandEvent& event);

	void OnTestDelays(wxCommandEvent& event);

private:

	BGUIBootloader *m_bootloader;
#ifdef BH8_262_HARDWARE
	Bootloader262 *m_bootloader262;
#endif
	BootloaderPuck2 *m_bootloaderPuck2;

	bool m_initialized;

	// Methods
	void setBaudRate(int value);

	void DrawHistrogram(float* array, int size);

	void ThrowError(wxString message);


	// Member variables (set in constructor)
	wxNotebook *bhNotebook;
	BHand *m_hand;
	BHControlFrame *m_bhcontrol;

	wxChoice *m_comportChoice;

	wxStaticText *m_max;
	wxStaticText *m_min;
	wxStaticText *m_avg;
	wxStaticText *m_outin;

	wxStaticText *m_firmwareOH;

	// More Member variables
	wxImage box;

	wxPanel *m_panel;
	mpWindow *m_histogram;

	// Configuration Section
	bool m_use_CAN;

	// Init Library State
	BHInitLibraryState initLibraryState;

	// Test Delays Section

	std::vector<mpBitmapLayer *> bars;
};

#endif // CONFIGURE_H_INCLUDED
