#include "NodeCondExpr.h"

NodeCondExpr::NodeCondExpr(NodeExpr* cond, NodeExpr* condTrue, NodeExpr* condFalse) {
    m_left = 0;
    m_right = 0;

    condition = cond;
    m_true = condTrue;
    m_false = condFalse;
}

NodeCondExpr::~NodeCondExpr() {

    if(condition)
        delete condition;
    if(m_true)
        delete m_true;
    if(m_false)
        delete m_false;
}

//! substitute recursively
bool NodeCondExpr::Subst(wxString name, float* val) {

    if(!condition || !m_true || !m_false)
        return false;

    condition->Subst(name, val);
    m_true->Subst(name, val);
    m_false->Subst(name, val);

    return true;
}

//! interpret recursively
float NodeCondExpr::Interp() {

    if(!condition || !m_true || !m_false)
        return 0;

    if(condition->Interp())
        return m_true->Interp();
    else
        return m_false->Interp();
}

//! print recursively
void NodeCondExpr::Print() {

    if(!condition || !m_true || !m_false)
        return;

    wxPuts(wxT("NodeCondExpr: cond-"));
    condition->Print();
    wxPuts(wxT("NodeCondExpr: true-"));
    m_true->Print();
    wxPuts(wxT("NodeCondExpr: false-"));
    m_false->Print();
}

