#include "NodeExpr.h"
#include <cmath>

NodeExpr::NodeExpr(NodeExpr* left, NodeExpr* right, wxString val) {
    m_left = left;
    m_right = right;
    m_val = val;
}

NodeExpr::~NodeExpr() {
    if(m_left != 0)
        delete m_left;
    if(m_right != 0)
        delete m_right;
    m_val.Clear();
}

//! substitute recursively
bool NodeExpr::Subst(wxString name, float* val) {

    if(!m_left || !m_right)
        return false;

    m_left->Subst(name, val);
    m_right->Subst(name, val);

    return true;
}

//! print recursively
void NodeExpr::Print() {
    wxPuts(wxT("NodeExpr: val is: ") + m_val);

    if(!m_left || !m_right)
        return;

    m_left->Print();
    m_right->Print();
}

//! intepret recursively
float NodeExpr::Interp() {

    if(!m_left || !m_right)
        return 0;

    if(m_val.Cmp(wxT("^")) == 0) {
        return pow(m_left->Interp(), m_right->Interp());
    } else if(m_val.Cmp(wxT("*")) == 0) {
        float left = m_left->Interp();
        float right = m_right->Interp();
        return left * right;
    } else if(m_val.Cmp(wxT("/")) == 0) {
        float left = m_left->Interp();
        float right = m_right->Interp();

        if(right < -0.00001 || right > 0.00001) {
            return left / right;
        } else {
            return 0;
        }
    } else if(m_val.Cmp(wxT("%")) == 0) {
        float result = m_right->Interp();
        if(result < -0.00001 || result > 0.00001) {
            return fmod(m_left->Interp(), m_right->Interp());
        } else {
            return 0;
        }
    } else if(m_val.Cmp(wxT("+")) == 0) {
        return m_left->Interp() + m_right->Interp();
    } else if(m_val.Cmp(wxT("-")) == 0) {
        return m_left->Interp() - m_right->Interp();
    } else if(m_val.Cmp(wxT("&&")) == 0) {
        return (bool)(m_left->Interp() != 0.0f && m_right->Interp() != 0.0f);
    } else if(m_val.Cmp(wxT("||")) == 0) {
        return (bool)(m_left->Interp() != 0.0f || m_right->Interp() != 0.0f);
    } else if(m_val.Cmp(wxT(">=")) == 0) {
        return (bool)(m_left->Interp() >= m_right->Interp());
    } else if(m_val.Cmp(wxT("<=")) == 0) {
        return (bool)(m_left->Interp() <= m_right->Interp());
    } else if(m_val.Cmp(wxT("!=")) == 0) {
        return (bool)(m_left->Interp() != m_right->Interp());
    } else if(m_val.Cmp(wxT("<")) == 0) {
        return (bool)(m_left->Interp() < m_right->Interp());
    } else if(m_val.Cmp(wxT(">")) == 0) {
        return (bool)(m_left->Interp() > m_right->Interp());
    } else if(m_val.Cmp(wxT("==")) == 0) {
        return (bool)(m_left->Interp() == m_right->Interp());
    }

    return 0;
}
