#include "NodeTrigExpr.h"
#include <cstdlib>
#include <cmath>
#include "time.h"

using namespace std;

NodeTrigExpr::NodeTrigExpr(NodeExpr* expr, wxString val) {
    m_expr = expr;
    m_func = val;
    m_left = 0;
    m_right = 0;
    srand ( time(NULL) );
}

NodeTrigExpr::~NodeTrigExpr() {
    if(m_expr)
        delete m_expr;
}

//! substitute recursively
bool NodeTrigExpr::Subst(wxString name, float* val) {

    if(!m_expr)
        return false;

    m_expr->Subst(name, val);
    return true;
}

//! print recursively
void NodeTrigExpr::Print() {

    wxPuts(wxT("NodeTrigExpr: val is ") + m_func);
    if(m_expr)
        m_expr->Print();
}

//! interpret recursively
float NodeTrigExpr::Interp() {

    if(!m_expr)
        return 0;

    if(m_func.Cmp(wxT("abs")) == 0) {
        return fabs(m_expr->Interp());
    } else if(m_func.Cmp(wxT("acos")) == 0) {
        return acos(m_expr->Interp());
    } else if(m_func.Cmp(wxT("asin")) == 0) {
        return asin(m_expr->Interp());
    } else if(m_func.Cmp(wxT("atan")) == 0) {
        return atan(m_expr->Interp());
    } else if(m_func.Cmp(wxT("ceil")) == 0) {
        return ceil(m_expr->Interp());
    } else if(m_func.Cmp(wxT("cos")) == 0) {
        return cos(m_expr->Interp());
    } else if(m_func.Cmp(wxT("exp")) == 0) {
        return exp(m_expr->Interp());
    } else if(m_func.Cmp(wxT("floor")) == 0) {
        return floor(m_expr->Interp());
    } else if(m_func.Cmp(wxT("ln")) == 0) {
        return log(m_expr->Interp());
    } else if(m_func.Cmp(wxT("log")) == 0) {
        return log10(m_expr->Interp());
    } else if(m_func.Cmp(wxT("random")) == 0) {
        return (((float) rand()/RAND_MAX) * m_expr->Interp());
    } else if(m_func.Cmp(wxT("round")) == 0) {
        return floor(m_expr->Interp() + 0.5);
    } else if(m_func.Cmp(wxT("sin")) == 0) {
        float expr = m_expr->Interp();
        return sin(expr);
    } else if(m_func.Cmp(wxT("sqrt")) == 0) {
        return sqrt(m_expr->Interp());
    } else if(m_func.Cmp(wxT("tan")) == 0) {
        return tan(m_expr->Interp());
    } else if(m_func.Cmp(wxT("!")) == 0) {
        float result = m_expr->Interp();
        return (result == 0 ? 1 : 0);
    } else if(m_func.Cmp(wxT("-")) == 0) {
        return -1.0 * (m_expr->Interp());
    }

    return 0;
}
