#include "ParserRT.h"

#include "NodeExpr.h"
#include "NodeVar.h"
#include "NodeNum.h"
#include "NodeTrigExpr.h"
#include "NodeCondExpr.h"

#define DEBUG_PARSER false

NodeExpr* ParserRT::Parse(wxString expr, int motor_num) {

    //! remove spaces and \r's \n's
    wxString cleaned = CleanWhiteSpace(&expr);

    wxPuts(cleaned);

    //! allow order of operations using multiple parentheses

    cleaned.Replace(wxT("("), wxT("(((("));
    cleaned.Replace(wxT(")"), wxT("))))"));

    cleaned.Replace(wxT("^"), wxT(")^("));
    cleaned.Replace(wxT("*"), wxT("))*(("));
    cleaned.Replace(wxT("/"), wxT("))/(("));
    cleaned.Replace(wxT("%"), wxT("))%(("));
    cleaned.Replace(wxT("+"), wxT(")))+((("));
    cleaned.Replace(wxT("-"), wxT(")))-((("));

    cleaned = wxT("((((") + cleaned + wxT("))))");

    //! clean all text that has "()" (loop until all instances of "()" are gone
    CleanParentheses(&cleaned);

    if(cleaned.IsEmpty()) {
        return NULL;
    } else {
        return ParseExpression(&cleaned, motor_num);
    }
}

//! LL(1) grammar - (only need to look one spot ahead to parse information)

NodeExpr* ParserRT::ParseExpression(wxString* expr, int motor_num) {

    wxString* toParse = expr;
    bool checkParen = false;

    NodeExpr* toReturn;

//#if DEBUG_PARSER
//    wxPuts(wxT("Here!"));
//#endif

    //! begin evaluating the expression
    //! once a match is found, continue parsing recursively

    if(toParse->StartsWith(wxT("abs("))) {
        *toParse = toParse->AfterFirst('s');
        return new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("abs"));
    } else if(toParse->StartsWith(wxT("acos("))) {
        *toParse = toParse->AfterFirst('s');
        return new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("acos"));
    } else if(toParse->StartsWith(wxT("asin("))) {
        *toParse = toParse->AfterFirst('n');
        return new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("asin"));
    } else if(toParse->StartsWith(wxT("atan("))) {
        *toParse = toParse->AfterFirst('n');
        return new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("atan"));
    } else if(toParse->StartsWith(wxT("ceil("))) {
        *toParse = toParse->AfterFirst('l');
        return new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("ceil"));
    } else if(toParse->StartsWith(wxT("cos("))) {
        *toParse = toParse->AfterFirst('s');
        return new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("cos"));
    } else if(toParse->StartsWith(wxT("exp("))) {
        *toParse = toParse->AfterFirst('p');
        return new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("exp"));
    } else if(toParse->StartsWith(wxT("floor("))) {
        *toParse = toParse->AfterFirst('r');
        return new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("floor"));
    } else if(toParse->StartsWith(wxT("ln("))) {
        *toParse = toParse->AfterFirst('n');
        return new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("ln"));
    } else if(toParse->StartsWith(wxT("log("))) {
        *toParse = toParse->AfterFirst('g');
        return new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("log"));
    } else if(toParse->StartsWith(wxT("random("))) {
        *toParse = toParse->AfterFirst('m');
        return new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("random"));
    } else if(toParse->StartsWith(wxT("round("))) {
        *toParse = toParse->AfterFirst('d');
        return new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("round"));
    } else if(toParse->StartsWith(wxT("sin("))) {
        *toParse = toParse->AfterFirst('n');
        return new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("sin"));
    } else if(toParse->StartsWith(wxT("sqrt("))) {
        *toParse = toParse->AfterFirst('t');
        return new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("sqrt"));
    } else if(toParse->StartsWith(wxT("tan("))) {
        *toParse = toParse->AfterFirst('n');
        return new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("tan"));
    } else if(toParse->StartsWith(wxT("!"))) {
        *toParse = toParse->AfterFirst('!');
        return ParseMath(toParse, new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("!")), motor_num);
    } else if(toParse->StartsWith(wxT("-"))) {
        *toParse = toParse->AfterFirst('-');
        return ParseMath(toParse, new NodeTrigExpr(ParseExpression(toParse, motor_num), wxT("-")), motor_num);
    } else if((toReturn = ParseVar(toParse, motor_num)) != NULL) {
        return ParseMath(toParse, toReturn, motor_num);
    } else if((toReturn = ParseNumber(toParse)) != NULL) {
        return ParseMath(toParse, toReturn, motor_num);
    } else if(toParse->StartsWith(wxT("("))) {
        *toParse = toParse->AfterFirst('(');
        toReturn = ParseExpression(toParse, motor_num);
        checkParen = true;
    } else {
        wxPuts(*toParse + wxT(" was toParse. Syntax Error Parsing.  Returning..."));
        return NULL;
    }

    if(checkParen && !toParse->StartsWith(wxT(")"))) {
        wxPuts(wxT("Uh oh. Missing an end ')'"));
    } else if (checkParen && toParse->StartsWith(wxT(")"))) {
        *toParse = toParse->AfterFirst(')');
        return ParseMath(toParse, toReturn, motor_num);
    }

    return NULL;
}

//! clean parentheses
void ParserRT::CleanParentheses(wxString* expr) {
    bool hit = true;

    while(hit) {
        hit = false;

        if(expr->Contains(wxT("()"))) {
            hit = true;
            expr->Replace(wxT("()"), wxT(""), false);
        }
    }
}

//! clean whitespace
wxString ParserRT::CleanWhiteSpace(wxString* expr) {
    wxString toReturn;

    while(!expr->IsEmpty()) {
        toReturn.Append(expr->BeforeFirst(' '));
        *expr = expr->AfterFirst(' ');
    }

    *expr = toReturn;
    toReturn = wxT("");

    while(!expr->IsEmpty()) {
        toReturn.Append(expr->BeforeFirst('\n'));
        *expr = expr->AfterFirst('\n');
    }

    *expr = toReturn;
    toReturn = wxT("");

    while(!expr->IsEmpty()) {
        toReturn.Append(expr->BeforeFirst('\r'));
        *expr = expr->AfterFirst('\r');
    }

    return toReturn;
}

//! parse a variable
NodeExpr* ParserRT::ParseVar(wxString* expr, int motor_num) {

    wxString* toParse = expr;
    wxString var = wxT("");
    int motor = motor_num;

    //! if no motor number is specified after an input,
    //! default motor value to 1
    //! example: D4 is delta for motor 4, D is delta for motor 1

    if(toParse->StartsWith(wxT("D"))) {
        *toParse = toParse->AfterFirst('D');
        var.Append('D');
    } else if(toParse->StartsWith(wxT("F"))) {
        *toParse = toParse->AfterFirst('F');
        var.Append('F');
    } else if(toParse->StartsWith(wxT("N"))) {
        *toParse = toParse->AfterFirst('N');
        var.Append('N');
    } else if(toParse->StartsWith(wxT("P"))) {
        *toParse = toParse->AfterFirst('P');
        var.Append('P');
    } else if(toParse->StartsWith(wxT("S"))) {
        *toParse = toParse->AfterFirst('S');
        var.Append('S');
    } else if(toParse->StartsWith(wxT("T"))) {
        *toParse = toParse->AfterFirst('T');
        var.Append('T');
    } else if(toParse->StartsWith(wxT("V"))) {
        *toParse = toParse->AfterFirst('V');
        var.Append('V');
    //allow lower case to work as well
    } else if(toParse->StartsWith(wxT("d"))) {
        *toParse = toParse->AfterFirst('d');
        var.Append('D');
    } else if(toParse->StartsWith(wxT("f"))) {
        *toParse = toParse->AfterFirst('f');
        var.Append('F');
    } else if(toParse->StartsWith(wxT("n"))) {
        *toParse = toParse->AfterFirst('n');
        var.Append('N');
    } else if(toParse->StartsWith(wxT("p"))) {
        *toParse = toParse->AfterFirst('p');
        var.Append('P');
    } else if(toParse->StartsWith(wxT("s"))) {
        *toParse = toParse->AfterFirst('s');
        var.Append('S');
    } else if(toParse->StartsWith(wxT("t"))) {
        *toParse = toParse->AfterFirst('t');
        var.Append('T');
    } else if(toParse->StartsWith(wxT("v"))) {
        *toParse = toParse->AfterFirst('v');
        var.Append('V');
    }

    //no valid variable was found
    if(var.IsEmpty()) {
        return NULL;
    }

    if(toParse->StartsWith(wxT("1"))) {
        *toParse = toParse->AfterFirst('1');
        var.Append('1');
        motor = 1;
    } else if(toParse->StartsWith(wxT("2"))) {
        *toParse = toParse->AfterFirst('2');
        var.Append('2');
        motor = 2;
    } else if(toParse->StartsWith(wxT("3"))) {
        *toParse = toParse->AfterFirst('3');
        var.Append('3');
        motor = 3;
    } else if(toParse->StartsWith(wxT("4"))) {
        *toParse = toParse->AfterFirst('4');
        var.Append('4');
        motor = 4;
    } else {
        var.Append(wxString::Format(wxT("%i"), motor_num));
    }

    return new NodeVar(var, motor);
}

//! parse a number
NodeExpr* ParserRT::ParseNumber(wxString* expr) {

    bool done = false;
    bool decimal = false;
    wxString number;

    //! allow for floats to be parsed
    while(!done) {

        if(expr->StartsWith(wxT("0"))) {
            *expr = expr->AfterFirst('0');
            number.Append('0');
        } else if(expr->StartsWith(wxT("1"))) {
            *expr = expr->AfterFirst('1');
            number.Append('1');
        } else if(expr->StartsWith(wxT("2"))) {
            *expr = expr->AfterFirst('2');
            number.Append('2');
        } else if(expr->StartsWith(wxT("3"))) {
            *expr = expr->AfterFirst('3');
            number.Append('3');
        } else if(expr->StartsWith(wxT("4"))) {
            *expr = expr->AfterFirst('4');
            number.Append('4');
        } else if(expr->StartsWith(wxT("5"))) {
            *expr = expr->AfterFirst('5');
            number.Append('5');
        } else if(expr->StartsWith(wxT("6"))) {
            *expr = expr->AfterFirst('6');
            number.Append('6');
        } else if(expr->StartsWith(wxT("7"))) {
            *expr = expr->AfterFirst('7');
            number.Append('7');
        } else if(expr->StartsWith(wxT("8"))) {
            *expr = expr->AfterFirst('8');
            number.Append('8');
        } else if(expr->StartsWith(wxT("9"))) {
            *expr = expr->AfterFirst('9');
            number.Append('9');
        } else if(expr->StartsWith(wxT(".")) && !decimal) {
            *expr = expr->AfterFirst('.');
            decimal = true;
            number.Append('.');
        } else {
            done = true;
        }
    }

    double value;

    if(!number.ToDouble(&value)) {
        return NULL;
    }

    return new NodeNum(value);
}

//! parsing for expressions that have the syntax < (expr) (operation) (expr) >
//! NOTE: conditional expressions are also handled in this function
NodeExpr* ParserRT::ParseMath(wxString* expr, NodeExpr* node, int motor_num) {

    wxString* toParse = expr;

    //! parse an operation and continue to parse recursively
    if(toParse->StartsWith(wxT("^"))) {
        *expr = expr->AfterFirst('^');
        return new NodeExpr(node, ParseExpression(expr, motor_num), wxT("^"));
    } else if(toParse->StartsWith(wxT("*"))) {
        *expr = expr->AfterFirst('*');
        return new NodeExpr(node, ParseExpression(expr, motor_num), wxT("*"));
    } else if(toParse->StartsWith(wxT("/"))) {
        *expr = expr->AfterFirst('/');
        return new NodeExpr(node, ParseExpression(expr, motor_num), wxT("/"));
    } else if(toParse->StartsWith(wxT("%"))) {
        *expr = expr->AfterFirst('%');
        return new NodeExpr(node, ParseExpression(expr, motor_num), wxT("%"));
    } else if(toParse->StartsWith(wxT("+"))) {
        *expr = expr->AfterFirst('+');
        return new NodeExpr(node, ParseExpression(expr, motor_num), wxT("+"));
    } else if(toParse->StartsWith(wxT("-"))) {
        *expr = expr->AfterFirst('-');
        return new NodeExpr(node, ParseExpression(expr, motor_num), wxT("-"));
    } else if(toParse->StartsWith(wxT("&&"))) {
        *expr = expr->AfterFirst('&');
        *expr = expr->AfterFirst('&');
        return new NodeExpr(node, ParseExpression(expr, motor_num), wxT("&&"));
    } else if(toParse->StartsWith(wxT("||"))) {
        *expr = expr->AfterFirst('|');
        *expr = expr->AfterFirst('|');
        return new NodeExpr(node, ParseExpression(expr, motor_num), wxT("||"));
    } else if(toParse->StartsWith(wxT(">="))) {
        *expr = expr->AfterFirst('>');
        *expr = expr->AfterFirst('=');
        return new NodeExpr(node, ParseExpression(expr, motor_num), wxT(">="));
    } else if(toParse->StartsWith(wxT("<="))) {
        *expr = expr->AfterFirst('<');
        *expr = expr->AfterFirst('=');
        return new NodeExpr(node, ParseExpression(expr, motor_num), wxT("<="));
    } else if(toParse->StartsWith(wxT("!="))) {
        *expr = expr->AfterFirst('!');
        *expr = expr->AfterFirst('=');
        return new NodeExpr(node, ParseExpression(expr, motor_num), wxT("!="));
    } else if(toParse->StartsWith(wxT("<"))) {
        *expr = expr->AfterFirst('<');
        return new NodeExpr(node, ParseExpression(expr, motor_num), wxT("<"));
    } else if(toParse->StartsWith(wxT(">"))) {
        *expr = expr->AfterFirst('>');
        return new NodeExpr(node, ParseExpression(expr, motor_num), wxT(">"));
    } else if(toParse->StartsWith(wxT("=="))) {
        *expr = expr->AfterFirst('=');
        *expr = expr->AfterFirst('=');
        return new NodeExpr(node, ParseExpression(expr, motor_num), wxT("=="));

    //conditional expression
    //parse the conditional expression in three segments
    } else if(toParse->StartsWith(wxT("?"))) {
        *expr = expr->AfterFirst('?');

//#if DEBUG_PARSER
//        wxPuts(wxT("Parsed cond section"));
//#endif

        //conditional expression is already parsed at this point
        //set it (node) to NodeExpr* cond
        NodeExpr* cond = node;

        //parse the condition true expression
        NodeExpr* condtrue = ParseExpression(expr, motor_num);

        //make sure the grammar (expr) ? (expr) : (expr) is followed
        if(toParse->StartsWith(wxT(":"))) {
            *expr = expr->AfterFirst(':');

//#if DEBUG_PARSER
//            wxPuts(wxT("Parsed true section"));
//#endif
            //parse the condition false expression
            NodeExpr* condfalse = ParseExpression(expr, motor_num);

//#if DEBUG_PARSER
//            wxPuts(wxT("Parsed false section"));
//#endif

            //if everything is valid, return true, otherwise NULL
            if(cond && condtrue && condfalse) {
                return new NodeCondExpr(cond, condtrue, condfalse);
            } else {
                return NULL;
            }

        } else {
            //return NULL if no ':' was found
            return NULL;
        }

    //NOTE: toParse->StartsWith(wxT(":")) is a special case
    //otherwise, stop recursively parsing when there is nothing left to parse or a ")" is found
    } else if(toParse->IsEmpty() || toParse->StartsWith(wxT(":")) || toParse->StartsWith(wxT(")"))) {
        return node;
    }

    return NULL;
}
