#ifndef REALPANEL_H
#define REALPANEL_H

//(*Headers(RealPanel)
#include <wx/panel.h>
class wxStaticBox;
class wxCheckBox;
class wxTextCtrl;
class wxStaticText;
class wxButton;
//*)

#include <wx/statbox.h>
#include <wx/textctrl.h>
#include <wx/panel.h>
#include <wx/stattext.h>
#include <wx/checkbox.h>
#include <wx/button.h>

#include <wx/filedlg.h>
#include <wx/stattext.h>
#include <wx/textctrl.h>

#include <vector>
#include <deque>

#include "Common/mathplot.h"
#include "Common/tmr.h"

class wxArrayInt;
class wxMessageDialog;

class BHand;

class SuperPanel;
class RealPlotPanel;

class ParserRT;
class NodeExpr;
class GraphData;


class RealPanel: public wxPanel
{
public:
	RealPanel(wxWindow* parent, BHand *bhand, SuperPanel *super_panel, wxWindowID id=wxID_ANY, const wxPoint& pos=wxDefaultPosition, const wxSize& size=wxDefaultSize);
	virtual ~RealPanel();

	void SetRealPlotPanel(RealPlotPanel *real_plot_panel);

	int RTRun();
	bool RTIsRunning();
	void RTAbort();

	void Service();

	void EnterTab();
	void LeaveTab();

	///////////////////////////////////////////////////////////////////////////
	// Used by RealPlotPanel
	///////////////////////////////////////////////////////////////////////////

	// Save the plot data into a .csv file example <N>, <control_velocity1>, <control_velocity2>, <control_velocity3>, <control_velocity4>, <strain1>, <strain2>, <strain3>, <strain4>
	void PlotSave(wxString toSave, wxArrayInt* displayed, int n_start, int n_end);

	// Updates the display (live updating during RealTime mode)
	void PlotUpdateDisplay(wxArrayInt* toDisplay);

	// Refresh the graph
	void PlotRefresh();


private:

	BHand *m_hand;

	SuperPanel *m_super_panel;
	RealPlotPanel *m_real_plot_panel;

	bool m_real_before_running;
	bool m_real_realtime_running;
	bool m_real_after_running;

	wxString cycleFilePath;

	HiResTimer t_real;

	std::vector<wxTextCtrl *> m_cv;  // control velocity expressions
	std::vector<wxTextCtrl *> m_cpg; // control proportional gain expressions


	void init();

	void LoadCode(wxString toLoad); // loads a real-time ".rt" file
	void SaveCode(wxString toSave); // saves a real-time ".rt" file
	void ClearCode(); // resets to default parameters in real-time tab
	void GenerateCpp(wxString toSave); // generates C++ code from the current real-time parameters

	bool PrepareRT(); // prepares realtime mode
	void Abort(); // issue an abort while running
	bool Run(); // run one iteration of real-time mode

	void LoadFile(wxString toLoad); // loads a file into a deque (to be accessed in realtime mode)

	void SetEnableForCVandCPGTextboxes(); // Sets the enabled state of the textboxes based on the checkbox state

	// Parse, substitute, interpret the expressions to be used in real-time mode
	bool ParseWindows();
	bool SubstituteVariables();
	void InterpretAndSet(float time);

	// Parsed Expressions
	NodeExpr *m_exprs[8];
	NodeExpr *m_term_expr;

	// Used for real-time loop
	int N;
	int time;
    long cvelmult, delmult, fvelmult;
	double sampleperiod[4];

	// For the real-time plot data tab and real-time graphs
	// 32 graphable functions total 4 x 8
	std::vector<GraphData *> m_plots[4];
	// wxmathplot graphs
	mpWindow *m_graphs[4];

	// Buffer for graph data
	std::deque<float> m_graph_data[4][8];   //0 - Position
											//1 - Velocity
											//2 - Strain
											//3 - Delta Position
											//4 - File Data
											//5 - Time Passed
											//6 - Control Velocity
											//7 - Control Proportional Gains
	
	// Matrix buffer for real-time mode
	float motor_data[4][7];     //0 - Delta Position
	                            //1 - File Data
	                            //2 - N - update number
	                            //3 - Position
	                            //4 - Strain
	                            //5 - Time passed
	                            //6 - Velocity

	// Pertains to loading a file into RT mode
	wxString m_file_path;
	std::deque<float*> file_data;

	// Event Handlers
	void OnRTHelp(wxCommandEvent& event);
	void OnRTLoadCode(wxCommandEvent& event);
	void OnRTSaveCode(wxCommandEvent& event);
	void OnRTClearCode(wxCommandEvent& event);
	void OnRTGenerateCpp(wxCommandEvent& event);
	void OnSetCVandCPGTextboxEnable(wxCommandEvent& event);
	void OnRTExecuteBefore(wxCommandEvent& event);
	void OnRTExecuteAfter(wxCommandEvent& event);
	void OnRTLoadBefore(wxCommandEvent& event);
	void OnRTLoadAfter(wxCommandEvent& event);
	void OnRTAbort(wxCommandEvent& event);
	void OnRTRun(wxCommandEvent& event);
	void OnRTLoadFile(wxCommandEvent& event);

	//(*Handlers(RealPanel)
	void OnREAL_Motor1_CheckBoxClick(wxCommandEvent& event);
	//*)

	//(*Declarations(RealPanel)
	wxCheckBox* REAL_Motor4_CheckBox;
	wxButton* REAL_Run_Button;
	wxButton* REAL_LoadCode_Button;
	wxTextCtrl* REAL_Cv3_TextCtrl;
	wxStaticBox* REAL_TerminationCondition_StaticBox;
	wxCheckBox* REAL_ControlPropGain_CheckBox;
	wxTextCtrl* REAL_FVelMultiplier_TextCtrl;
	wxButton* REAL_LoadAfter_Button;
	wxButton* REAL_GenerateCpp_Button;
	wxStaticBox* REAL_DataFrpmFile_StaticBox;
	wxCheckBox* REAL_ExecuteBefore_CheckBox;
	wxTextCtrl* REAL_Cv1_TextCtrl;
	wxTextCtrl* REAL_DeltaMultiplier_TextCtrl;
	wxCheckBox* REAL_Delta_CheckBox;
	wxTextCtrl* REAL_CVelMultiplier_TextCtrl;
	wxStaticText* REAL_File_StaticText;
	wxStaticText* REAL_FileName_StaticText;
	wxButton* REAL_Abort_Button;
	wxTextCtrl* REAL_Cpg3_TextCtrl;
	wxStaticBox* REAL_File_StaticBox;
	wxButton* REAL_LoadBefore_Button;
	wxStaticBox* REAL_RealTimeParameters_StaticBox;
	wxStaticBox* REAL_RealTimeControl_StaticBox;
	wxStaticText* StaticText1;
	wxStaticText* REAL_DeltaMultiplier_StaticText;
	wxTextCtrl* REAL_Cpg1_TextCtrl;
	wxButton* REAL_ClearCode_Button;
	wxCheckBox* REAL_Position_CheckBox;
	wxCheckBox* REAL_Motor3_CheckBox;
	wxCheckBox* REAL_Motor2_CheckBox;
	wxCheckBox* REAL_ExecuteAfter_CheckBox;
	wxTextCtrl* REAL_After_TextCtrl;
	wxTextCtrl* REAL_Cv2_TextCtrl;
	wxTextCtrl* REAL_Before_TextCtrl;
	wxStaticText* REAL_CVelMultiplier_StaticText;
	wxButton* Real_Help_Button;
	wxButton* REAL_LoadFile_Button;
	wxTextCtrl* REAL_Cpg4_TextCtrl;
	wxCheckBox* REAL_ControlVelocity_CheckBox;
	wxTextCtrl* REAL_Cv4_TextCtrl;
	wxStaticText* REAL_FVelMultiplier_StaticText;
	wxCheckBox* REAL_Motor1_CheckBox;
	wxTextCtrl* REAL_Cpg2_TextCtrl;
	wxCheckBox* REAL_Velocity_CheckBox;
	wxCheckBox* REAL_Strain_CheckBox;
	wxTextCtrl* REAL_TerminationCondition_TextCtrl;
	wxButton* REAL_SaveCode_Button;
	//*)

	//(*Identifiers(RealPanel)
	static const long ID_REAL_RealTimeControl_StaticBox;
	static const long ID_REAL_ControlVelocity_CheckBox;
	static const long ID_REAL_ControlPropGain_CheckBox;
	static const long ID_REAL_Motor1_CheckBox;
	static const long ID_REAL_Motor2_CheckBox;
	static const long ID_REAL_Motor3_CheckBox;
	static const long ID_REAL_Motor4_CheckBox;
	static const long ID_REAL_Cv1_TextCtrl;
	static const long ID_REAL_Cv2_TextCtrl;
	static const long ID_REAL_Cv3_TextCtrl;
	static const long ID_REAL_Cv4_TextCtrl;
	static const long ID_REAL_Cpg1_TextCtrl;
	static const long ID_REAL_Cpg2_TextCtrl;
	static const long ID_REAL_Cpg3_TextCtrl;
	static const long ID_REAL_Cpg4_TextCtrl;
	static const long ID_REAL_LoadCode_Button;
	static const long ID_REAL_GenerateCpp_Button;
	static const long ID_REAL_SaveCode_Button;
	static const long ID_REAL_ClearCode_Button;
	static const long ID_REAL_ExecuteBefore_CheckBox;
	static const long ID_REAL_LoadBefore_Button;
	static const long ID_REAL_ExecuteAfter_CheckBox;
	static const long ID_REAL_LoadAfter_Button;
	static const long ID_REAL_Before_TextCtrl;
	static const long ID_REAL_After_TextCtrl;
	static const long ID_REAL_Run_Button;
	static const long ID_REAL_Abort_Button;
	static const long ID_REAL_RealTimeParameters_StaticBox;
	static const long ID_REAL_TerminationCondition_StaticBox;
	static const long ID_REAL_TerminationCondition_TextCtrl;
	static const long ID_REAL_Position_CheckBox;
	static const long ID_REAL_Strain_CheckBox;
	static const long ID_REAL_Velocity_CheckBox;
	static const long ID_REAL_Delta_CheckBox;
	static const long ID_REAL_FVelMultiplier_StaticText;
	static const long ID_REAL_DeltaMultiplier_StaticText;
	static const long ID_REAL_CVelMultiplier_StaticText;
	static const long ID_REAL_FVelMultiplier_TextCtrl;
	static const long ID_REAL_DeltaMultiplier_TextCtrl;
	static const long ID_REAL_CVelMultiplier_TextCtrl;
	static const long ID_REAL_DataFrpmFile_StaticBox;
	static const long ID_REAL_File_StaticText;
	static const long ID_REAL_File_StaticBox;
	static const long ID_REAL_LoadFile_Button;
	static const long ID_REAL_FileName_StaticText;
	static const long ID_STATICTEXT1;
	static const long ID_REAL_HELP_BUTTON;
	//*)


	DECLARE_EVENT_TABLE()
};

#endif
