#ifndef SUPERPANEL_H
#define SUPERPANEL_H

//(*Headers(SuperPanel)
#include <wx/panel.h>
class wxStaticBox;
class wxCheckBox;
class wxTextCtrl;
class wxRadioButton;
class wxStaticText;
class wxButton;
class wxChoice;
//*)

class wxFile;
class wxFrame;
class wxFileDialog;
class wxMessageDialog;

class BHand;
class BHandProperty;


class SuperPanel : public wxPanel
{
public:

	SuperPanel(wxWindow* parent, BHand *hand, wxWindowID id=wxID_ANY, const wxPoint& pos=wxDefaultPosition, const wxSize& size=wxDefaultSize);
	virtual ~SuperPanel();

	void EnablePanel(bool flag);

	void Service();
	bool IsRunning();
	void LeaveTab();

	// Used by RealTimePanel
	bool ExternalRunDone();
	bool SupervisoryExternalRun(bool *cycle = NULL);
	void PrepareSupervisoryExternalRun(wxString toRun);

	//void useRealTimeCyclePath(bool );

	void InitLibrary();

private:

	BHand *m_hand;

	bool m_supervise_running;
	bool shouldAbort;

	wxMessageDialog *m_response;


	char m_motors[5];
	char m_motors_param[5];
	wxString m_motors_string;
	wxString m_motors_param_string;


	void init();

	void fillPropertiesListBoxes();
	void fillPropertyChoice();

	void updatePropertyAttributes();
	void updatePropertyValues();
	int setPropertyValues();

	int getSelectedPropertyIndex();
	BHandProperty * getSelectedProperty();

	void ThrowError(wxString message);

	//! Return false if no motors are selected, otherwise return true
	bool motorsSelected();

	//! update session history and command buffer windows
	void UpdateTextCrtl(wxString ascii_message, wxString cpp_message);

	//! update char arrays with currently selected motors
	void generateMotorString(int group);

	//! used in TerminalRun
	int parseMotorString(wxString input, char* buf);
	bool parseCommandValue(wxString input, long* value);
	bool asciiToFunctionCalls(wxString input);
	wxString asciiToCpp(wxString input);
	//! used to format string commands to hand
	void FixSpaces(wxString * toFix);



	//! Supervise Terminal Buttons
	bool TerminalRun();

	//! Supervise Control Parameters
	void ParamSet(wxString param);

	bool *cyclePtr;
	wxString cycleFilePath;

	//!Terminal Section
	wxString    ascii_session;
	wxString    cpp_session;

	//!Terminal Flag
	bool ascii_selected;

	bool m_panelEnabled;

	//! External Utility
	wxString externalToRun;


	//! saving files - note, you must set the wildcard before using this filedialog
	//! as it is used by multiple functions
	wxFileDialog *FileSave;


	void enableControlPanelButtons();

	void runCommandsInBuffer();


	//!Supervise - Terminal
	void OnSuperTerminalClearSession(wxCommandEvent& event);
	void OnSuperTerminalClearBuffer(wxCommandEvent& event);
	void OnSuperTerminalCopy(wxCommandEvent& event);
	void OnSuperTerminalRun(wxCommandEvent& event);
	void OnSuperTerminalEnterHit(wxCommandEvent& event);
	void OnSuperTerminalGenerateCpp(wxCommandEvent& event);
	void OnSuperTerminalLoad(wxCommandEvent& event);
	void OnSuperTerminalSave(wxCommandEvent& event);
	void OnSuperAutoRunCheckBox(wxCommandEvent& event);


	//!Supervise - Control Panel - Motors
	void OnSuperControlMotor1Box(wxCommandEvent& event);
	void OnSuperControlMotor2Box(wxCommandEvent& event);
	void OnSuperControlMotor3Box(wxCommandEvent& event);
	void OnSuperControlMotor4Box(wxCommandEvent& event);

	//!Supervise - Control Panel - Buttons
	void OnSuperControlOpen(wxCommandEvent& event);
	void OnSuperControlClose(wxCommandEvent& event);
	void OnSuperControlTorqueOpen(wxCommandEvent& event);
	void OnSuperControlTorqueClose(wxCommandEvent& event);
	void OnSuperControlGoHome(wxCommandEvent& event);
	void OnSuperControlGoDefault(wxCommandEvent& event);
	void OnSuperControlInitialize(wxCommandEvent& event);
	void OnSuperControlStop(wxCommandEvent& event);
	void OnSuperControlGoPosition(wxCommandEvent& event);
	void OnSuperControlStepOpen(wxCommandEvent& event);
	void OnSuperControlStepClose(wxCommandEvent& event);
	void OnSuperControlAbort(wxCommandEvent& event);
	void OnSuperControlDelay(wxCommandEvent& event);
	void OnSuperControlTemperature(wxCommandEvent& event);
	void OnSuperControlDefault(wxCommandEvent& event);
	void OnSuperControlSave(wxCommandEvent& event);
	void OnSuperControlLoad(wxCommandEvent& event);

	//!Supervise - Control Panel - Parameter
	void OnSuperParamMotor1Box(wxCommandEvent& event);
	void OnSuperParamMotor2Box(wxCommandEvent& event);
	void OnSuperParamMotor3Box(wxCommandEvent& event);
	void OnSuperParamMotor4Box(wxCommandEvent& event);
	void OnSuperParamGet(wxCommandEvent& event);
	void OnSuperParamSet(wxCommandEvent& event);
	void OnSuperParamSetDP(wxCommandEvent& event);
	void OnSuperParamCategoryChoice(wxCommandEvent& event);
	void OnSuperParamChoice(wxCommandEvent& event);

	//(*Handlers(SuperPanel)
	void OnSUPER_terminal_ascii_RadioButtonSelect(wxCommandEvent& event);
	void OnSUPER_terminal_cpp_RadioButtonSelect(wxCommandEvent& event);
	void OnSUPER_ParametersMotor1_CheckBoxClick(wxCommandEvent& event);
	void OnSUPER_ParametersMotor2_CheckBoxClick(wxCommandEvent& event);
	void OnSUPER_ParametersMotor3_CheckBoxClick(wxCommandEvent& event);
	void OnSUPER_ParametersMotor4_CheckBoxClick(wxCommandEvent& event);
	void OnSUPER_ControlLoad_ButtonClick(wxCommandEvent& event);
	void OnSUPER_ControlSave_ButtonClick(wxCommandEvent& event);
	void OnSUPER_terminal_ascii_RadioButtonSelect1(wxCommandEvent& event);
	//*)

	void OnMotorMovementIncludedCheckBox(wxCommandEvent& event);

	//(*Declarations(SuperPanel)
	wxCheckBox* SUPER_Motor2_CheckBox;
	wxCheckBox* SUPER_ParametersMotor1_CheckBox;
	wxTextCtrl* notesTextCtrl;
	wxButton* SUPER_StepClose_Button;
	wxCheckBox* SUPER_ParametersMotor2_CheckBox;
	wxButton* SUPER_GoPosition_Button;
	wxStaticText* StaticTextProperty;
	wxButton* SUPER_ParameterGet_Button;
	wxStaticText* SUPER_ms_StaticText;
	wxButton* SUPER_TorqueOpen_Button;
	wxButton* SUPER_StepOpen_Button;
	wxStaticBox* MOTOR_MOVEMENT_VALUES_StaticBox;
	wxStaticBox* SUPER_Terminal_StaticBox;
	wxTextCtrl* SUPER_CommandBuffer_TextCtrl;
	wxChoice* Super_Parameters_Category_Choice;
	wxButton* SUPER_ParameterSet_Button;
	wxTextCtrl* SUPER_ParametersMotor4_TextCtrl;
	wxStaticBox* SUPER_ControlPanel_StaticBox;
	wxTextCtrl* SUPER_ParametersMotor3_TextCtrl;
	wxButton* SUPER_GenerateCpp_Button;
	wxStaticBox* SUPER_Parameters_StaticBox;
	wxButton* SUPER_Run_Button;
	wxButton* SUPER_ClearSession_Button;
	wxStaticText* SUPER_C_StaticText;
	wxButton* SUPER_StopMotor_Button;
	wxCheckBox* SUPER_Motor3_CheckBox;
	wxTextCtrl* SUPER_SessionHistory_TextCtrl;
	wxCheckBox* MotorMovementIncludedCheckBox;
	wxTextCtrl* SUPER_Temperature_TextCtrl;
	wxStaticText* notesStaticText;
	wxRadioButton* SUPER_terminal_cpp_RadioButton;
	wxStaticText* SUPER_CommandBuffer_StaticText;
	wxButton* SUPER_Copy_Button;
	wxRadioButton* SUPER_terminal_ascii_RadioButton;
	wxButton* SUPER_GoHome_Button;
	wxCheckBox* SUPER_Motor4_CheckBox;
	wxButton* SUPER_GoDefault_Button;
	wxStaticText* StaticTextCategory;
	wxButton* SUPER_TorqueClose_Button;
	wxButton* SUPER_ControlLoad_Button;
	wxButton* SUPER_Close_Button;
	wxChoice* SUPER_Parameters_Choice;
	wxButton* SUPER_Default_Button;
	wxButton* SUPER_TerminalSave_Button;
	wxStaticText* purposeStaticText;
	wxTextCtrl* SUPER_ParametersMotor2_TextCtrl;
	wxTextCtrl* SUPER_MovementAmount_TextCtrl;
	wxButton* SUPER_TerminalLoad_Button;
	wxStaticBox* SUPER_INIT_StaticBox;
	wxTextCtrl* SUPER_ParametersMotor1_TextCtrl;
	wxButton* SUPER_Temperature_Button;
	wxButton* SUPER_ParameterSetDPAndGo_Button;
	wxButton* SUPER_ControlSave_Button;
	wxButton* SUPER_Initialize_Button;
	wxStaticBox* StaticBoxMotorMovement;
	wxButton* SUPER_ClearBuffer_Button;
	wxCheckBox* SUPER_AutoRun_CheckBox;
	wxStaticText* EncoderTicksStaticText;
	wxButton* SUPER_Abort_Button;
	wxCheckBox* SUPER_ParametersMotor4_CheckBox;
	wxStaticBox* MiscStaticBox;
	wxCheckBox* SUPER_ParametersMotor3_CheckBox;
	wxCheckBox* SUPER_Motor1_CheckBox;
	wxButton* SUPER_Delay_Button;
	wxStaticBox* SUPER_Motors_StaticBox;
	wxStaticText* SUPER_SessionHistory_StaticText;
	wxTextCtrl* purposeTextCtrl;
	wxButton* SUPER_Open_Button;
	wxTextCtrl* SUPER_Delay_TextCtrl;
	//*)

	//(*Identifiers(SuperPanel)
	static const long ID_SUPER_ControlPanel_StaticBox;
	static const long ID_SUPER_INIT_STATICBOX;
	static const long ID_MOTOR_MOVEMENT_VALUES_STATICBOX;
	static const long ID_SUPER_Parameters_StaticBox;
	static const long ID_StaticBoxMotorMovement;
	static const long ID_MiscStaticBox;
	static const long ID_NOTES_TEXTCTRL;
	static const long ID_SUPER_Terminal_StaticBox;
	static const long ID_SUPER_SessionHistory_StaticText;
	static const long ID_SUPER_SessionHistory_TextCtrl;
	static const long ID_SUPER_AutoRun_CheckBox;
	static const long ID_SUPER_Run_Button;
	static const long ID_SUPER_Copy_Button;
	static const long ID_SUPER_CommandBuffer_TextCtrl;
	static const long ID_SUPER_CommandBuffer_StaticText;
	static const long ID_SUPER_GenerateCpp_Button;
	static const long ID_SUPER_ClearSession_Button;
	static const long ID_SUPER_TerminalLoad_Button;
	static const long ID_SUPER_TerminalSave_Button;
	static const long ID_SUPER_ClearBuffer_Button;
	static const long ID_SUPER_Motors_StaticBox;
	static const long ID_SUPER_Motor1_CheckBox;
	static const long ID_SUPER_Motor2_CheckBox;
	static const long ID_SUPER_Motor3_CheckBox;
	static const long ID_SUPER_Motor4_CheckBox;
	static const long ID_SUPER_Open_Button;
	static const long ID_SUPER_Close_Button;
	static const long ID_SUPER_TorqueOpen_Button;
	static const long ID_SUPER_TorqueClose_Button;
	static const long ID_SUPER_GoHome_Button;
	static const long ID_SUPER_GoDefault_Button;
	static const long ID_SUPER_Initialize_Button;
	static const long ID_SUPER_StopMotor_Button;
	static const long ID_SUPER_Abort_Button;
	static const long ID_SUPER_MovementAmount_TextCtrl;
	static const long ID_SUPER_GoPosition_Button;
	static const long ID_SUPER_StepOpen_Button;
	static const long ID_SUPER_StepClose_Button;
	static const long ID_SUPER_Default_Button;
	static const long ID_SUPER_ControlLoad_Button;
	static const long ID_SUPER_ControlSave_Button;
	static const long ID_SUPER_Delay_TextCtrl;
	static const long ID_SUPER_Temperature_TextCtrl;
	static const long ID_SUPER_ms_StaticText;
	static const long ID_SUPER_C_StaticText;
	static const long ID_SUPER_Delay_Button;
	static const long ID_SUPER_Temperature_Button;
	static const long ID_SUPER_Parameters_Choice;
	static const long ID_SUPER_ParametersMotor1_CheckBox;
	static const long ID_SUPER_ParametersMotor2_CheckBox;
	static const long ID_SUPER_ParametersMotor3_CheckBox;
	static const long ID_SUPER_ParametersMotor4_CheckBox;
	static const long ID_SUPER_ParametersMotor1_TextCtrl;
	static const long ID_SUPER_ParametersMotor2_TextCtrl;
	static const long ID_SUPER_ParametersMotor3_TextCtrl;
	static const long ID_SUPER_ParametersMotor4_TextCtrl;
	static const long ID_SUPER_ParameterGet_Button;
	static const long ID_SUPER_ParameterSet_Button;
	static const long ID_SUPER_ParameterSetDPAndGo_Button;
	static const long ID_SUPER_terminal_ascii_RadioButton;
	static const long ID_SUPER_terminal_cpp_RadioButton;
	static const long ID_EncoderTicksStaticText;
	static const long ID_StaticTextProperty;
	static const long ID_StaticTextCategory;
	static const long ID_SUPER_Parameters_Category_Choice;
	static const long ID_PURPOSE_STATIC_TEXT;
	static const long ID_PURPOSE_TEXTCTRL;
	static const long ID_NOTES_STATICTEXT;
	static const long ID_MOTOR_MOVEMENT_INCLUDED_CHECKBOX;
	//*)

	DECLARE_EVENT_TABLE()
};

#endif
