#ifndef HANDVISUAL_H_INCLUDED
#define HANDVISUAL_H_INCLUDED

#pragma warning ( disable : 4005 )

#include "GL/gl.h"
// According to http://www.mingw.org/wiki/HOWTO_Use_Mark_J_Kilgards_OpenGL_Utility_Toolkit_GLUT_with_MinGW
// these macros should be defined before including <GL/glut.h>
#define GLUT_NO_LIB_PRAGMA
#define GLUT_NO_WARNING_DISABLE
#define GLUT_DISABLE_ATEXIT_HACK
//#define _STDCALL_SUPPORTED
//#define _M_IX86
#ifndef LINUX
#include <windows.h>
#endif
#include "GL/glut.h"

#include <vector>

#include "BHandKinematics.h"

class VRMLModel;
class HandBase;
class HandJoint;
class HandKnuckle;
class HandMiddle;
class HandFinger;
class HandFingerSpreadable;
class BoundingSphere;

class VisualFingerTip;
class VisualPalm;
class VisualData;

static const int NUM_MODELS = 12;
static const int NUM_BOUNDING_SPHERES = 18;

class HandManager {

    public:
        HandManager(float x, float y, float z, float pitch, float yaw, float roll, float scale);
        ~HandManager();

        //! rigid-body animations of various hand components
        void Move(VRMLModel* selected, int xMove, int yMove);
        void MoveSelected(int xMove, int yMove);

        //! draw the hand (with or without color picking)
        void DrawHand(const bool ColorPicking);

        //! color picking / selection functions
        VRMLModel* FindSelectedPart(const unsigned char* pixel);

        void SelectUpdate();
        void DeselectUpdate();
        bool isSelected(const VRMLModel* toCheck);

        float getFingerRot1();
        float getFingerRot2();
        float getFingerRot3();
        float getSpreadRot();

        void RefreshLocation(float f1, float f2, float f3, float spread);

		void showKinematics(bool show) { m_showKinematics = show; }

		void displayStrain(bool display) { m_displayStrain = display; }
		void displayPPS(bool display) { m_displayPPS = display; }

    private:

        //! for collision detection
        void Cache();
        void Revert();

        //! helper function (for selection)
        void UpdateSelection();

        //! models

        HandBase*                   Base;

        HandKnuckle*                F1_Knuckle;
        HandMiddle*                 F1_Middle;
        HandFinger*                 F1_Finger;

        HandJoint*                  F2_Joint;
        HandKnuckle*                F2_Knuckle;
        HandMiddle*                 F2_Middle;
        HandFingerSpreadable*       F2_Finger;

        HandJoint*                  F3_Joint;
        HandKnuckle*                F3_Knuckle;
        HandMiddle*                 F3_Middle;
        HandFingerSpreadable*       F3_Finger;

        //! bounding spheres on HandFinger or HandFingerSpreadable

        BoundingSphere*             F1_ftip;
        BoundingSphere*             F2_ftip;
        BoundingSphere*             F3_ftip;

        BoundingSphere*             F1_fmid;
        BoundingSphere*             F2_fmid;
        BoundingSphere*             F3_fmid;

        BoundingSphere*             F1_fend;
        BoundingSphere*             F2_fend;
        BoundingSphere*             F3_fend;

        //! bounding spheres on HandMiddle

        BoundingSphere*             F1_mmid;
        BoundingSphere*             F2_mmid;
        BoundingSphere*             F3_mmid;

        BoundingSphere*             F1_mend;
        BoundingSphere*             F2_mend;
        BoundingSphere*             F3_mend;

        BoundingSphere*             F1_mfar;
        BoundingSphere*             F2_mfar;
        BoundingSphere*             F3_mfar;

        //! overall hand parameters
        float m_pos[3];
        float m_rot[3];
        float m_scale;

        //buffers for caching and reverting data
        //the "+6" is for extra parameters regarding specific models
        //change accordingly
        float rot_prev[NUM_MODELS * 3 + 3];
        GLdouble* transforms[NUM_BOUNDING_SPHERES];
        GLdouble* transmats[NUM_BOUNDING_SPHERES];

        // parts associated with the hand
        std::vector<VRMLModel* > m_parts;
        // all bounding spheres associated with the hand
        std::vector<BoundingSphere* > m_b_spheres;
        // selected parts on the hand
        std::vector<VRMLModel*> selected_parts;


		// For Kinematics
		bool m_showKinematics;
		BHandKinematics m_kinematics;

		VisualData *visualData;

		GLuint smallSphere;

		GLUquadricObj *smooth;
		GLUquadricObj *flat;

		void createSmallSphere();

		// Finger Tip
		VisualFingerTip *fingerTips[3];

		// Palm
		VisualPalm *palm;

		// Display sensors options
		bool m_displayStrain;
		bool m_displayPPS;

};

#endif // HANDVISUAL_H_INCLUDED
