#include "VRMLModel.h"

// used for assigned values for color picking (static variable)
unsigned int VRMLModel::globalColorID[3] = {0,0,0};

VRMLModel::~VRMLModel(){
    if(partDL)
        glDeleteLists(partDL, 1);
    if(partDL_selected)
        glDeleteLists(partDL_selected, 1);

    m_b_spheres.clear();
}

//! get functions

int VRMLModel::getR() {
    return m_ColorID[0];
}
int VRMLModel::getG() {
    return m_ColorID[1];
}
int VRMLModel::getB() {
    return m_ColorID[2];
}

float* VRMLModel::getRotation() {
    return m_rot;
}

float* VRMLModel::getPosition() {
    return m_pos;
}

float VRMLModel::getScale() {
    return scale;
}

bool VRMLModel::isSelected() {
    return m_selected;
}

bool VRMLModel::isPickable() {
    return m_pickable;
}

//! set functions

void VRMLModel::setRotation(float *rot) {
    m_rot[0] = rot[0];
    m_rot[1] = rot[1];
    m_rot[2] = rot[2];
}

void VRMLModel::setPosition(float *pos) {
    m_pos[0] = pos[0];
    m_pos[1] = pos[1];
    m_pos[2] = pos[2];
}

void VRMLModel::setScale(float amount) {
    scale = amount;
}

//only selectable objects should have this overridden
void VRMLModel::setSelected(bool flag) {
    m_selected = false;
}

//! collision detection functions
void VRMLModel::addBoundingSphere(BoundingSphere* sphere) {
    m_b_spheres.push_back(sphere);
}

void VRMLModel::clearBoundingSpheres() {
    m_b_spheres.clear();
}

//! for color picking
void VRMLModel::CreateColor() {

    //assign model's color to current global color value
    m_ColorID[0] = globalColorID[0];
    m_ColorID[1] = globalColorID[1];
    m_ColorID[2] = globalColorID[2];

    //increment global color value accordingly
    if(globalColorID[0] + 8 > 254)
        globalColorID[0] = 255;
    else
        globalColorID[0] += 8;

    if(globalColorID[0] > 254)
    {
        globalColorID[0] = 0;

        if(globalColorID[1] + 8 > 254)
            globalColorID[1] = 255;
        else
            globalColorID[1] += 8;

        if(globalColorID[1] > 254)
        {
            globalColorID[1] = 255;

            if(globalColorID[2] + 8 > 254)
                globalColorID[2] = 255;
            else
                globalColorID[2] += 8;
            }
        }

        if(globalColorID[2] > 254) {
            globalColorID[2] = 255;
    }
}

//! initializing opengl draw commands
void VRMLModel::InitDL() {
    //compile the openGL draw commands for faster rendering on GPU
    //calling from a list is faster than individual commands (in most cases)
    partDL = glGenLists(1);
    //begin
    glNewList(partDL, GL_COMPILE);
    //draw the model
    DrawPart();
    //end
    glEndList();
}

//! display the object
void VRMLModel::Render() {

    // push current modelview matrix on the stack
    glPushMatrix();
    // translate, rotate, and scale the object accordingly
    glTranslatef(m_pos[0], m_pos[1], m_pos[2]);
    glRotatef(m_rot[0], 1.0, 0.0, 0.0);
    glRotatef(m_rot[1], 0.0, 1.0, 0.0);
    glRotatef(m_rot[2], 0.0, 0.0, 1.0);
    glScalef(scale, scale, scale);

    // call the pre-compiled openGL list of commands
    glCallList(partDL);
    // restore the modelview matrix
    glPopMatrix();
}
