#ifndef VRMLMODEL_H_INCLUDED
#define VRMLMODEL_H_INCLUDED

#include "HandModel.h"

#include <vector>
#include "BoundingSphere.h"

class VRMLModel {

  public:
        virtual ~VRMLModel();
        //! display the model to the wxGLCanvas
        virtual void Render();              //pure virtual

        //! set functions
        virtual void setRotation(float* rot);
        virtual void setPosition(float* pos);
        virtual void setScale(float amount);
        virtual void setSelected(bool flag);

        //! collision detection functions
        virtual void addBoundingSphere(BoundingSphere* sphere);
        virtual void clearBoundingSpheres();

        //! get functions
        virtual float* getRotation();
        virtual float* getPosition();
        virtual float getScale();
        int getR();
        int getG();
        int getB();
        bool isPickable();
        virtual bool isSelected();

  protected:
        //! helper functions for rendering
        virtual void InitDL();
        virtual void DrawPart() = 0;            //pure virtual
        virtual void CreateColor();

        //! two "skins" for selected models
        GLuint partDL;
        GLuint partDL_selected;

        //! for color picking
        static unsigned int globalColorID[3];
        unsigned int m_ColorID[3];

        //! model parameters
        float m_pos[3];
        float m_rot[3];
        float scale;

        //! buffer for model transformation
        GLdouble trans[16];

        //! bounding spheres associated with the model
        std::vector<BoundingSphere *> m_b_spheres;

        bool m_selected;
        bool m_pickable;
};

#endif // VRMLMODEL_H_INCLUDED
