#include "VisualData.h"

#include "BHand.h"

extern bool use280Config;

VisualData* VisualData::m_visualData = NULL;

VisualData::VisualData(BHand *hand) :
	m_hand(hand), tareStrain(true)
{
	for (int i = 0; i < 4; i++)
	{
		strainActual[i] = 0;
		strainTared[i] = 0;
	}
}

VisualData::~VisualData()
{
}

int VisualData::update()
{
	// Average filter for strain
	const int FILTER_SIZE = 4;
	static float strainFiltered[3][FILTER_SIZE] = {
		{ 0.0f, 0.0f, 0.0f, 0.0f },
		{ 0.0f, 0.0f, 0.0f, 0.0f },
		{ 0.0f, 0.0f, 0.0f, 0.0f }
	};
	static int strainFilterIndex = 0;

	int result;

	// Read strain
	int strain[3];
	if (use280Config)
	{
		if ((result = m_hand->RTUpdate("G", "SG", strain)))
			return result;
	}
	else
	{
		for (int i = 0; i < 3; i++)
			strain[i] = ((int)m_hand->RTGetStrain('1' + (char)i)) << 4;
	}

	for (int i = 0; i < 3; i++)
	{
		strainFiltered[i][strainFilterIndex] = (float)strain[i];
		float s = 0;
		for (int n = 0; n < FILTER_SIZE; n++)
			s += strainFiltered[i][n];
		strainActual[i] = s / FILTER_SIZE;
	}
	if (++strainFilterIndex == FILTER_SIZE)
		strainFilterIndex = 0;

	//printf("%6d %6d %6d\n", strain[0], strain[1], strain[2]);

	if (tareStrain)
	{
		//printf("tare strain\n");
		for (int i = 0; i < 3; i++)
			strainTared[i] = strainActual[i];
		tareStrain = false;
	}

	return 0;
}

void VisualData::tareStrainGauges()
{
	tareStrain = true;
}
