#include "VisualPanel.h"

//(*InternalHeaders(VisualPanel)
#include <wx/intl.h>
#include <wx/string.h>
//*)

#include "BHand.h"

#include "Acceleglove_Tab/Glove.h"
#include "Super_Tab/SuperPanel.h"

#include "Visualizer.h"
#include "VisualData.h"

#include <wx/msgdlg.h>


//(*IdInit(VisualPanel)
const long VisualPanel::ID_RTControlStaticBox = wxNewId();
const long VisualPanel::ID_VISUAL_Camera_ToggleButton = wxNewId();
const long VisualPanel::ID_VISUAL_Interact_ToggleButton = wxNewId();
const long VisualPanel::ID_VISUAL_SLIDER1 = wxNewId();
const long VisualPanel::ID_VISUAL_SLIDER2 = wxNewId();
const long VisualPanel::ID_VISUAL_SLIDER3 = wxNewId();
const long VisualPanel::ID_VISUAL_SLIDER4 = wxNewId();
const long VisualPanel::ID_STATICTEXT1 = wxNewId();
const long VisualPanel::ID_STATICTEXT2 = wxNewId();
const long VisualPanel::ID_STATICTEXT3 = wxNewId();
const long VisualPanel::ID_STATICTEXT4 = wxNewId();
const long VisualPanel::ID_VISUAL_GoSupervisory_Button = wxNewId();
const long VisualPanel::ID_VISUAL_RefreshOpenGL_Button = wxNewId();
const long VisualPanel::ID_VISUAL_ShowTrackball_CheckBox = wxNewId();
const long VisualPanel::ID_ACCELEGLOVE_STATICBOX = wxNewId();
const long VisualPanel::ID_CHOICE3 = wxNewId();
const long VisualPanel::ID_ACCELEGLOVE_INITIALIZE = wxNewId();
const long VisualPanel::ID_ACCELEGLOVE_RUN = wxNewId();
const long VisualPanel::ID_ACCELEGLOVE_ABORT = wxNewId();
const long VisualPanel::ID_VISUAL_StartStopRTControl_Button = wxNewId();
const long VisualPanel::ID_LCVMultiplierTextCtrl = wxNewId();
const long VisualPanel::ID_LCVMultiplierStaticText = wxNewId();
const long VisualPanel::ID_RTGainStaticText = wxNewId();
const long VisualPanel::ID_RTGainTextCtrl = wxNewId();
const long VisualPanel::ID_VelDerivGainTextCtrl = wxNewId();
const long VisualPanel::ID_VelPropGainTextCtrl = wxNewId();
const long VisualPanel::ID_VelPGainStaticText = wxNewId();
const long VisualPanel::ID_VelDGainStaticText = wxNewId();
const long VisualPanel::ID_VISUAL_VELOCITY_MODE_RADIOBUTTON = wxNewId();
const long VisualPanel::ID_VISUAL_POSITION_MODE_RADIOBUTTON = wxNewId();
const long VisualPanel::ID_VISUAL_TORQUE_MODE_RADIOBUTTON = wxNewId();
const long VisualPanel::ID_VisualModeStaticText = wxNewId();
const long VisualPanel::ID_VISUAL_Show_Kinematics_CheckBox = wxNewId();
const long VisualPanel::ID_DisplayStaticBox = wxNewId();
const long VisualPanel::ID_VISUAL_Tare_Button = wxNewId();
const long VisualPanel::ID_VISUAL_Display_Strain_CheckBox = wxNewId();
const long VisualPanel::ID_VISUAL_Display_PPS_CheckBox = wxNewId();
//*)


BEGIN_EVENT_TABLE(VisualPanel,wxPanel)
	//(*EventTable(VisualPanel)
	//*)
END_EVENT_TABLE()



///////////////////////////////////////////////////////////////////////////////
// Conversion to and from encoder tick counts and radians
///////////////////////////////////////////////////////////////////////////////

#define MAX_FINGER_ANGLE (140.0 * 3.1416 / 180.0) // in radians
#define MAX_SPREAD_ANGLE (180.0 * 3.1416 / 180.0) // in radians

const int OPEN_FINGER_ENCODER_COUNT_280 = 200000;
const int OPEN_SPREAD_ENCODER_COUNT_280 = 36000;

const int OPEN_FINGER_ENCODER_COUNT_262 = 17500;
const int OPEN_SPREAD_ENCODER_COUNT_262 = 3150;

extern bool use280Config;

int fingerRadiansToEncoderTicks(float angleInRadians)
{
	return use280Config ?
		(int)(OPEN_FINGER_ENCODER_COUNT_280 * angleInRadians / MAX_FINGER_ANGLE) :
		(int)(OPEN_FINGER_ENCODER_COUNT_262 * angleInRadians / MAX_FINGER_ANGLE);
}
float fingerEncoderTicksToRadians(int angleInEncoderTicks)
{
	return use280Config ?
		(MAX_FINGER_ANGLE * (float)angleInEncoderTicks / OPEN_FINGER_ENCODER_COUNT_280) :
		(MAX_FINGER_ANGLE * (float)angleInEncoderTicks / OPEN_FINGER_ENCODER_COUNT_262);
}
int spreadRadiansToEncoderTicks(float angleInRadians)
{
	return use280Config ?
		(int)(OPEN_SPREAD_ENCODER_COUNT_280 * angleInRadians / MAX_SPREAD_ANGLE) :
		(int)(OPEN_SPREAD_ENCODER_COUNT_262 * angleInRadians / MAX_SPREAD_ANGLE);
}
float spreadEncoderTicksToRadians(int angleInEncoderTicks)
{
	return use280Config ?
		(MAX_SPREAD_ANGLE * (float)angleInEncoderTicks / OPEN_SPREAD_ENCODER_COUNT_280) :
		(MAX_SPREAD_ANGLE * (float)angleInEncoderTicks / OPEN_SPREAD_ENCODER_COUNT_262);
}



///////////////////////////////////////////////////////////////////////////////
// VisualPanel Methods
///////////////////////////////////////////////////////////////////////////////

VisualPanel::VisualPanel(wxWindow* parent, BHand *bh, SuperPanel *super_panel) :
	m_hand(bh),
	m_visual_realtime_running(false), m_glove_running(false),
	m_super_panel(super_panel), startStopToggled(false)
{
	//(*Initialize(VisualPanel)
	Create(parent, wxID_ANY, wxDefaultPosition, wxSize(773,384), wxTAB_TRAVERSAL, _T("wxID_ANY"));
	SetMinSize(wxSize(-1,-1));
	SetMaxSize(wxSize(-1,-1));
	RTControlStaticBox = new wxStaticBox(this, ID_RTControlStaticBox, _("RealTime Control"), wxPoint(648,204), wxSize(132,372), 0, _T("ID_RTControlStaticBox"));
	VISUAL_Camera_ToggleButton = new wxToggleButton(this, ID_VISUAL_Camera_ToggleButton, _("Camera"), wxPoint(676,52), wxSize(76,28), 0, wxDefaultValidator, _T("ID_VISUAL_Camera_ToggleButton"));
	VISUAL_Interact_ToggleButton = new wxToggleButton(this, ID_VISUAL_Interact_ToggleButton, _("Interact"), wxPoint(676,84), wxSize(77,28), 0, wxDefaultValidator, _T("ID_VISUAL_Interact_ToggleButton"));
	VISUAL_Slider1 = new wxSlider(this, ID_VISUAL_SLIDER1, 0, 0, 140, wxPoint(656,468), wxSize(19,100), wxSL_VERTICAL|wxSL_BOTH|wxSL_INVERSE|wxNO_BORDER, wxDefaultValidator, _T("ID_VISUAL_SLIDER1"));
	VISUAL_Slider2 = new wxSlider(this, ID_VISUAL_SLIDER2, 0, 0, 140, wxPoint(684,468), wxSize(19,100), wxSL_VERTICAL|wxSL_BOTH|wxSL_INVERSE|wxNO_BORDER, wxDefaultValidator, _T("ID_VISUAL_SLIDER2"));
	VISUAL_Slider3 = new wxSlider(this, ID_VISUAL_SLIDER3, 0, 0, 140, wxPoint(716,468), wxSize(19,100), wxSL_VERTICAL|wxSL_BOTH|wxSL_INVERSE|wxNO_BORDER, wxDefaultValidator, _T("ID_VISUAL_SLIDER3"));
	VISUAL_Slider4 = new wxSlider(this, ID_VISUAL_SLIDER4, 0, 0, 180, wxPoint(744,468), wxSize(19,100), wxSL_VERTICAL|wxSL_BOTH|wxSL_INVERSE|wxNO_BORDER, wxDefaultValidator, _T("ID_VISUAL_SLIDER4"));
	StaticText1 = new wxStaticText(this, ID_STATICTEXT1, _("F1"), wxPoint(656,448), wxDefaultSize, 0, _T("ID_STATICTEXT1"));
	StaticText2 = new wxStaticText(this, ID_STATICTEXT2, _("F2"), wxPoint(684,448), wxDefaultSize, 0, _T("ID_STATICTEXT2"));
	StaticText3 = new wxStaticText(this, ID_STATICTEXT3, _("F3"), wxPoint(716,448), wxDefaultSize, 0, _T("ID_STATICTEXT3"));
	StaticText4 = new wxStaticText(this, ID_STATICTEXT4, _("S"), wxPoint(752,448), wxDefaultSize, 0, _T("ID_STATICTEXT4"));
	VISUAL_GoSupervisory_Button = new wxButton(this, ID_VISUAL_GoSupervisory_Button, _("Go Supervisory"), wxPoint(656,120), wxSize(116,32), 0, wxDefaultValidator, _T("ID_VISUAL_GoSupervisory_Button"));
	VISUAL_RefreshOpenGL_Button = new wxButton(this, ID_VISUAL_RefreshOpenGL_Button, _("Refresh OpenGL"), wxPoint(656,160), wxSize(116,32), 0, wxDefaultValidator, _T("ID_VISUAL_RefreshOpenGL_Button"));
	VISUAL_ShowTrackball_CheckBox = new wxCheckBox(this, ID_VISUAL_ShowTrackball_CheckBox, _("Show Trackball"), wxPoint(648,28), wxSize(144,22), 0, wxDefaultValidator, _T("ID_VISUAL_ShowTrackball_CheckBox"));
	VISUAL_ShowTrackball_CheckBox->SetValue(false);
	ACCELEGLOVE_STATICBOX = new wxStaticBox(this, ID_ACCELEGLOVE_STATICBOX, _("AcceleGlove"), wxPoint(496,548), wxSize(132,106), 0, _T("ID_ACCELEGLOVE_STATICBOX"));
	ACCELEGLOVE_STATICBOX->Hide();
	Portselect = new wxChoice(this, ID_CHOICE3, wxPoint(504,568), wxSize(104,28), 0, 0, 0, wxDefaultValidator, _T("ID_CHOICE3"));
	Portselect->Append(_("USB Port 1"));
	Portselect->Append(_("USB Port 2"));
	Portselect->Append(_("USB Port 3"));
	Portselect->Append(_("USB Port 4"));
	Portselect->Append(_("USB Port 5"));
	Portselect->Append(_("USB Port 6"));
	Portselect->Hide();
	ACCELEGLOVE_INITIALIZE = new wxButton(this, ID_ACCELEGLOVE_INITIALIZE, _("Initialize"), wxPoint(500,588), wxSize(120,32), 0, wxDefaultValidator, _T("ID_ACCELEGLOVE_INITIALIZE"));
	ACCELEGLOVE_INITIALIZE->Hide();
	ACCELEGLOVE_RUN = new wxButton(this, ID_ACCELEGLOVE_RUN, _("Start RT Control"), wxPoint(500,600), wxSize(120,32), 0, wxDefaultValidator, _T("ID_ACCELEGLOVE_RUN"));
	ACCELEGLOVE_RUN->Disable();
	ACCELEGLOVE_RUN->Hide();
	ACCELEGLOVE_ABORT = new wxButton(this, ID_ACCELEGLOVE_ABORT, _("Abort RT Control"), wxPoint(500,616), wxSize(120,32), 0, wxDefaultValidator, _T("ID_ACCELEGLOVE_ABORT"));
	ACCELEGLOVE_ABORT->Disable();
	ACCELEGLOVE_ABORT->Hide();
	VISUAL_StartStopRTControl_Button = new wxToggleButton(this, ID_VISUAL_StartStopRTControl_Button, _("Start RTControl"), wxPoint(656,412), wxSize(116,32), 0, wxDefaultValidator, _T("ID_VISUAL_StartStopRTControl_Button"));
	LCVMultiplierTextCtrl = new wxTextCtrl(this, ID_LCVMultiplierTextCtrl, _("1"), wxPoint(716,308), wxSize(52,21), wxTE_RIGHT, wxDefaultValidator, _T("ID_LCVMultiplierTextCtrl"));
	LCVMultiplierStaticText = new wxStaticText(this, ID_LCVMultiplierStaticText, _("LCVC"), wxPoint(660,312), wxSize(44,17), 0, _T("ID_LCVMultiplierStaticText"));
	RTGainStaticText = new wxStaticText(this, ID_RTGainStaticText, _("RTGain"), wxPoint(660,284), wxDefaultSize, 0, _T("ID_RTGainStaticText"));
	RTGainTextCtrl = new wxTextCtrl(this, ID_RTGainTextCtrl, _("20"), wxPoint(716,280), wxSize(52,21), wxTE_RIGHT, wxDefaultValidator, _T("ID_RTGainTextCtrl"));
	VelDerivGainTextCtrl = new wxTextCtrl(this, ID_VelDerivGainTextCtrl, _("0"), wxPoint(716,252), wxSize(52,21), wxTE_RIGHT, wxDefaultValidator, _T("ID_VelDerivGainTextCtrl"));
	VelDerivGainTextCtrl->SetToolTip(_("Velocity Controller Derivative Gain"));
	VelPropGainTextCtrl = new wxTextCtrl(this, ID_VelPropGainTextCtrl, _("0.05"), wxPoint(716,224), wxSize(52,21), wxTE_RIGHT, wxDefaultValidator, _T("ID_VelPropGainTextCtrl"));
	VelPropGainTextCtrl->SetToolTip(_("Velocity Controller Proportional Gain"));
	VelPGainStaticText = new wxStaticText(this, ID_VelPGainStaticText, _("P Gain"), wxPoint(660,228), wxDefaultSize, 0, _T("ID_VelPGainStaticText"));
	VelDGainStaticText = new wxStaticText(this, ID_VelDGainStaticText, _("D Gain"), wxPoint(660,256), wxDefaultSize, 0, _T("ID_VelDGainStaticText"));
	velocityModeRadioButton = new wxRadioButton(this, ID_VISUAL_VELOCITY_MODE_RADIOBUTTON, _("Velocity"), wxPoint(676,348), wxSize(84,22), 0, wxDefaultValidator, _T("ID_VISUAL_VELOCITY_MODE_RADIOBUTTON"));
	velocityModeRadioButton->SetValue(true);
	positionModeRadioButton = new wxRadioButton(this, ID_VISUAL_POSITION_MODE_RADIOBUTTON, _("Position"), wxPoint(676,368), wxSize(84,22), 0, wxDefaultValidator, _T("ID_VISUAL_POSITION_MODE_RADIOBUTTON"));
	torqueModeRadioButton = new wxRadioButton(this, ID_VISUAL_TORQUE_MODE_RADIOBUTTON, _("Torque"), wxPoint(676,388), wxSize(84,22), 0, wxDefaultValidator, _T("ID_VISUAL_TORQUE_MODE_RADIOBUTTON"));
	VisualModeStaticText = new wxStaticText(this, ID_VisualModeStaticText, _("Visual Mode"), wxPoint(660,332), wxDefaultSize, 0, _T("ID_VisualModeStaticText"));
	VISUAL_Show_Kinematics_CheckBox = new wxCheckBox(this, ID_VISUAL_Show_Kinematics_CheckBox, _("Show Kinematics"), wxPoint(648,9), wxSize(144,22), 0, wxDefaultValidator, _T("ID_VISUAL_Show_Kinematics_CheckBox"));
	VISUAL_Show_Kinematics_CheckBox->SetValue(false);
	DisplayStaticBox = new wxStaticBox(this, ID_DisplayStaticBox, _("Display"), wxPoint(648,580), wxSize(132,72), 0, _T("ID_DisplayStaticBox"));
	VISUAL_Tare_Button = new wxButton(this, ID_VISUAL_Tare_Button, _("Tare"), wxPoint(720,600), wxSize(51,23), 0, wxDefaultValidator, _T("ID_VISUAL_Tare_Button"));
	VISUAL_Tare_Button->SetMinSize(wxSize(-1,-1));
	VISUAL_Display_Strain_CheckBox = new wxCheckBox(this, ID_VISUAL_Display_Strain_CheckBox, _("Strain"), wxPoint(654,600), wxSize(64,22), 0, wxDefaultValidator, _T("ID_VISUAL_Display_Strain_CheckBox"));
	VISUAL_Display_Strain_CheckBox->SetValue(false);
	VISUAL_Display_PPS_CheckBox = new wxCheckBox(this, ID_VISUAL_Display_PPS_CheckBox, _("PPS"), wxPoint(654,628), wxSize(104,22), 0, wxDefaultValidator, _T("ID_VISUAL_Display_PPS_CheckBox"));
	VISUAL_Display_PPS_CheckBox->SetValue(false);

	Connect(ID_VISUAL_SLIDER4,wxEVT_SCROLL_THUMBRELEASE,(wxObjectEventFunction)&VisualPanel::OnVISUAL_Slider4CmdScrollThumbRelease);
	Connect(wxID_ANY,wxEVT_PAINT,(wxObjectEventFunction)&VisualPanel::OnPaint);
	Connect(wxID_ANY,wxEVT_ERASE_BACKGROUND,(wxObjectEventFunction)&VisualPanel::OnEraseBackground);
	//*)


	// Initialization code
	init();


	///////////////////////////////////////////////
	// MANUALLY ADDED EVENT HANDLERS FOR THE GUI //
	///////////////////////////////////////////////
	Connect(ID_VISUAL_ShowTrackball_CheckBox, wxEVT_COMMAND_CHECKBOX_CLICKED, (wxObjectEventFunction)&VisualPanel::OnShowTrackball);
	Connect(ID_VISUAL_Show_Kinematics_CheckBox, wxEVT_COMMAND_CHECKBOX_CLICKED, (wxObjectEventFunction)&VisualPanel::OnShowKinematics);
	//void OnShowTrackball(wxCommandEvent& event);

	Connect(ID_VISUAL_Camera_ToggleButton,wxEVT_COMMAND_TOGGLEBUTTON_CLICKED,(wxObjectEventFunction)&VisualPanel::OnCameraInteractButtonToggle);
	Connect(ID_VISUAL_Interact_ToggleButton,wxEVT_COMMAND_TOGGLEBUTTON_CLICKED,(wxObjectEventFunction)&VisualPanel::OnCameraInteractButtonToggle);

	Connect(ID_VISUAL_GoSupervisory_Button, wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction)&VisualPanel::OnVisualGoSupervisory);
	Connect(ID_VISUAL_RefreshOpenGL_Button, wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction)&VisualPanel::OnVisualRefreshOpenGL);

	Connect(ID_VISUAL_StartStopRTControl_Button, wxEVT_COMMAND_TOGGLEBUTTON_CLICKED, (wxObjectEventFunction)&VisualPanel::OnVisualStartStopRTControl);

	Connect(ID_VISUAL_SLIDER1,wxEVT_COMMAND_SLIDER_UPDATED,(wxObjectEventFunction)&VisualPanel::OnVisualSliderUpdate);
	Connect(ID_VISUAL_SLIDER2,wxEVT_COMMAND_SLIDER_UPDATED,(wxObjectEventFunction)&VisualPanel::OnVisualSliderUpdate);
	Connect(ID_VISUAL_SLIDER3,wxEVT_COMMAND_SLIDER_UPDATED,(wxObjectEventFunction)&VisualPanel::OnVisualSliderUpdate);
	Connect(ID_VISUAL_SLIDER4,wxEVT_COMMAND_SLIDER_UPDATED,(wxObjectEventFunction)&VisualPanel::OnVisualSliderUpdate);

	//! AcceleGlove
	Connect(ID_ACCELEGLOVE_RUN, wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction)&VisualPanel::OnGloveRun);
	Connect(ID_ACCELEGLOVE_ABORT, wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction)&VisualPanel::OnRTAbort);
	Connect(ID_CHOICE3, wxEVT_COMMAND_CHOICE_SELECTED, (wxObjectEventFunction)&VisualPanel::OnPortSelect);
	Connect(ID_ACCELEGLOVE_INITIALIZE, wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction)&VisualPanel::OnGloveInitialize);

	// New
	Connect(ID_VISUAL_Tare_Button, wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction)&VisualPanel::OnVisualTare);
	Connect(ID_VISUAL_Display_Strain_CheckBox, wxEVT_COMMAND_CHECKBOX_CLICKED, (wxObjectEventFunction)&VisualPanel::OnDisplayStrain);
	Connect(ID_VISUAL_Display_PPS_CheckBox, wxEVT_COMMAND_CHECKBOX_CLICKED, (wxObjectEventFunction)&VisualPanel::OnDisplayPPS);
}

VisualPanel::~VisualPanel()
{
	delete m_visualControl;
	delete m_opengl;
	//(*Destroy(VisualPanel)
	//*)
}

void VisualPanel::init()
{
	refreshOpenGLModel = false;

	// Initialize Visualizer
	m_opengl = new Visualizer(this, VISUAL_Slider1, VISUAL_Slider2, VISUAL_Slider3, VISUAL_Slider4);
	m_opengl->ShowTrackball(false);

	setVisualCameraMode(false);

	m_visualControl = new VisualControl(this, m_hand);
}


void VisualPanel::EnablePanel(bool flag)
{
	//VISUAL_Show_Kinematics_CheckBox->Enable(flag);
	//VISUAL_ShowTrackball_CheckBox->Enable(flag);
	VISUAL_GoSupervisory_Button->Enable(flag);
	VISUAL_RefreshOpenGL_Button->Enable(flag);

	RTControlStaticBox->Enable(flag);
	VelPGainStaticText->Enable(flag);
	VelDGainStaticText->Enable(flag);
	RTGainStaticText->Enable(flag);
	LCVMultiplierStaticText->Enable(flag);
	VisualModeStaticText->Enable(flag);

	VelPropGainTextCtrl->Enable(flag);
	VelDerivGainTextCtrl->Enable(flag);
	RTGainTextCtrl->Enable(flag);
	LCVMultiplierTextCtrl->Enable(flag);

	velocityModeRadioButton->Enable(flag);
	positionModeRadioButton->Enable(flag);
	torqueModeRadioButton->Enable(flag);
	VISUAL_StartStopRTControl_Button->Enable(flag);

	StaticText1->Enable(flag);
	StaticText2->Enable(flag);
	StaticText3->Enable(flag);
	StaticText4->Enable(flag);

	VISUAL_Slider1->Enable(flag);
	VISUAL_Slider2->Enable(flag);
	VISUAL_Slider3->Enable(flag);
	VISUAL_Slider4->Enable(flag);

	//DisplayStaticBox->Enable(flag);

	VISUAL_Display_Strain_CheckBox->Enable(flag);
	//VISUAL_Display_PPS_CheckBox->Enable(flag);
	VISUAL_Tare_Button->Enable(flag);
}


void VisualPanel::setVisualCameraMode(bool flag)
{
	// Sets Visual/Camera mode
	if (flag)
	{
		VISUAL_Camera_ToggleButton->SetValue(true);
		VISUAL_Interact_ToggleButton->SetValue(false);
	}
	else
	{
		VISUAL_Camera_ToggleButton->SetValue(false);
		VISUAL_Interact_ToggleButton->SetValue(true);
	}

	m_opengl->setVisualizerMode(flag);
}

void VisualPanel::OnCameraInteractButtonToggle(wxCommandEvent& event)
{
	// Toggles between camera and interact mode
	setVisualCameraMode(event.GetId() == ID_VISUAL_Camera_ToggleButton);
}


void VisualPanel::OnVisualSliderUpdate(wxCommandEvent& event)
{
	// Update the hand visual when slider positions change
	float f1 = VISUAL_Slider1->GetValue();
	float f2 = VISUAL_Slider2->GetValue();
	float f3 = VISUAL_Slider3->GetValue();
	float spread = VISUAL_Slider4->GetValue();

	m_opengl->RefreshOpenGL(f1, f2, f3, spread);
	m_opengl->SetFocus();
}

void VisualPanel::OnVISUAL_Slider4CmdScrollThumbRelease(wxScrollEvent& event)
{
	// Update the actual hand position to if in RealTime mode
	float f4_rot = m_opengl->getSpreadRot() * 3.1416f / 180.0f;
	int spread_pos_at_release = spreadRadiansToEncoderTicks(f4_rot);

	MoveSpread(spread_pos_at_release);
}


void VisualPanel::OnShowTrackball(wxCommandEvent& event)
{
	// enable/disable display of trackball in visual mode
	m_opengl->ShowTrackball(VISUAL_ShowTrackball_CheckBox->GetValue());
}

void VisualPanel::OnShowKinematics(wxCommandEvent& event)
{
	m_opengl->ShowKinematics(VISUAL_Show_Kinematics_CheckBox->GetValue());
}

void VisualPanel::OnDisplayStrain(wxCommandEvent& event)
{
	m_visualControl->updateStrain(VISUAL_Display_Strain_CheckBox->GetValue());
	m_opengl->DisplayStrain(VISUAL_Display_Strain_CheckBox->GetValue());

	// Make the user check/uncheck chechboxes before starting visual mode
	/*if (!use280Config && m_visualControl->InRealTimeMode() && VISUAL_Display_Strain_CheckBox->GetValue())
	{
		m_visualControl->StopRealTimeMode();
		StopRTControl();

		// Start RT control again
		m_opengl->SetFocus();
		SetFocus();

		if (PrepareRT())
		{
			VISUAL_StartStopRTControl_Button->SetLabel(wxT("Abort RTControl"));
			VISUAL_StartStopRTControl_Button->SetValue(true);
			startStopToggled = true;

			VISUAL_GoSupervisory_Button->Enable(false);
			VISUAL_RefreshOpenGL_Button->Enable(false);

			m_visual_realtime_running = true;
		}
	}*/
}

void VisualPanel::OnDisplayPPS(wxCommandEvent& event)
{
	m_visualControl->updatePPS(VISUAL_Display_PPS_CheckBox->GetValue());
	m_opengl->DisplayPPS(VISUAL_Display_PPS_CheckBox->GetValue());

	// Make the user check/uncheck chechboxes before starting visual mode
	/*if (use280Config && m_visualControl->InRealTimeMode() && VISUAL_Display_PPS_CheckBox->GetValue())
	{
		m_visualControl->StopRealTimeMode();
		StopRTControl();

		m_hand->Set("GS", "LFPPS", 1);

		// Start RT control again
		m_opengl->SetFocus();
		SetFocus();

		if (PrepareRT())
		{
			VISUAL_StartStopRTControl_Button->SetLabel(wxT("Abort RTControl"));
			VISUAL_StartStopRTControl_Button->SetValue(true);
			startStopToggled = true;

			VISUAL_GoSupervisory_Button->Enable(false);
			VISUAL_RefreshOpenGL_Button->Enable(false);

			m_visual_realtime_running = true;
		}
	}
	else if (use280Config)
	{
		m_hand->Set("GS", "LFPPS", VISUAL_Display_PPS_CheckBox->GetValue() ? 1 : 0);
	}*/
}


void VisualPanel::OnVisualGoSupervisory(wxCommandEvent &event)
{
	VISUAL_StartStopRTControl_Button->Enable(false);

	// Create supervisory command to send so that the hand matches the visual
	float f1_rot = m_opengl->getFinger1Rot() * 3.1416f / 180.0f;
	float f2_rot = m_opengl->getFinger2Rot() * 3.1416f / 180.0f;
	float f3_rot = m_opengl->getFinger3Rot() * 3.1416f / 180.0f;
	float f4_rot = m_opengl->getSpreadRot() * 3.1416f / 180.0f;

	int f1_val = fingerRadiansToEncoderTicks(f1_rot);
	int f2_val = fingerRadiansToEncoderTicks(f2_rot);
	int f3_val = fingerRadiansToEncoderTicks(f3_rot);
	int f4_val = spreadRadiansToEncoderTicks(f4_rot);

	// Move hand to match the openGL visualization
	m_hand->GoToDifferentPositions(f1_val, f2_val, f3_val, f4_val);

	m_opengl->SetFocus();
	VISUAL_StartStopRTControl_Button->Enable(true);
}


void VisualPanel::RefreshOpenGL()
{
	//VISUAL_StartStopRTControl_Button->Enable(false);

	int f1, f2, f3, spread;
	if (!m_hand->Get("1", "P", &f1) && !m_hand->Get("2", "P", &f2) &&
		!m_hand->Get("3", "P", &f3) && !m_hand->Get("S", "P", &spread))
		m_opengl->RefreshOpenGL(
			fingerEncoderTicksToRadians(f1) * 180.0f / 3.1416f,
			fingerEncoderTicksToRadians(f2) * 180.0f / 3.1416f,
			fingerEncoderTicksToRadians(f3) * 180.0f / 3.1416f,
			spreadEncoderTicksToRadians(spread) * 180.0f / 3.1416f);

	//VISUAL_StartStopRTControl_Button->Enable(true);
}


void VisualPanel::OnVisualRefreshOpenGL(wxCommandEvent &event)
{
	// Redraw the openGL visualization to match the physical hand's orientation
    RefreshOpenGL();
    m_opengl->SetFocus();
}


bool VisualPanel::PrepareRT()
{
	if (m_visualControl->InRealTimeMode())
		return false;

	// Read and save PD gains from textboxes
	double velPGain;
	double velDGain;
	if (!(VelPropGainTextCtrl->GetValue()).ToDouble(&velPGain))
		return false;
	if (!(VelDerivGainTextCtrl->GetValue()).ToDouble(&velDGain))
		return false;

	// Read and save RealTime Gain and LCV Multiplier from textboxes
	long realtimeGain;
	long LCVMultiplier;
	if (!(RTGainTextCtrl->GetValue()).ToLong(&realtimeGain))
		return false;
	if (!(LCVMultiplierTextCtrl->GetValue()).ToLong(&LCVMultiplier))
		return false;

	m_visualControl->setParameters(velPGain, velDGain, realtimeGain, LCVMultiplier);

	if (use280Config)
	{
		// Check for setting different visual modes
		if (velocityModeRadioButton->GetValue()) m_visualControl->setVisualMode(BHVisualVelocityMode);
		else if (positionModeRadioButton->GetValue()) m_visualControl->setVisualMode(BHVisualPositionMode);
		else if (torqueModeRadioButton->GetValue()) m_visualControl->setVisualMode(BHVisualTorqueMode);

		// Setup PPS feedback
		bool ppsFeedback = VISUAL_Display_PPS_CheckBox->GetValue();
		m_visualControl->updatePPS(ppsFeedback);
		m_opengl->DisplayPPS(ppsFeedback);
	}

	// Set opengl_positions
	float opengl_pos[4];

	opengl_pos[0] = m_opengl->getFinger1Rot() * 3.1416f / 180.0f; // In degrees
	opengl_pos[1] = m_opengl->getFinger2Rot() * 3.1416f / 180.0f;
	opengl_pos[2] = m_opengl->getFinger3Rot() * 3.1416f / 180.0f;
	opengl_pos[3] = m_opengl->getSpreadRot() * 3.1416f / 180.0f;

	m_visualControl->setOpenGLPos(opengl_pos);

	m_visualControl->StartRealTimeMode();

	return true;
}


void VisualPanel::OnVisualStartStopRTControl(wxCommandEvent &event)
{
	if (!startStopToggled) // Right now there is a bug with wxToggleButton and GetValue() otherwise I would use VISUAL_StartStopRTControl_Button->GetValue() instead
	{
		m_opengl->SetFocus();
		SetFocus();

		if (PrepareRT())
		{
			VISUAL_StartStopRTControl_Button->SetLabel(wxT("Abort RTControl"));
			VISUAL_StartStopRTControl_Button->SetValue(true);
			startStopToggled = true;

			VISUAL_GoSupervisory_Button->Enable(false);
			VISUAL_RefreshOpenGL_Button->Enable(false);

			m_visual_realtime_running = true;
		}
	}
	else
	{
		m_visualControl->StopRealTimeMode();
		StopRTControl();
	}
}


void VisualPanel::StopRTControl()
{
	if (m_visual_realtime_running)
	{
		VISUAL_StartStopRTControl_Button->SetLabel(wxT("Start RTControl"));
		VISUAL_StartStopRTControl_Button->SetValue(false);

		VISUAL_GoSupervisory_Button->Enable(true);
		VISUAL_RefreshOpenGL_Button->Enable(true);
		startStopToggled = false;
		m_visual_realtime_running = false;
	}
}


// These are events are implemented to try to remove a flicker with the OpenGL widget in Linux
void VisualPanel::OnEraseBackground(wxEraseEvent &event) { event.Skip(); } // doesn't help
void VisualPanel::OnPaint(wxPaintEvent &event)           { event.Skip(); } // doesn't help


void VisualPanel::OnGloveRun(wxCommandEvent &event)
{
	// AcceleGlove
	//if (m_glove->PrepareRT())
	//	m_glove_running = true;
	// TODO: Enable/Disable buttons
}

void VisualPanel::OnRTAbort(wxCommandEvent& WXUNUSED(event))
{
	// AcceleGlove
	//m_visualControl->StopRealTimeMode();
}


void VisualPanel::OnPortSelect(wxCommandEvent &event)
{
	// AcceleGlove
	/*wxString choice = Portselect->GetStringSelection();

	if      (!choice.Cmp(wxT("USB Port 1"))) m_glove->PortSelect(1);
	else if (!choice.Cmp(wxT("USB Port 2"))) m_glove->PortSelect(2);
	else if (!choice.Cmp(wxT("USB Port 3"))) m_glove->PortSelect(3);
	else if (!choice.Cmp(wxT("USB Port 4"))) m_glove->PortSelect(4);
	else if (!choice.Cmp(wxT("USB Port 5"))) m_glove->PortSelect(5);
	else if (!choice.Cmp(wxT("USB Port 6"))) m_glove->PortSelect(6);*/
}

void VisualPanel::OnGloveInitialize(wxCommandEvent &event)
{
	// AcceleGlove
	/*if (m_glove->Initialize())
	{
		ACCELEGLOVE_INITIALIZE->Disable();
		ACCELEGLOVE_RUN->Enable();
		ACCELEGLOVE_ABORT->Enable();
	}
	else
	{
		wxMessageDialog response(NULL, wxT("Could not initialize the AcceleGlove with port specified"), wxT("AcceleGlove - Error"), wxOK | wxICON_ERROR);
		response.ShowModal();
	}*/
}

void VisualPanel::OnVisualizerRightMouseButton(bool up)
{
	static int spread_pos_at_press = 0;

	if (m_visual_realtime_running)
	{
		if (!up)
		{
			// User pressed right mouse button in visualizer window
			// Save spread position
			float f4_rot = m_opengl->getSpreadRot() * 3.1416f / 180.0f;
			spread_pos_at_press = spreadRadiansToEncoderTicks(f4_rot);
		}
		else
		{
			// User released right mouse button in visualizer window
			float f4_rot = m_opengl->getSpreadRot() * 3.1416f / 180.0f;
			int spread_pos_at_release = spreadRadiansToEncoderTicks(f4_rot);

			if (spread_pos_at_release != spread_pos_at_press)
				MoveSpread(spread_pos_at_release);
		}
	}
}

void VisualPanel::OnVisualTare(wxCommandEvent& event)
{
	VisualData::getInstance()->tareStrainGauges();
}

void VisualPanel::MoveSpread(int newSpreadPosition)
{
	if (m_visual_realtime_running && !use280Config)
	{
		wxPuts(wxString::Format(wxT("Moving spread to %d\n"), newSpreadPosition));
		m_visualControl->MoveSpread(newSpreadPosition);
	}
}


void VisualPanel::Service()
{
	// See if Visual is running in RealTime mode
	if (m_visual_realtime_running)
	{
		// Set opengl_positions
		float opengl_pos[4];

		opengl_pos[0] = m_opengl->getFinger1Rot() * 3.1416f / 180.0f; // Convert to radians
		opengl_pos[1] = m_opengl->getFinger2Rot() * 3.1416f / 180.0f;
		opengl_pos[2] = m_opengl->getFinger3Rot() * 3.1416f / 180.0f;
		opengl_pos[3] = m_opengl->getSpreadRot() * 3.1416f / 180.0f;

		m_visualControl->setOpenGLPos(opengl_pos);

		// Necessary to catch errors that must abort RealTime mode
		m_visualControl->Service();
	}

	// This must be called if the glove needs to work (Visual should be passed a pointer to the glove to call this)
	//if (m_glove_running)
	//	m_glove->RunOnce();
	if (refreshOpenGLModel)
	{
		// Redraw the openGL visualization to match the physical hand's orientation
		RefreshOpenGL();
		m_opengl->SetFocus();
		refreshOpenGLModel = false;
	}
}

void VisualPanel::EnterTab()
{
	if (use280Config)
	{
		RTGainTextCtrl->Enable(0);
		LCVMultiplierTextCtrl->Enable(0);
	}
	else
	{
		RTGainTextCtrl->Enable(1);
		LCVMultiplierTextCtrl->Enable(1);

		positionModeRadioButton->Enable(0);
		torqueModeRadioButton->Enable(0);
	}

	refreshOpenGLModel = true;
}

void VisualPanel::LeaveTab()
{
	// Stop RealTime control before leaving the Visual Tab in GUI
	if (m_visual_realtime_running)
	{
		m_visualControl->StopRealTimeMode();
		StopRTControl();
	}
	m_hand->Command("T");
}
