#ifndef VISUALPANEL_H
#define VISUALPANEL_H

//(*Headers(VisualPanel)
#include <wx/stattext.h>
#include <wx/textctrl.h>
#include <wx/checkbox.h>
#include <wx/tglbtn.h>
#include <wx/radiobut.h>
#include <wx/statbox.h>
#include <wx/slider.h>
#include <wx/panel.h>
#include <wx/choice.h>
#include <wx/button.h>
//*)

#include "VisualControl.h"

#include "Common/tmr.h"


int fingerRadiansToEncoderTicks(float angleInRadians);
float fingerEncoderTicksToRadians(int angleInEncoderTicks);
int spreadRadiansToEncoderTicks(float angleInRadians);
float spreadEncoderTicksToRadians(int angleInEncoderTicks);


class wxMessageDialog;

class BHand;
class Visualizer;
class SuperPanel;
//class Glove;


class VisualPanel : public wxPanel
{
public:
	VisualPanel(wxWindow* parent, BHand *bh, SuperPanel* super_panel);
	virtual ~VisualPanel();

	Visualizer * getOpenGLVisual() { return m_opengl; }

	void EnablePanel(bool flag);

	void Service();

	void EnterTab();
	void LeaveTab();

	void OnVisualizerRightMouseButton(bool up);

	bool IsRunning() { return m_visual_realtime_running; }

	// Methods Available for VisualControl
	void StopRTControl();

protected:

	//(*Identifiers(VisualPanel)
	static const long ID_RTControlStaticBox;
	static const long ID_VISUAL_Camera_ToggleButton;
	static const long ID_VISUAL_Interact_ToggleButton;
	static const long ID_VISUAL_SLIDER1;
	static const long ID_VISUAL_SLIDER2;
	static const long ID_VISUAL_SLIDER3;
	static const long ID_VISUAL_SLIDER4;
	static const long ID_STATICTEXT1;
	static const long ID_STATICTEXT2;
	static const long ID_STATICTEXT3;
	static const long ID_STATICTEXT4;
	static const long ID_VISUAL_GoSupervisory_Button;
	static const long ID_VISUAL_RefreshOpenGL_Button;
	static const long ID_VISUAL_ShowTrackball_CheckBox;
	static const long ID_ACCELEGLOVE_STATICBOX;
	static const long ID_CHOICE3;
	static const long ID_ACCELEGLOVE_INITIALIZE;
	static const long ID_ACCELEGLOVE_RUN;
	static const long ID_ACCELEGLOVE_ABORT;
	static const long ID_VISUAL_StartStopRTControl_Button;
	static const long ID_LCVMultiplierTextCtrl;
	static const long ID_LCVMultiplierStaticText;
	static const long ID_RTGainStaticText;
	static const long ID_RTGainTextCtrl;
	static const long ID_VelDerivGainTextCtrl;
	static const long ID_VelPropGainTextCtrl;
	static const long ID_VelPGainStaticText;
	static const long ID_VelDGainStaticText;
	static const long ID_VISUAL_VELOCITY_MODE_RADIOBUTTON;
	static const long ID_VISUAL_POSITION_MODE_RADIOBUTTON;
	static const long ID_VISUAL_TORQUE_MODE_RADIOBUTTON;
	static const long ID_VisualModeStaticText;
	static const long ID_VISUAL_Show_Kinematics_CheckBox;
	static const long ID_DisplayStaticBox;
	static const long ID_VISUAL_Tare_Button;
	static const long ID_VISUAL_Display_Strain_CheckBox;
	static const long ID_VISUAL_Display_PPS_CheckBox;
	//*)

private:

	BHand *m_hand;

	VisualControl *m_visualControl;
	VisualData *m_visualData;

	bool m_visual_realtime_running;
	bool m_glove_running;

	SuperPanel *m_super_panel;

	bool startStopToggled;

	Visualizer *m_opengl;

	//Glove *m_glove;

	bool refreshOpenGLModel;

	// Private methods
	void init();

	void setVisualCameraMode(bool flag);

	bool PrepareRT();
	void RefreshOpenGL();

	void MoveSpread(int newSpreadPosition);

	void OnVisualTare(wxCommandEvent& event);


	//!lock/unlock Run/Abort buttons for Real Time mode
	void SetRTButtons(bool isRunning);

	//! Visual
	void OnShowTrackball(wxCommandEvent& event);
	void OnShowKinematics(wxCommandEvent& event);
	void OnDisplayStrain(wxCommandEvent& event);
	void OnDisplayPPS(wxCommandEvent& event);
	//void OnVisualSettings(wxCommandEvent& event);
	void OnVisualGoSupervisory(wxCommandEvent& event);
	void OnVisualRefreshOpenGL(wxCommandEvent& event);
	void OnVisualStartStopRTControl(wxCommandEvent& event);
	void OnRTAbort(wxCommandEvent& event);

	//! AcceleGlove
	void OnGloveRun(wxCommandEvent& event);
	void OnGloveAbort(wxCommandEvent& event);
	void OnPortSelect(wxCommandEvent& event);
	void OnGloveInitialize(wxCommandEvent& event);

	//(*Handlers(VisualPanel)
	void OnCameraInteractButtonToggle(wxCommandEvent& event);
	void OnVisualSliderUpdate(wxCommandEvent& event);
	void OnEraseBackground(wxEraseEvent& event);
	void OnPaint(wxPaintEvent& event);
	void OnVISUAL_Slider4CmdScrollThumbRelease(wxScrollEvent& event);
	void OnVISUAL_Show_Kinematics_CheckBoxClick(wxCommandEvent& event);
	//*)

	//(*Declarations(VisualPanel)
	wxSlider* VISUAL_Slider4;
	wxTextCtrl* RTGainTextCtrl;
	wxStaticText* VisualModeStaticText;
	wxStaticText* VelDGainStaticText;
	wxToggleButton* VISUAL_StartStopRTControl_Button;
	wxCheckBox* VISUAL_ShowTrackball_CheckBox;
	wxToggleButton* VISUAL_Camera_ToggleButton;
	wxStaticText* StaticText2;
	wxCheckBox* VISUAL_Display_PPS_CheckBox;
	wxButton* VISUAL_Tare_Button;
	wxRadioButton* torqueModeRadioButton;
	wxSlider* VISUAL_Slider1;
	wxTextCtrl* VelDerivGainTextCtrl;
	wxStaticText* VelPGainStaticText;
	wxButton* ACCELEGLOVE_RUN;
	wxStaticText* StaticText1;
	wxStaticBox* DisplayStaticBox;
	wxStaticText* StaticText3;
	wxChoice* Portselect;
	wxRadioButton* positionModeRadioButton;
	wxRadioButton* velocityModeRadioButton;
	wxTextCtrl* VelPropGainTextCtrl;
	wxButton* VISUAL_GoSupervisory_Button;
	wxStaticBox* RTControlStaticBox;
	wxCheckBox* VISUAL_Display_Strain_CheckBox;
	wxButton* ACCELEGLOVE_ABORT;
	wxCheckBox* VISUAL_Show_Kinematics_CheckBox;
	wxStaticText* StaticText4;
	wxSlider* VISUAL_Slider2;
	wxButton* ACCELEGLOVE_INITIALIZE;
	wxButton* VISUAL_RefreshOpenGL_Button;
	wxSlider* VISUAL_Slider3;
	wxToggleButton* VISUAL_Interact_ToggleButton;
	wxStaticText* RTGainStaticText;
	wxTextCtrl* LCVMultiplierTextCtrl;
	wxStaticText* LCVMultiplierStaticText;
	wxStaticBox* ACCELEGLOVE_STATICBOX;
	//*)

	DECLARE_EVENT_TABLE()
};



#endif
