// NOTE: To run, it is recommended not to be in Compiz or Beryl, they have shown some instability.

#ifndef Visualizer_H
#define Visualizer_H

#pragma warning ( disable : 4005 )

#include "GL/gl.h"
// According to http://www.mingw.org/wiki/HOWTO_Use_Mark_J_Kilgards_OpenGL_Utility_Toolkit_GLUT_with_MinGW
// these macros should be defined before including <GL/glut.h>
#define GLUT_NO_LIB_PRAGMA
#define GLUT_NO_WARNING_DISABLE

//#define _STDCALL_SUPPORTED
//#define _M_IX86
#define GLUT_DISABLE_ATEXIT_HACK
#ifndef LINUX
#include <windows.h>
#endif
#include <GL/glut.h>
#include <wx/glcanvas.h>
#include <math.h>
#include <vector>

#ifndef WIN32
#include <unistd.h> // FIXME: ¿This work/necessary in Windows?
                    //Not necessary, but if it was, it needs to be replaced by process.h AND io.h
#endif

class Vector4;
class HandManager;
class VRMLModel;
class BHand;
class wxSlider;


class Visualizer: public wxGLCanvas
{
	public:
		Visualizer(wxWindow* parent, wxSlider* s1, wxSlider* s2, wxSlider* s3, wxSlider* s4);
		~Visualizer();

		//! events
		void OnPaint(wxPaintEvent& event);
		void OnIdle(wxIdleEvent & event);
		void OnMouse(wxMouseEvent & event);
		void OnEraseBackground(wxEraseEvent & event);
		void OnEnterWindow(wxMouseEvent & event);

		//! functions
		void setVisualizerMode(const bool camera);            //set to camera or interact
		void ShowTrackball(const bool flag);

		//! redraw openGL hand to values specified (in degrees)
		void RefreshOpenGL(float f1, float f2, float f3, float spread);

		float getFinger1Rot();
		float getFinger2Rot();
		float getFinger3Rot();
		float getSpreadRot();

		void ShowKinematics(bool show);
		void DisplayStrain(bool display);
		void DisplayPPS(bool display);

	protected:
		//! regarding rendering
		void Init() const;
		void Render() const;
		void UpdateCamera();
		void ToggleColorPickingOn(const bool flag);
		void LoadParts();

	private:
		bool init;
		bool colorPicking;

		bool visual_camera;
		bool visual_interact;

		//! mouse position variables

		//varaibles in wxGLCanvas-space
		float current_x;
		float current_y;
		float old_x;
		float old_y;

		//variables in absolute-screen-space
		float current_x_absolute;
		float current_y_absolute;
		float old_x_absolute;
		float old_y_absolute;

		//zooming for camera
		float zoom;

		wxSlider* m_sliders[4];

		//! color picking variables
		VRMLModel* selected_part;
		HandManager* hand_model;

		DECLARE_EVENT_TABLE()
};

#endif
