///////////////////////////////////////////////////////////
//                                                       //
//           Automatically Generated C++ Code            //
//           BHand Control Center Version 1.0            //
//                                                       //
//                   RealTime Mode                       //
//                                                       //
//      Force Response Program                           //
//      v1.0 990803 dr - Created program                 //
//                                                       //
///////////////////////////////////////////////////////////



#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#ifndef LINUX
#include <conio.h>
#endif
#include "BHand.h"

// Control F2 with velocities or torques (see descriptions below)
#define USE_VELOCITY_MODE

#ifdef USE_VELOCITY_MODE

/*
	The ForceResponse demo will use RealTime Velocity control.  It will also
	show one method of estimating the desired motor torque based on the
	proportional velocity control equation for the Motor Command Output
	defined in the HCTL-1100 datasheet:

	   MCn = (K/4)*Yn

	K depends on the Boolean LCPG flag.  If LCPG is true than K is equal to the
	loop control proportional gain, otherwise K is equal to the value of the
	FPG property.  Yn is the velocity error equal to control velocity
	multiplied by the loop control velocity coefficient minus the actual velocity.
 */

int lcvc[4] = { 5, 5, 5, 5 };
int fpg[4] = { 10, 10, 10, 10 };
int lfvc[4] = { 5, 5, 5, 5 };
bool lcpg[4] = { 0, 0, 0, 0 };
int samplereg[4];
double sampleperiod[4];

/*
 See code below to see how MCn is estimated in RealTime Velocity mode.  Several
 Force measurements were taken with various desired control velocities. These were
 graphed to scatter plot with OpenOffice and the equation of best fit to the
 data was:

	f(x) = 0.0000311x + 0.0001523 ~ 0.0000311x

 where the input is estimated MCn and the output is Torque in Nm.
 The following is a method that will return an estimate of the desired torque
 with units of Nm given an estimate of MCn.
 */
float rtTorque(int mcn)
{
	// Return estimated motor torque applied in units of Nm
	if (mcn > 0)
		return (float)mcn * 0.0003419f + 0.0016749;
	else if (mcn < 0)
		return (float)mcn * 0.0003419f - 0.0016749;
	else
		return 0;
}

#else
/*
	This ForceResponse demo uses the RealTime Torque control of the 262 hand.
	This torque control is implemented such that commanded motor position is
	set to actual position plus the desired torque.

	Hand motors apply torque of 0.12 N @ 11 inches for every 300 units passed to RTTorque
	A lever arm was attached to the motor output shaft, the motor was commanded
	between 0 and 750 units of RT torque, and the force was measured at a distance
	of 11 inches.  The force sensor used had a resolution of 0.01 N and motor stall
	was at 0.16 N at 11 inches with an RTTorque of 450.

	Check:
	Equation given by OpenOffice Calc:
		F(t) = 0.0004t - 0.0005 ~ 0.0004t    or
		t = F(t) / (0.0004 N(11 in))
		t = F(t) / (0.0004 N(11 in)(0.2794 m/11 in)
		t = F(t) / (0.00011176 Nm)
 */
int rtTorque(float torqueNm) { return (int)(torqueNm / 0.00011176f); }
void setRTTorque(BHand &bh, int motorNum, float torqueNm)
{
	if (motorNum < 3)
	{
		const float move140DegPerMotorRevs = 17500.0f / 360.0f;
		float ratio = move140DegPerMotorRevs * 360.0f / 140.0f; // 125
		bh.RTSetTorque('1' + motorNum, rtTorque(torqueNm / ratio)); // before breakaway
		//bh.RTSetTorque('1' + motorNum, rtTorque(torqueNm / 375)); // after breakaway
		printf("raw = %6d  ", rtTorque(torqueNm / ratio));
	}
	else
	{
		const float move180DegPerMotorRevs = 3150.0f / 360.0f;
		float ratio = move180DegPerMotorRevs * 360.0f / 180.0f; // 17.5
		bh.RTSetTorque('1' + motorNum, rtTorque(torqueNm / ratio));
	}
}

#endif

#ifdef LINUX
    #include <sys/time.h> // Needed for ms time.

    struct HiResTimer {
        void Start(void) { gettimeofday(&mTimeStart, NULL); };
        double GetElapsedInMilliSecs(void)
        {
            struct timeval mTimeNow;
            gettimeofday(&mTimeNow, NULL);
            double duration = (double)(mTimeNow.tv_sec - mTimeStart.tv_sec) * 1000 + (double)(mTimeNow.tv_usec - mTimeStart.tv_usec) / 1000;
            return duration;
        }

        struct timeval mTimeStart;
    };

#else
    #include <Windows.h>

    struct HiResTimer {
        void Start(void) { QueryPerformanceCounter(&mTimeStart); };
        double GetElapsedInMilliSecs(void)
        {
            LARGE_INTEGER mTimeNow;
            QueryPerformanceCounter(&mTimeNow);
            LARGE_INTEGER freq;
            QueryPerformanceFrequency(&freq);
            double duration = 1000 * (double)(mTimeNow.QuadPart-mTimeStart.QuadPart)/(double)freq.QuadPart;
            return duration;
        }

        LARGE_INTEGER mTimeStart;
    };

#endif


BHand   bh;               // Handles all hand communication
int     value[3];         // Hand parameter obtained with Get
int     result;           // Return value (error) of all BHand calls
struct HiResTimer tmr;


///////////////////////////////////////////////////////////
// Error Handler - called whenever result!=0
// Feel free to customize this function

void Error()
{
	printf(" ERROR: %d\n%s\n", result, bh.ErrorMessage(result));
	fflush(stdout);
	while (1);
	//exit(0);
}


///////////////////////////////////////////////////////////
//  Initialize hand, set timeouts and baud rate

void Initialize()
{
	int port;

	printf("\n\n\r\t\tInitializing Software...");
	printf("\n\n\r\t\tPlease enter desired serial port> ");
	scanf("%d", &port);
	printf("\n");

	if (result=bh.InitSoftware(port,THREAD_PRIORITY_TIME_CRITICAL))
		Error();

	if (result=bh.ComSetTimeouts(100,15000,100,5000))
		Error();

	//if (result=bh.Baud(38400))
	if (result=bh.Baud(9600))
		Error();

	// Set better filter parameters for spread
	char FIP[4] = "FIP";
	char FDZ[4] = "FDZ";
	char motor4[2] = "4";

	if (result=bh.Set(motor4, FIP, 0))
		Error();

	if (result=bh.Set(motor4, FDZ, 251))
		Error();

	if (result=bh.Command("4FSET MPE 1000"))
		Error();

	printf("\r\t\tInitializing Hand...           ");

	if( result=bh.InitHand("") )
		Error();
}


///////////////////////////////////////////////////////////
//  Set parameters

void PrepareRealTime()
{
#ifdef USE_VELOCITY_MODE
	if (result=bh.RTSetFlags("1", 0, lcvc[0], 0, 0, 0, lfvc[0], 1, 0, 0, 0, 0, 0, 0, 0))
		Error();

	// Velocity mode
	for (int i = 0; i < 4; i++)
	{
		char motor[] = "1";
		motor[0] += i;

		if (result=bh.Set(motor, "FPG", fpg[i]))
			Error();

		if (result=bh.Get(motor, "SAMPLE", &samplereg[i]))
			Error();

		int lcpgFlag;
		if (result=bh.Get(motor, "LCPG", &lcpgFlag))
				Error();
		lcpg[i] = lcpgFlag ? true : false;

		// From BHControl
		//printf("samplereg[%d] = %d\n", i, samplereg[i]);
		sampleperiod[i] = (16.0 * (((double)samplereg[i]) + 1.0)) / 2000000.0; // 2.0 MHz external clock (units are in seconds)
		//printf("sampleperiod[%d] = %0.6f\n", i, sampleperiod[i]);
	}

	// int  RTSetFlags (const char *motor, bool LCV, int LCVC, bool LCPG, bool LCT, bool LFV, int LFVC, bool LFS, bool LFAP, bool LFDP, int LFDPC, bool LFBP, bool LFAIN, bool LFDPD, bool LFT) 
	if (result=bh.RTSetFlags("2", 1, lcvc[1], lcpg[1], 0, 1, lfvc[1], 1, 0, 0, 0, 0, 0, 0, 0))
		Error();

	if (result=bh.Get("", "FPG", fpg))
		Error();
	if (result=bh.Get("", "LCVC", lcvc))
		Error();
	if (result=bh.Get("", "LFVC", lfvc))
		Error();
#else
	// Torque mode
	//if (result=bh.RTSetFlags("2", 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0))
	if (result=bh.RTSetFlags("2", 0, 1, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0)) // get strain feedback
		Error();
	if (result=bh.Set("2", "FPG", 1))
		Error();
	if (result=bh.Set("2", "FIP", 0))
		Error();
	if (result=bh.Set("2", "FDZ", 0))
		Error();
#endif
}

#define SG_FILTER_SIZE 15

///////////////////////////////////////////////////////////
//  Run RealRime loop, return 1 if interrupted with a key

int RunRealTime()
{
	int lastDesVel = 0;
	double var[4][7];
	double time;
	bool terminate=false;
	int m, i, j;

	int	 SGinit[3];
	double deadband=5; //30;
	double factorK = .7 * 3, factorD = 0; // .1 * 3;
	double valuelast[3];
	bool motion[3] = {false, false, false};
	double deadbandMotion=20;
	double sg[3][SG_FILTER_SIZE];
	double sgfilter[3];
	double val = 0.0;

	// Stop Motor
	if (result=bh.StopMotor("4"))
		Error();

	// Read Strain
	if (result=bh.Get("1", "SG", &SGinit[0]))
		Error();
	if (result=bh.Get("2", "SG", &SGinit[1]))
		Error();
	if (bh.Get("3", "SG", &SGinit[2]))
		Error();

	// initialize sgfilter variables
	for (i = 0; i < 3; i++)
	{
		for (j = 0; j < SG_FILTER_SIZE; j++)
			sg[i][j] = SGinit[i];
		valuelast[i] = SGinit[i];
	}

	if (result=bh.RTStart("12"))
		Error();
	tmr.Start();
	if (result=bh.RTUpdate())
		Error();

	printf("\n");

	int refVel = 0;
	bool onFlag = false;

	while (!terminate && !_kbhit())
	{
		time = tmr.GetElapsedInMilliSecs();

		for (m = 0; m < 4; m++)
		{
			var[m][0] = bh.RTGetPosition( m+'1' );
			var[m][1] = bh.RTGetVelocity( m+'1' );
			var[m][2] = bh.RTGetStrain( m+'1' );
			var[m][3] = bh.RTGetDeltaPos( m+'1' );
			var[m][5] = time;
			//var[m][6] = (double)N;
		}

		// shift sg values
		for (i = 0; i < 3; i++)
		{
			for (j = 14; j >= 1; j--)
				sg[i][j] = sg[i][j-1];
		}

		// put present value in sg[0]
		for (i = 0; i < 3; i++)
			sg[i][0] = var[i][2];

		// filter sg data
		for (i = 0; i < 3; i++)
			sgfilter[i] = (sg[i][0]+sg[i][1]+sg[i][2]+sg[i][3]+sg[i][4]+sg[i][5]+sg[i][6]+sg[i][7]+sg[i][8]+sg[i][9]+sg[i][10]+sg[i][11]+sg[i][12]+sg[i][13]+sg[i][14]) / 15;

		
		// Used to set desired control velocities (disable _kbhit check for while loop if you use this)
		/*if (_kbhit())
		{
			char c = _getch();
			if (c == 'q' || c == 'Q')
				break;
			else if (c == 'o' || c == 'O')
				onFlag = !onFlag;
			else if (c == '-' || c == '_')
				refVel -= 16;
			else if (c == '+' || c == '=')
				refVel += 16;
		}*/



		// Loop for all fingers
		for (i = 0; i < 3; i++)
		{
			// if not moving
			if (!motion[i])
			{
				if ((sgfilter[i] > ((double)SGinit[i]+deadband)) || (sgfilter[i] < ((double)SGinit[i]-deadband)))
					motion[i] = true; // if force is applied, set motion TRUE
				else
					value[i] = 0; // if no force is applied set velocity to zero
			}

			// if moving
			if (motion[i])
			{
				if ( (sgfilter[i] > ((double)SGinit[i]+deadbandMotion)) || (sgfilter[i] < ((double)SGinit[i]-deadbandMotion)))
					// if force is still appplied, calculate new velocity
					value[i] = (int)(factorK * (sgfilter[i]-(double)SGinit[i]) - factorD * (sgfilter[i]-valuelast[i]));
				else // if no force is applied, set velocity to zero and motion FALSE
				{
					value[i] = 0;
					motion[i] = false;
				}
			}

			if (i == 0)
			{
#ifdef USE_VELOCITY_MODE

				// Set control variables
				int desPropGain = 10;

				// Use input from keyboard to set desired control velocity
				//int desVel = refVel;
				//if (!onFlag)
				//	desVel = 0;

				// Use input from finger 1 strain to set desired control velocity
				int desVel = -(value[0] * 10) / 16;

				if (desVel > 127)
					desVel = 127;
				else if (desVel < -127)
					desVel = -127;

				// Set desired control velocity
				bh.RTSetVelocity('2', desVel);

				// Estimate MCn
				int k;
				if (lcpg[1])
				{
					k = desPropGain;
					bh.RTSetGain('2', desPropGain);
				}
				else
				{
					k = fpg[1];
				}

				// Find command and feedback velocity (units are in counts/sample time)

				// Set the Command Velocity
				// Command Velocity Format has a fractional component(IIII IIII IIII FFFF)
				int commandedVelocity = (lastDesVel * lcvc[1]) / 16; // counts per sample period

				// Set the Actual Velocity
				// HCTL-1100 Actual Velocity has no fractional component (IIII IIII IIII IIII)
				int actualVelocity = (((int)bh.RTGetVelocity('2')) * lfvc[1]) * 100; // counts per 10 milliseconds -> multiply by 100 to get counts per second
				actualVelocity = (int)((double)actualVelocity * sampleperiod[1]);

				int Yn = (int)(commandedVelocity - actualVelocity);

				int estMCn = (int)((k / 4.0f) * Yn) + 0x80;
				if (estMCn > 240) // HCTL-1100 internally clips (or saturates) the motor command output
					estMCn = 240;
				else if (estMCn < 15)
					estMCn = 15;
				estMCn -= 128; // subtract the 0x80 offset so that estMCn is signed

				// Used with modified firmware for checking that estimated MCn matched the actual MCn
				//printf("lastDesVel = %3d, estMCn = %3d actualMCn = %3d, vel = %3d  %3d \n", lastDesVel, estMCn, ((int)bh.RTGetStrain('2')) - 128, bh.RTGetVelocity('2') * lfvc[1], actualVelocity);

				printf("   lastDesVel = %3d estimated motor torque = %1.1f (N-cm)     \r", lastDesVel, rtTorque(estMCn) * 100);

				lastDesVel = desVel;

				/*
	K depends on the Boolean LCPG flag.  If LCPG is true than K is equal to the
	loop control proportional gain, otherwise K is equal to the value of the
	FPG property.  Yn is the velocity error equal to control velocity
	multiplied by the loop control velocity coefficient minus the actual velocity.
 */

//int propGain;
/*int lcvc[4];
int fpg[4];

bool lcpgFlag[4];

char estimateMCn(unsigned int propGain, int controlVel, int actualVel)
{
	int mcn = (propGain / 4) * (controlVel * LCVC - actualVel);
	if (mcn > 127)
		mcn = 127;
	else if (mcn < -128)
		mcn = -128;
	return (char)mcn;*/




#else
				val = val * 0.92 + value[0] * 0.08;

				// Used to take data
				//int torque = (((int)(val * 1)) / 50) * 150;

				//bh.RTSetTorque('2', torque); // (int)(val * 0.75));
				//printf("tor = %6d, strain = %6d         \r", torque, (int)sgfilter[1]);
				float desTorque = val / 50;
				setRTTorque(bh, 1, desTorque); // finger 2
				printf("   tor = %1.3f, strain = %6d         \n", desTorque, (int)sgfilter[1]);

#endif
			}
		}

		if (result=bh.RTUpdate())
		{
			printf("\nmain RTUpdate returned error %d\n", result);
			Error();
		}

		// stores last sg value
		for (i = 0; i < 3;i++)
			valuelast[i] = sgfilter[i];

	}

	bh.RTAbort();

	if (_kbhit())
		{ _getch(); return 1; }
	else
		return 0;
}


void PrintBanner()
{
#ifdef LINUX
	system("clear");
#else
	system("cls");
#endif
	printf("\n\t\t---------------------------------------");
	printf("\n\t\t|                                     |");
	printf("\n\t\t|     Barrett Technology, Inc.        |");
	printf("\n\t\t|     BarrettHand Software Example    |");
	printf("\n\t\t|     Force Responce Program v1.0     |");
	printf("\n\t\t|                                     |");
	printf("\n\t\t---------------------------------------");
}


///////////////////////////////////////////////////////////
//  Main function - initialize, execute

int main()
{

	PrintBanner();

	printf("\n\nNOTE: This program requires installation of the Joint-Torque Sensors.");

	Initialize();

	printf("\rThis program monitors the force applied to the fingertip of finger 1.");
	printf("\nFinger 2 will move proportional to the amount of force on finger 1.");


	PrepareRealTime();
	printf("\n\nThe force monitoring demonstration is now running.");
	printf("\nPress any key to end the program.\n");
	RunRealTime();

	bh.StopMotor("GS");
	bh.Reset();

}
