///////////////////////////////////////////////////////////
//                                                       //
//           Automatically Generated C++ Code            //
//           BHand Control Center Version 1.0            //
//                                                       //
//                   Real Time Mode                      //
//                                                       //
//             BarrettHand Demo Program                  //
//      v1.0 990803 dr - Created Program                 //
//      v2.0 001120 ac - Modified Program, see note below
//      v4.5 030919 bz - Made acceleration smooth
//						Tries to re-init 5 times after error
//						Other minor enhancements
//
///////////////////////////////////////////////////////////

// Modified on 001120 by ac to allow user selection of serial port.


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#ifndef LINUX
#include <conio.h>
#endif
#include "BHand.h"
#include "BHandAppHelper.h"

#ifdef LINUX
    #include <sys/time.h> // Needed for ms time.

    struct HiResTimer {
        void Start(void) { gettimeofday(&mTimeStart, NULL); };
        double GetElapsedInMilliSecs(void)
        {
            struct timeval mTimeNow;
            gettimeofday(&mTimeNow, NULL);
            double duration = (double)(mTimeNow.tv_sec - mTimeStart.tv_sec) * 1000 + (double)(mTimeNow.tv_usec - mTimeStart.tv_usec) / 1000;
            return duration;
        }

        struct timeval mTimeStart;
    };

#else
    #include <Windows.h>

    struct HiResTimer {
        void Start(void) { QueryPerformanceCounter(&mTimeStart); };
        double GetElapsedInMilliSecs(void)
        {
            LARGE_INTEGER mTimeNow;
            QueryPerformanceCounter(&mTimeNow);
            LARGE_INTEGER freq;
            QueryPerformanceFrequency(&freq);
            double duration = 1000 * (double)(mTimeNow.QuadPart-mTimeStart.QuadPart)/(double)freq.QuadPart;
            return duration;
        }

        LARGE_INTEGER mTimeStart;
    };

#endif

BHand  bh;           // Handles all hand communication
int    value;        // Hand parameter obtained with Get
int    result;       // Return value (error) of all BHand calls

int    szbuffer;     // Size of data buffers
int*   pdata[4][8];  // Pointers to data buffers

struct HiResTimer tmr;



///////////////////////////////////////////////////////////
// Error Handler - called whenever result!=0
// Feel free to customize this function

void Error(void)
{
	int err = 0;

	printf( "ERROR: %d\n%s\n", result, bh.ErrorMessage(result) ); fflush(stdout);


	//Re-initialize the hand - BZ 19Sep2003
	printf( "\r\t\tInitializing Hand...             " ); fflush(stdout);
	while(err < 5)
	{
		if( result=bh.InitHand("") )
			++err;
		else
			break;
	}
	if(err == 5)
	{
		printf("\n\nCould not initialize hand!  Check cable & power.");
		printf("\n\nPress Ctrl-C to exit..."); fflush(stdout);
		while(1);
	}

}


///////////////////////////////////////////////////////////
//  Initialize hand, set timeouts and baud rate

void Initialize(void)
{
	// Set hardware description before initialization
	int hwIndex = BHandHardware::getBHandHardwareIndex("BH8-262");
	//int hwIndex = BHandHardware::getBHandHardwareIndex("BH8-280");
	if (hwIndex < 0)
	{
		printf("\n\nThe API has not been compiled to include target hand.\n");
		Error();
	}

	bh.setHardwareDesc(hwIndex);

	bool use280Config = (strcmp(bh.getHardwareDesc()->getModelNumber(), "BH8-280") == 0);
	//printf("\nuse280Config = %d\n", use280Config);


	if (result = handInitWithMenu(&bh))
		Error();

/*	int port;

	printf( "\n\n\r\t\tInitializing Software..." );
	printf( "\n\n\r\t\tPlease enter desired serial port> " );
	scanf("%d", &port);
	printf( "\n" );

	if( result=bh.InitSoftware(port,THREAD_PRIORITY_TIME_CRITICAL) )   //allow use of any serial port
		Error();

	if( result=bh.ComSetTimeouts(100,25000,100,5000) )
		Error();*/

	//if( result=bh.Baud(38400) )
	//	Error();






	if( result=bh.Command ( "SFSET MPE 1000" ) )  //AC 010711 spread MPE was 100
		Error();

	printf( "\r\t\tInitializing Hand...             \n" ); fflush(stdout);


	if( result=bh.InitHand("") )
		Error();
}




///////////////////////////////////////////////////////////
//  Set parameters, allocate data buffers, load files

void PrepareRealTime(void)
{
	szbuffer = 2000;

	for( int m=0; m<4; m++ )
		for( int v=0; v<8; v++ )
		{
			pdata[m][v] = new int[szbuffer];
			memset((void*)pdata[m][v], 0, szbuffer*sizeof(int));
		}

	if( result=bh.RTSetFlags( "123", 1, 1, 0, 0, 1, 0, 1, 0, 1 ))
		Error();

}


///////////////////////////////////////////////////////////
//  Run RealRime loop, return 1 if interrupted with a key

int RunRealTime1(void)
{
	double var[4][7];
	int N=0;
	unsigned int  time;
	bool terminate=false;
	int pos, m;

// AC 010711 Set control parameters so RT velocity control will work w/ firmware v4.21 (revision 4.21)
/*	if( result=bh.Command ( "123FSET SAMPLE 16" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

 Don't mess with the spread - BZ 19Sep2003
	if( result=bh.Command ( "SFSET FDZ 248" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "SFSET FIP 0" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "SFSET FPG 100" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "GFSET FDZ 221" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "GFSET FIP 66" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "GFSET FPG 200" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }
*/

	if( result=bh.Command ( "123FSET SAMPLE 63" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "GFSET FPG 42" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	bh.RTStart( "123" );
	tmr.Start();
	bh.RTUpdate();

	while( !terminate && !_kbhit() )
	{
		time = (unsigned int)tmr.GetElapsedInMilliSecs();

		pos = N%szbuffer;
		for( m=0; m<4; m++ )
		{
			pdata[m][0][pos] = bh.RTGetPosition( m+'1' );
			pdata[m][1][pos] = bh.RTGetVelocity( m+'1' );
			pdata[m][2][pos] = bh.RTGetStrain( m+'1' );
			pdata[m][3][pos] = bh.RTGetDeltaPos( m+'1' );
			pdata[m][5][pos] = (int)time;
		}
		pos = (N+1)%szbuffer;

		for( int m=0; m<4; m++ )
		{
			var[m][0] = bh.RTGetPosition( m+'1' );
			var[m][1] = bh.RTGetVelocity( m+'1' );
			var[m][2] = bh.RTGetStrain( m+'1' );
			var[m][3] = bh.RTGetDeltaPos( m+'1' );
			var[m][4] = pdata[m][4][N%szbuffer];
			var[m][5] = (double)time;
			var[m][6] = (double)N;
		}

		value = (int)((200.00) * (sin( (2.00) * ((3.14) * ((var[0][5]) / (2000.00))) )));
		bh.RTSetVelocity( '1', value );
		pdata[0][6][pos] = value;

		value = (int)((200.00) * (sin( ((2.00) * ((3.14) * ((var[1][5]) / (2000.00)))) - ((2.00) * ((3.14) / (3.00))) )));
		bh.RTSetVelocity( '2', value );
		pdata[1][6][pos] = value;

		value = (int)((200.00) * (sin( ((2.00) * ((3.14) * ((var[2][5]) / (2000.00)))) - ((3.14) / (3.00)) )));
		bh.RTSetVelocity( '3', value );
		pdata[2][6][pos] = value;

		terminate = (0<(int)((var[0][5]) > (8000.00)));

		N++;
		bh.RTUpdate();
	}

	bh.RTAbort();

	if( result=bh.Command ( "FDEF" ) ) //AC 010711 Restore control parameters to defaults (version 4.21)
		Error();

	if( _kbhit() )
		{ _getch(); return 1; }
	else
		return 0;
}

int RunRealTime2(void)
{
	double var[4][7];
	int N=0;
	unsigned int time;
	bool terminate=false;
	int pos, m;

// AC 010711 Set control parameters so RT velocity control will work w/ firmware v4.21 (revision 4.21)
/*	if( result=bh.Command ( "123FSET SAMPLE 16" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

 Don't mess with the spread - BZ 19Sep2003
	if( result=bh.Command ( "SFSET FDZ 248" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "SFSET FIP 0" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "SFSET FPG 100" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "GFSET FDZ 221" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "GFSET FIP 66" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "GFSET FPG 200" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }
*/

	if( result=bh.Command ( "GFSET SAMPLE 63" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "GFSET FPG 42" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	bh.RTStart( "123" );
	tmr.Start();
	bh.RTUpdate();

	while( !terminate && !_kbhit() )
	{
		time = (unsigned int)tmr.GetElapsedInMilliSecs();

		pos = N%szbuffer;
		for( m=0; m<4; m++ )
		{
			pdata[m][0][pos] = bh.RTGetPosition( m+'1' );
			pdata[m][1][pos] = bh.RTGetVelocity( m+'1' );
			pdata[m][2][pos] = bh.RTGetStrain( m+'1' );
			pdata[m][3][pos] = bh.RTGetDeltaPos( m+'1' );
			pdata[m][5][pos] = (int)time;
		}
		pos = (N+1)%szbuffer;

		for( int m=0; m<4; m++ )
		{
			var[m][0] = bh.RTGetPosition( m+'1' );
			var[m][1] = bh.RTGetVelocity( m+'1' );
			var[m][2] = bh.RTGetStrain( m+'1' );
			var[m][3] = bh.RTGetDeltaPos( m+'1' );
			var[m][4] = pdata[m][4][N%szbuffer];
			var[m][5] = (double)time;
			var[m][6] = (double)N;
		}

		value = (var[0][0] > 16000) ? 0 : (int)(215.00);
		bh.RTSetVelocity( '1', value );
		pdata[0][6][pos] = value;

		value = (int)(((var[2][0]) > (5000.00)) ? (250.00) : (0.00));
		bh.RTSetVelocity( '2', value );
		pdata[1][6][pos] = value;

		value = (int)(((var[0][0]) > (5000.00)) ? (235.00) : (0.00));
		bh.RTSetVelocity( '3', value );
		pdata[2][6][pos] = value;

		terminate = (0<(int)((var[0][5]) > (2000.00)));

		N++;
		bh.RTUpdate();
	}

	bh.RTAbort();

	if( result=bh.Command ( "FDEF" ) ) //AC 010711 Restore control parameters to defaults (version 4.21)
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }
	else
		return 0;
}

int Execute1(void)
{
	int i;

	/* Set up FPG, SAMPLE, ACCEL, and Max Velocities for fingers 1-3 - BZ 19Sep2003 */
	if( result=bh.Command ( "123FSET FPG 10" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123FSET MCV 200" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123FSET MOV 200" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123FSET SAMPLE 63" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123FSET ACCEL 4" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	/* Set up FPG, SAMPLE, ACCEL, and Max Velocities for spread - BZ 19Sep2003 */
	if( result=bh.Command ( "4FSET FPG 8" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "4FSET MCV 100" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "4FSET MOV 100" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "4FSET SAMPLE 200" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "4FSET ACCEL 8" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	/* Perform supervisory demonstration */
	if( result=bh.Command ( "GO" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "SO" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123C" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123O" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "SC" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123C" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123O" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "1M 5000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "3M 10000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "2M 15000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123SO" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

//AC 010711 Removed following command because it was casing fingers to hit eachother (version 4.21)

	//if( result=bh.Command ( "123SC" ) )
	//	Error();
	//if( _kbhit() )
	//	{ _getch(); return 1; }

	if( result=bh.Command ( "SC" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123C" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "1O" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "3O" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "2O" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "SM 1570" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "12M 11500" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "3C" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123O" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }


	/* Do the "Back and Forth" */
	if( result=bh.Command ( "3FSET DP 11500" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	for(i = 0; i < 3; i++)
	{
		if( result=bh.Command ( "1FSET DP 11500" ) )
			Error();
		if( _kbhit() )
			{ _getch(); return 1; }

		if( result=bh.Command ( "2FSET DP 6000" ) )
			Error();
		if( _kbhit() )
			{ _getch(); return 1; }

		if( result=bh.Command ( "GM" ) )
			Error();
		if( _kbhit() )
			{ _getch(); return 1; }

		if( result=bh.Command ( "1FSET DP 6000" ) )
			Error();
		if( _kbhit() )
			{ _getch(); return 1; }

		if( result=bh.Command ( "2FSET DP 11500" ) )
			Error();

		if( result=bh.Command ( "GM" ) )
			Error();
		if( _kbhit() )
			{ _getch(); return 1; }
	}

	if( result=bh.Command ( "123M 11500" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	/* End of Back and Forth */

	if( result=bh.Command ( "1FSET MCV 200" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "2FSET MCV 200" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "3FSET MCV 200" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123M 5000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "SC" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123C" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123M 2000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	/* Do finger jumble */

	if( result=bh.Command ( "SM 1000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "1C" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "2M 14000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "3M 11500" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "O" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	/* End finger Jumble */

	if( result=bh.Command ( "1FSET MCV 65" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "2FSET MCV 65" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "3FSET MCV 85" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "SFSET MCV 50" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123SC" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "1FSET MCV 75" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "2FSET MCV 75" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "3FSET MCV 75" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "1M 13000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "3M 13000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "2M 13000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "1M 17000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "3M 17000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "2M 17000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "1M 9000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "3M 9000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "2M 9000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "1234O" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	/* Do Spread Inc Close with finger movement */

	if( result=bh.Command ( "SFSET MCV 100" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123FSET MCV 200" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "123FSET DP 4000" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "4FSET DS 630" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	for(i = 0; i < 5; i++)
	{
		if( result=bh.Command ( "123M 11000" ) )
			Error();
		if( _kbhit() )
			{ _getch(); return 1; }

		if( result=bh.Command ( "123M" ) )
			Error();
		if( _kbhit() )
			{ _getch(); return 1; }

		if( result=bh.Command ( "SIC" ) )
			Error();
		if( _kbhit() )
			{ _getch(); return 1; }
	}


	if( result=bh.Command ( "C" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	if( result=bh.Command ( "GO" ) )
		Error();
	if( _kbhit() )
		{ _getch(); return 1; }

	return 0;
}


///////////////////////////////////////////////////////////
//  Save all buffers into a text file

/*void SaveData(char* name)
{
	FILE* fp = fopen(name, "wt");
	if( !fp )
		return;

	for( int r=0; r<szbuffer; r++ )
	{
		for( int v=0; v<8; v++ )
		{
			for( int m=0; m<4; m++ )
				fprintf(fp, "%d ", pdata[m][v][r]);
			fprintf(fp, "  ");
		}
		fprintf(fp, "\n");
	}

	fclose(fp);
}*/


void PrintBanner()
{
#ifdef LINUX
	system("clear");
#else
	system("cls");
#endif
	printf("\n\t\t---------------------------------------");
	printf("\n\t\t|                                     |");
	printf("\n\t\t|     Barrett Technology, Inc.        |");
	printf("\n\t\t|     BarrettHand Software Example    |");
	printf("\n\t\t|     BarrettHand Demo Program v4.5   |");
	printf("\n\t\t|                                     |");
	printf("\n\t\t---------------------------------------"); fflush(stdout);


}


///////////////////////////////////////////////////////////
//  Main function - initialize, execute

int main(void)
{
	int i=0;


	PrintBanner();

	Initialize();

	printf("\n\n\r\t\tThis program will run a demonstration until a key is pressed.\n"); fflush(stdout);
	PrepareRealTime();

	while(1)
	{
		if(Execute1())
		{
			bh.Command("T");
			bh.Reset();
			exit(0);
		}
		if(RunRealTime1())
		{
			bh.Command("T");
			bh.Reset();
			exit(0);
		}

		if(Execute1())
		{
			bh.Command("T");
			bh.Reset();
			exit(0);
		}

		for(i=0;i<3;i++)
		{
			bh.Command("GO");
			if(RunRealTime2())
			{
				bh.Command("T");
				bh.Reset();
				exit(0);
			}
		}

	}

	//SaveData( (char*)"ConferenceWiggle" );
}
