///////////////////////////////////////////////////////////
//                                                       //
//           Automatically Generated C++ Code            //
//           BHand Control Center Version 1.0            //
//                                                       //
//                                                       //
//                                                       //
///////////////////////////////////////////////////////////


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <deque>
#include "BHand.h"
#include "BHandAppHelper.h" // for connection menu
#include <sys/time.h>// Needed for ms time.
#include <time.h>// Needed for ms time.
using namespace std;

BHand  bh;           // Handles all hand communication

char   buf[100];     // Buffer for reading back hand parameters
int    value;        // Some commands use an int* instead of
                     // a buffer to relay information

int    result;       // Return value (error) of all BHand calls


///////////////////////////////////////////////////////////
// Error Handler - called whenever result != 0           //
// Feel free to customize this function                  //
///////////////////////////////////////////////////////////

void Error()
{
	printf("ERROR: %d\n%s\n", result, bh.ErrorMessage(result));
	exit(0);
}


// get the difference in time between two timevals (difference in ms)
double diffclock(timeval* currentTime, timeval* startTime)
{
	if (((currentTime->tv_usec)/1000. - (startTime->tv_usec)/1000.) < 0.0) {
		return ((1000.0 + (currentTime->tv_usec)/1000.) - (startTime->tv_usec)/1000.);
	} else {
		return ((currentTime->tv_usec)/1000. - (startTime->tv_usec)/1000.);
	}
}


///////////////////////////////////////////////////////////
//  Initialize hand, set timeouts and baud rate          //
///////////////////////////////////////////////////////////
void Initialize()
{
	// Set hardware description before initialization
	//int hwIndex = BHandHardware::getBHandHardwareIndex("BH8-262");
	int hwIndex = BHandHardware::getBHandHardwareIndex("BH8-280");
	if (hwIndex < 0)
	{
		printf("\n\nThe API has not been compiled to include target hand.\n");
			Error();
	}
	bh.setHardwareDesc(hwIndex);
	bool use280Config = (strcmp(bh.getHardwareDesc()->getModelNumber(), "BH8-280") == 0);
	//printf("\nuse280Config = %d\n", use280Config);
	if (result = handInitWithMenu(&bh))
		Error();
	//if (result = bh.InitSoftware(com_port, THREAD_PRIORITY_TIME_CRITICAL))
	//	Error();

	printf("Initialization...");
	if (result = bh.InitHand(""))
		Error();
	else
		printf(" Done\n");
}


///////////////////////////////////////////////////////////
//  Loads a file to be run in realtime mode              //
///////////////////////////////////////////////////////////
void LoadFile(char *toLoad, deque<float*> *file_data)
{

	FILE* fp = fopen(toLoad, "rt");
	if (!fp)
		return;

	int v1,v2,v3,v4;

	while (fscanf(fp, "%d %d %d %d", &v1, &v2, &v3, &v4) != EOF)
{
		float* values = new float[4];
		values[0] = v1;
		values[1] = v2;
		values[2] = v3;
		values[3] = v4;


		file_data->push_back(values);

	}

	fclose(fp);
}


///////////////////////////////////////////////////////////
//  Execute commands, return 1 if interrupted with a key //
///////////////////////////////////////////////////////////

int Before()
{
	printf("Before: Press Any Key to Abort...");


	if (result = bh.Command("sc"))
		Error();
	if (UnbufferedGetChar() != EOF)
		return 1;

	if (result = bh.Command("go"))
		Error();
	if (UnbufferedGetChar() != EOF)
		return 1;

	if (result = bh.Command("GFSET DP 100000"))
		Error();
	if (UnbufferedGetChar() != EOF)
		return 1;

	if (result = bh.Command("123m"))
		Error();
	if (UnbufferedGetChar() != EOF)
		return 1;

	return 0;
}

///////////////////////////////////////////////////////////
//  Set parameters, prepare data buffers                 //
///////////////////////////////////////////////////////////
void PrepareRealTime()
{
	if (result = bh.RTSetFlags( "123", 1, 3, 0, 0, 1, 1, 0, 1, 0, 1,0,0,0,0 ))
		Error();
	sprintf(buf, "%s", "");
}

///////////////////////////////////////////////////////////
// Run RealRime loop, return 1 if interrupted with a key //
///////////////////////////////////////////////////////////
int RunRealTime()
{
	double var[4][7];
	int N=0;
	unsigned int time = 0, tmstart;
	bool terminate = false;
	int pos, m;

	struct timezone m_tz;
	struct timeval startTime;
	struct timeval currentTime;

	gettimeofday(&startTime, &m_tz);
	deque<float*> file_data;
	LoadFile(buf, &file_data);
	bh.RTStart( "123" );
	bh.RTUpdate();
	printf("Press Any Key to Abort...");
	while(!terminate && UnbufferedGetChar() == EOF)
	{

		gettimeofday(&currentTime, &m_tz);
		time += (unsigned int)diffclock(&currentTime,&startTime);
		gettimeofday(&startTime, &m_tz);

		for (int m = 0; m < 4; m++)
		{
			var[m][0] = bh.RTGetDeltaPos(m + '1');

			if (!file_data.empty())
			{
			var[m][1] = ((file_data[N % file_data.size()])[m]);   /* file data */
		}

		var[m][2] = (double)N;
		var[m][3] = bh.RTGetPosition(m + '1');
		var[m][4] = bh.RTGetStrain(m + '1');
		var[m][5] = (double)time;
		var[m][6] = bh.RTGetVelocity(m + '1');
	}

		value = (int)(65*sin( ( var[0][5] / 3000)*2*3.14));
		bh.RTSetVelocity('1', value);
		value = (int)(65*sin( ( var[0][5]/3000)*2*3.14 - 3.14));
		bh.RTSetVelocity('2', value);
		value = (int)(65*sin( ( var[0][5]/3000)*2*3.14 - 3.14/2));
		bh.RTSetVelocity('3', value);
		terminate =  var[0][5] > 10000;
		N++;
		bh.RTUpdate();
	}
	file_data.clear();
	bh.RTAbort();

		if (UnbufferedGetChar() != EOF)
			return 1;
		else
			return 0;
}
///////////////////////////////////////////////////////////
//  Execute commands, return 1 if interrupted with a key //
///////////////////////////////////////////////////////////

int After()
{
	printf("After: Press Any Key to Abort...");


	if (result = bh.Command("go"))
		Error();
	if (UnbufferedGetChar() != EOF)
		return 1;

	if (result = bh.Command("t"))
		Error();
	if (UnbufferedGetChar() != EOF)
		return 1;
    DELAY(1);

	return 0;
}

int main()
{
	// Turn off output buffering for printf()
	setvbuf(stdout,0,_IONBF,0);

	// Turn on unbuffered input
	UnbufferedInputStart();

	printf("\n\n\r\t\tInitializing Software...");
	Initialize();

	printf("Before - ");
	if (Before())
		return 1;
	printf(" Done\n");

	PrepareRealTime();

	printf("RealTime Loop - ");
	if (RunRealTime())
		return 1;
	printf(" Done\n");

	printf("After - ");
	if (After())
		return 1;
	printf(" Done\n");
	// Turn off unbuffered input
	UnbufferedInputEnd();
	return 0;
}
