///////////////////////////////////////////////////////////
//                                                       //
// Example of Instantiating 2 BHand Objects              //
//                                                       //
///////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
//#include <conio.h>
#include "BHand.h"

// Instantiate two objects of type BHand, one per serial port
BHand bh1;  // Handles all hand 1 communication
BHand bh2;  // Handles all hand 2 communication
int result; // Return value (error) of all BHand calls

const int BHAND1_COMPORT = 1;
const int BHAND2_COMPORT = 2;

///////////////////////////////////////////////////////////
// Error Handler for hand 1 - called whenever result!=0
// Feel free to customize this function
///////////////////////////////////////////////////////////

void Error1()
{
	printf( "ERROR: %d\n%s\n", result, bh1.ErrorMessage(result) );
	exit(0);
}

///////////////////////////////////////////////////////////
// Error Handler for hand 2 - called whenever result!=0
// Feel free to customize this function
///////////////////////////////////////////////////////////

void Error2()
{
	printf( "ERROR: %d\n%s\n", result, bh2.ErrorMessage(result) );
	exit(0);
}


///////////////////////////////////////////////////////////
// Initialize hand 1, set timeouts and baud rate
///////////////////////////////////////////////////////////

void Initialize1()
{
	// Set hardware description before initialization
	int hwIndex = BHandHardware::getBHandHardwareIndex("BH8-262");
	if (hwIndex < 0)
	{
		printf("\n\nThe API has not been compiled to include target hand.\n");
		exit(0);
	}
	bh1.setHardwareDesc(hwIndex);
	if (result=bh1.Init(BHAND1_COMPORT, THREAD_PRIORITY_TIME_CRITICAL, BH_SERIAL_COMMUNICATION))
		Error1();
	//if (result=bh1.ComSetTimeouts(0,100,15000,100,5000))
	//	Error1();
	//if (result=bh1.Baud(9600))
	//	Error1();
	if (result=bh1.InitHand("2"))
		Error1();
}

///////////////////////////////////////////////////////////
// Initialize hand 2, set timeouts and baud rate
///////////////////////////////////////////////////////////

void Initialize2()
{
	// Set hardware description before initialization
	int hwIndex = BHandHardware::getBHandHardwareIndex("BH8-262");
	if (hwIndex < 0)
	{
		printf("\n\nThe API has not been compiled to include target hand.\n");
		exit(0);
	}
	bh2.setHardwareDesc(hwIndex);
	if (result=bh2.Init(BHAND2_COMPORT, THREAD_PRIORITY_TIME_CRITICAL, BH_SERIAL_COMMUNICATION))
		Error2();
	//if (result=bh2.ComSetTimeouts(0,100,15000,100,5000))
	//	Error2();
	//if (result=bh2.Baud(9600))
	//	Error2();
	if (result=bh2.InitHand("2"))
		Error2();
}

///////////////////////////////////////////////////////////
// Execute commands, return 1 if interrupted with a key
///////////////////////////////////////////////////////////

int Execute()
{
	printf( "Press Any Key to Abort..." );

	// Execute commands on hand 1
	if (result=bh1.Command("2hi"))
		Error1();
	
	if (_kbhit())
	{
		_getch();
		return 1;
	}
	
	if (result=bh1.Command("2c"))
		Error1();
	
	if (_kbhit())
	{
		_getch();
		return 1;
	}
	
	if (result=bh1.Command("2o"))
		Error1();
	
	if (_kbhit())
	{
		_getch();
		return 1;
	}

	// Execute commands on hand 2
	if (result=bh2.Command("2hi"))
		Error2();
	
	if (_kbhit())
	{
		_getch();
		return 1;
	}
	
	if (result=bh2.Command("2c"))
		Error2();
	
	if (_kbhit())
	{
		_getch();
		return 1;
	}

	if (result=bh2.Command("2o"))
		Error2();
	if (_kbhit())
	{
		_getch();
		return 1;
	}
	return 0;
}


///////////////////////////////////////////////////////////
// Main function - initialize, execute
///////////////////////////////////////////////////////////

int main()
{
	// Turn off output buffering for printf()
	setvbuf(stdout, 0, _IONBF, 0);

	// Turn on unbuffered input
	UnbufferedInputStart();

	printf( "Initialization..." );
	Initialize1(); // Initialize hand 1
	Initialize2(); // Initialize hand 2
	printf( " Done\n" );
	printf( "Executing - " );
	Execute(); // Execute commands on both hands
	printf( " Done\n" );

	// Turn off unbuffered input
	UnbufferedInputEnd();
	return 0;
}

